Type: Package
Package: EpiCompare
Title: Comparison, Benchmarking & QC of Epigenomic Datasets
Version: 1.14.0
Authors@R: c(
    person(given = "Sera", family = "Choi", 
        email = "serachoi1230@gmail.com", 
        role = "aut",
        comment = c(ORCID = "0000-0002-5077-1984")),
    person(given="Brian", family="Schilder", 
        email = "brian_schilder@alumni.brown.edu", 
        role = "aut",
        comment = c(ORCID = "0000-0001-5949-2191")),
     person(given="Leyla", family="Abbasova", 
        email = "profernitsky@gmail.com", 
        role = "aut"), 
    person(given="Alan", family="Murphy", 
        email = "alanmurph94@hotmail.com", 
        role = "aut",
        comment = c(ORCID = "0000-0002-2487-8753")),
    person(given="Nathan", family="Skene", 
        email = "nathan.skene@gmail.com", 
        role = "aut",
        comment = c(ORCID = "0000-0002-6807-3180")),
    person(given="Thomas", family="Roberts",
        email = "tomroberts.work15@gmail.com",
        role = "ctb"),
    person(given="Hiranyamaya", family="Dash", 
        email = "hdash.work@gmail.com",
        role = "cre",
        comment = c(ORCID = "0009-0005-5514-505X"))
  )
Description: EpiCompare is used to compare and analyse epigenetic datasets 
  for quality control and benchmarking purposes. 
  The package outputs an HTML report consisting of three sections: 
  (1. General metrics) Metrics on peaks (percentage of blacklisted and 
  non-standard peaks, and peak widths) and fragments (duplication rate) 
  of samples, 
  (2. Peak overlap) Percentage and statistical significance of 
  overlapping and non-overlapping peaks. Also includes upset plot and 
  (3. Functional annotation) functional annotation 
  (ChromHMM, ChIPseeker and enrichment analysis) of peaks. 
  Also includes peak enrichment around TSS.
License: GPL-3
URL: https://github.com/neurogenomics/EpiCompare
BugReports: https://github.com/neurogenomics/EpiCompare/issues
Depends: R (>= 4.2.0)
Imports: AnnotationHub, ChIPseeker, data.table, genomation,
        GenomicRanges, IRanges (>= 2.41.3), Seqinfo (>= 0.99.2),
        GenomeInfoDb (>= 1.45.7), ggplot2 (>= 3.5.0), htmltools,
        methods, plotly, reshape2, rmarkdown, rtracklayer, stats,
        stringr, utils, BiocGenerics, downloadthis, parallel
Suggests: rworkflows, BiocFileCache, BiocParallel, BiocStyle,
        clusterProfiler, GenomicAlignments, grDevices, knitr,
        org.Hs.eg.db, testthat (>= 3.0.0), tidyr,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Mmusculus.UCSC.mm9.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene,
        BSgenome.Hsapiens.UCSC.hg19, BSgenome.Hsapiens.UCSC.hg38,
        BSgenome.Mmusculus.UCSC.mm9, BSgenome.Mmusculus.UCSC.mm10,
        ComplexUpset, plyranges, scales, Matrix, consensusSeekeR,
        heatmaply, viridis
VignetteBuilder: knitr
biocViews: Epigenetics, Genetics, QualityControl, ChIPSeq,
        MultipleComparison, FunctionalGenomics, ATACSeq, DNaseSeq
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/EpiCompare
git_branch: RELEASE_3_22
git_last_commit: c5d211a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:49:48 UTC; biocbuild
Author: Sera Choi [aut] (ORCID: <https://orcid.org/0000-0002-5077-1984>),
  Brian Schilder [aut] (ORCID: <https://orcid.org/0000-0001-5949-2191>),
  Leyla Abbasova [aut],
  Alan Murphy [aut] (ORCID: <https://orcid.org/0000-0002-2487-8753>),
  Nathan Skene [aut] (ORCID: <https://orcid.org/0000-0002-6807-3180>),
  Thomas Roberts [ctb],
  Hiranyamaya Dash [cre] (ORCID: <https://orcid.org/0009-0005-5514-505X>)
Maintainer: Hiranyamaya Dash <hdash.work@gmail.com>

# Generated by roxygen2: do not edit by hand

export(EpiCompare)
export(bpplapply)
export(check_workers)
export(compute_consensus_peaks)
export(compute_corr)
export(download_button)
export(fragment_info)
export(gather_files)
export(group_files)
export(liftover_grlist)
export(overlap_heatmap)
export(overlap_percent)
export(overlap_stat_plot)
export(overlap_upset_plot)
export(peak_info)
export(plot_ChIPseeker_annotation)
export(plot_chromHMM)
export(plot_corr)
export(plot_enrichment)
export(plot_precision_recall)
export(precision_recall)
export(predict_precision_recall)
export(rebin_peaks)
export(report_command)
export(report_header)
export(tidy_chromosomes)
export(tidy_peakfile)
export(translate_genome)
export(tss_plot)
export(width_boxplot)
export(write_example_peaks)
import(GenomicRanges)
import(ggplot2)
importFrom(AnnotationHub,AnnotationHub)
importFrom(BiocGenerics,"%in%")
importFrom(BiocGenerics,`%in%`)
importFrom(ChIPseeker,annotatePeak)
importFrom(ChIPseeker,enrichPeakOverlap)
importFrom(ChIPseeker,getPromoters)
importFrom(ChIPseeker,getTagMatrix)
importFrom(ChIPseeker,plotAvgProf)
importFrom(ChIPseeker,readPeakFile)
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomeInfoDb,mapGenomeBuilds)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomeInfoDb,standardChromosomes)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,binnedAverage)
importFrom(GenomicRanges,coverage)
importFrom(GenomicRanges,elementMetadata)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,seqinfo)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,tileGenome)
importFrom(IRanges,from)
importFrom(IRanges,reduce)
importFrom(IRanges,slice)
importFrom(IRanges,to)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqlevelsInUse)
importFrom(Seqinfo,seqnames)
importFrom(Seqinfo,sortSeqlevels)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)
importFrom(downloadthis,download_file)
importFrom(downloadthis,download_this)
importFrom(genomation,heatTargetAnnotation)
importFrom(genomation,readBed)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(htmltools,save_html)
importFrom(htmltools,tagList)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,show)
importFrom(parallel,detectCores)
importFrom(plotly,as_widget)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(reshape2,melt)
importFrom(rmarkdown,render)
importFrom(rtracklayer,import)
importFrom(rtracklayer,liftOver)
importFrom(stats,cor)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stringr,str_split)
importFrom(stringr,str_wrap)
importFrom(utils,browseURL)
importFrom(utils,data)
importFrom(utils,packageVersion)
importFrom(utils,tail)
importMethodsFrom(ChIPseeker,plotAnnoBar)
importMethodsFrom(GenomicRanges,width)
importMethodsFrom(IRanges,findOverlaps)
importMethodsFrom(IRanges,subsetByOverlaps)
importMethodsFrom(genomation,annotateWithFeatures)
importMethodsFrom(rtracklayer,liftOver)

⚖<code>EpiCompare</code>⚖<br>QC and Benchmarking of Epigenomic Datasets
================
<img src='https://github.com/neurogenomics/EpiCompare/raw/master/inst/hex/hex.png' title='Hex sticker for EpiCompare' height='300'><br>
[![](https://img.shields.io/badge/release%20version-1.13.1-green.svg)](https://www.bioconductor.org/packages/EpiCompare)
[![](https://img.shields.io/badge/download-152/month-green.svg)](https://bioconductor.org/packages/stats/bioc/EpiCompare)
[![](https://img.shields.io/badge/download-2874/total-green.svg)](https://bioconductor.org/packages/stats/bioc/EpiCompare)
[![download](http://www.bioconductor.org/shields/downloads/release/EpiCompare.svg)](https://bioconductor.org/packages/stats/bioc/EpiCompare)
[![License:
GPL-3](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://cran.r-project.org/web/licenses/GPL-3)
[![](https://img.shields.io/badge/doi-https://doi.org/10.1101/2022.07.22.501149-blue.svg)](https://doi.org/https://doi.org/10.1101/2022.07.22.501149)
<br>
[![](https://img.shields.io/badge/devel%20version-1.13.1-black.svg)](https://github.com/neurogenomics/EpiCompare)
[![](https://img.shields.io/github/languages/code-size/neurogenomics/EpiCompare.svg)](https://github.com/neurogenomics/EpiCompare)
[![](https://img.shields.io/github/last-commit/neurogenomics/EpiCompare.svg)](https://github.com/neurogenomics/EpiCompare/commits/master)
<br> [![R build
status](https://github.com/neurogenomics/EpiCompare/workflows/rworkflows/badge.svg)](https://github.com/neurogenomics/EpiCompare/actions)
[![](https://codecov.io/gh/neurogenomics/EpiCompare/branch/master/graph/badge.svg)](https://app.codecov.io/gh/neurogenomics/EpiCompare)
<br>
<a href='https://app.codecov.io/gh/neurogenomics/EpiCompare/tree/master' target='_blank'><img src='https://codecov.io/gh/neurogenomics/EpiCompare/branch/master/graphs/icicle.svg' title='Codecov icicle graph' width='200' height='50' style='vertical-align: top;'></a>  
<h4>  
Authors: <i>Sera Choi, Brian Schilder, Leyla Abbasova, Alan Murphy,
Nathan Skene, Thomas Roberts, Hiranyamaya Dash</i>  
</h4>
<h5>  
<i>Updated</i>: Sep-23-2025  
</h5>

# Introduction

`EpiCompare` is an R package for comparing multiple epigenomic datasets
for quality control and benchmarking purposes. The function outputs a
report in HTML format consisting of three sections:

1.  **General Metrics**: Metrics on peaks (percentage of blacklisted and
    non-standard peaks, and peak widths) and fragments (duplication
    rate) of samples.
2.  **Peak Overlap**: Frequency, percentage, statistical significance of
    overlapping and non-overlapping peaks. This also includes Upset,
    precision-recall and correlation plots.
3.  **Functional Annotation**: Functional annotation (ChromHMM,
    ChIPseeker and enrichment analysis) of peaks. Also includes peak
    enrichment around Transcription Start Site.

*Note*: Peaks located in blacklisted regions and non-standard
chromosomes are removed from the files prior to analysis.

# Installation

## Standard

To install `EpiCompare` use:

``` r
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")
BiocManager::install("EpiCompare") 
```

## All dependencies

<details>

<summary>

👈 <strong>Details</strong>
</summary>

Installing all *Imports* and *Suggests* will allow you to use the full
functionality of `EpiCompare` right away, without having to stop and
install extra dependencies later on.

To install these packages as well, use:

``` r
BiocManager::install("EpiCompare", dependencies=TRUE) 
```

Note that this will increase installation time, but it means that you
won’t have to worry about installing any R packages when using functions
with certain suggested dependencies

</details>

## Development

<details>

<summary>

👈 <strong>Details</strong>
</summary>

To install the development version of `EpiCompare`, use:

``` r
if (!require("remotes")) install.packages("remotes")
remotes::install_github("neurogenomics/EpiCompare")
```

</details>

## Citation

If you use `EpiCompare`, please cite:

<!-- Modify this by editing the file: inst/CITATION  -->

> EpiCompare: R package for the comparison and quality control of
> epigenomic peak files (2022) Sera Choi, Brian M. Schilder, Leyla
> Abbasova, Alan E. Murphy, Nathan G. Skene, bioRxiv, 2022.07.22.501149;
> doi: <https://doi.org/10.1101/2022.07.22.501149>

# Documentation

## [EpiCompare website](https://neurogenomics.github.io/EpiCompare)

## [Docker/Singularity container](https://neurogenomics.github.io/EpiCompare/articles/docker)

## [Bioconductor page](https://doi.org/doi:10.18129/B9.bioc.EpiCompare)

### :warning: Note on documentation versioning

The documentation in this README and the [GitHub Pages
website](https://neurogenomics.github.io/EpiCompare/) pertains to the
*development* version of `EpiCompare`. Older versions of `EpiCompare`
may have slightly different documentation (e.g. available functions,
parameters). For documentation in older versions of `EpiCompare`, please
see the **Documentation** section of the relevant version on
[Bioconductor](https://doi.org/doi:10.18129/B9.bioc.EpiCompare)

# Usage

Load package and example datasets.

``` r
library(EpiCompare)
data("encode_H3K27ac") # example peakfile
data("CnT_H3K27ac") # example peakfile
data("CnR_H3K27ac") # example peakfile
data("CnT_H3K27ac_picard") # example Picard summary output
data("CnR_H3K27ac_picard") # example Picard summary output
```

Prepare input files:

``` r
# create named list of peakfiles 
peakfiles <- list("CnT"=CnT_H3K27ac, 
                  "CnR"=CnR_H3K27ac) 
# set ref file and name 
reference <- list("ENCODE_H3K27ac" = encode_H3K27ac) 
# create named list of Picard summary
picard_files <- list("CnT"=CnT_H3K27ac_picard, 
                     "CnR"=CnR_H3K27ac_picard) 
```

<details>

<summary>

<strong>👈 Tips on importing user-supplied files</strong>
</summary>

`EpiCompare::gather_files` is helpful for identifying and importing peak
or picard files.

``` r
# To import BED files as GRanges object
peakfiles <- EpiCompare::gather_files(dir = "path/to/peaks/", 
                                      type = "peaks.stringent")
# EpiCompare alternatively accepts paths (to BED files) as input 
peakfiles <- list(sample1="/path/to/peaks/file1_peaks.stringent.bed", 
                  sample2="/path/to/peaks/file2_peaks.stringent.bed")
# To import Picard summary output txt file as data frame
picard_files <- EpiCompare::gather_files(dir = "path/to/peaks", 
                                         type = "picard")
```

</details>

Run `EpiCompare()`:

``` r
EpiCompare::EpiCompare(peakfiles = peakfiles,
                       genome_build = list(peakfiles="hg19",
                                           reference="hg38"),
                       genome_build_output = "hg19", 
                       picard_files = picard_files,
                       reference = reference,
                       run_all = TRUE
                       output_dir = tempdir())
```

#### Required Inputs

These input parameters must be provided:

<details>

<summary>

👈 <strong>Details</strong>
</summary>

- `peakfiles` : Peakfiles you want to analyse. EpiCompare accepts
  peakfiles as GRanges object and/or as paths to BED files. Files must
  be listed and named using `list()`. E.g.
  `list("name1"=peakfile1, "name2"=peakfile2)`.
- `genome_build` : A named list indicating the human genome build used
  to generate each of the following inputs:
  - `peakfiles` : Genome build for the `peakfiles` input. Assumes genome
    build is the same for each element in the `peakfiles` list.
  - `reference` : Genome build for the `reference` input.
  - `blacklist` : Genome build for the `blacklist` input. <br> E.g.
    `genome_build = list(peakfiles="hg38", reference="hg19", blacklist="hg19")`
- `genome_build_output` Genome build to standardise all inputs to.
  Liftovers will be performed automatically as needed. Default is
  “hg19”.
- `blacklist` : Peakfile as GRanges object specifying genomic regions
  that have anomalous and/or unstructured signals independent of the
  cell-line or experiment. For human hg19 and hg38 genome, use built-in
  data `data(hg19_blacklist)` and `data(hg38_blacklist)` respectively.
  For mouse mm10 genome, use built-in data `data(mm10_blacklist)`.
- `output_dir` : Please specify the path to directory, where all
  `EpiCompare` outputs will be saved.

</details>

#### Optional Inputs

The following input files are optional:

<details>

<summary>

👈 <strong>Details</strong>
</summary>

- `picard_files` : A list of summary metrics output from
  [Picard](https://broadinstitute.github.io/picard/). *Picard
  MarkDuplicates* can be used to identify the duplicate reads amongst
  the alignment. This tool generates a summary output, normally with the
  ending *.markdup.MarkDuplicates.metrics.txt*. If this input is
  provided, metrics on fragments (e.g. mapped fragments and duplication
  rate) will be included in the report. Files must be in data.frame
  format and listed using `list()` and named using `names()`. To import
  Picard duplication metrics (.txt file) into R as data frame, use
  `picard <- read.table("/path/to/picard/output", header = TRUE, fill = TRUE)`.
- `reference` : Reference peak file(s) is used in `stat_plot` and
  `chromHMM_plot`. File must be in `GRanges` object, listed and named
  using `list("reference_name" = GRanges_obect)`. If more than one
  reference is specified, `EpiCompare` outputs individual reports for
  each reference. However, please note that this can take awhile.

</details>

#### Optional Plots

By default, these plots will not be included in the report unless set to
`TRUE`. To turn on all features at once, simply use the `run_all=TRUE`
argument:

<details>

<summary>

👈 <strong>Details</strong>
</summary>

- `upset_plot` : Upset plot of overlapping peaks between samples.
- `stat_plot` : included only if a `reference` dataset is provided. The
  plot shows statistical significance (p/q-values) of sample peaks that
  are overlapping/non-overlapping with the `reference` dataset.
- `chromHMM_plot` : ChromHMM annotation of peaks. If a `reference`
  dataset is provided, ChromHMM annotation of overlapping and
  non-overlapping peaks with the `reference` is also included in the
  report.
- `chipseeker_plot` : ChIPseeker annotation of peaks.
- `enrichment_plot` : KEGG pathway and GO enrichment analysis of peaks.
- `tss_plot` : Peak frequency around (+/- 3000bp) transcriptional start
  site. Note that it may take awhile to generate this plot for large
  sample sizes.
- `precision_recall_plot` : Plot showing the precision-recall score
  across the peak calling stringency thresholds.
- `corr_plot` : Plot showing the correlation between the quantiles when
  the genome is binned at a set size. These quantiles are based on the
  intensity of the peak, dependent on the peak caller used (q-value for
  MACS2).

</details>

#### Other Options

<details>

<summary>

👈 <strong>Details</strong>
</summary>

- `chromHMM_annotation` : Cell-line annotation for ChromHMM. Default is
  K562. Options are:
  - “K562” = K-562 cells
  - “Gm12878” = Cellosaurus cell-line GM12878
  - “H1hesc” = H1 Human Embryonic Stem Cell
  - “Hepg2” = Hep G2 cell
  - “Hmec” = Human Mammary Epithelial Cell
  - “Hsmm” = Human Skeletal Muscle Myoblasts
  - “Huvec” = Human Umbilical Vein Endothelial Cells
  - “Nhek” = Normal Human Epidermal Keratinocytes
  - “Nhlf” = Normal Human Lung Fibroblasts
- `interact` : By default, all heatmaps (percentage overlap and ChromHMM
  heatmaps) in the report will be interactive. If set FALSE, all
  heatmaps will be static. N.B. If `interact=TRUE`, interactive heatmaps
  will be saved as html files, which may take time for larger sample
  sizes.
- `output_filename` : By default, the report is named *EpiCompare.html*.
  You can specify the file name of the report here.
- `output_timestamp` : By default FALSE. If TRUE, the filename of the
  report includes the date.

</details>

#### Outputs

`EpiCompare` outputs the following:

1.  **HTML report**: A summary of all analyses saved in specified
    `output_dir`
2.  **EpiCompare_file**: if `save_output=TRUE`, all plots generated by
    `EpiCompare` will be saved in *EpiCompare_file* directory also in
    specified `output_dir`

An example report comparing ATAC-seq and DNase-seq can be found
[here](https://neurogenomics.github.io/EpiCompare/articles/example_report)

## Datasets

`EpiCompare` includes several built-in datasets:

<details>

<summary>

👈 <strong>Details</strong>
</summary>

- `encode_H3K27ac`: Human H3K27ac peak file generated with ChIP-seq
  using K562 cell-line. Taken from
  [ENCODE](https://www.encodeproject.org/files/ENCFF044JNJ/) project.
  For more information, run `?encode_H3K27ac`.  
- `CnT_H3K27ac`: Human H3K27ac peak file generated with CUT&Tag using
  K562 cell-line from [Kaya-Okur et al.,
  (2019)](https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8383507).
  For more information, run `?CnT_H3K27ac`.
- `CnR_H3K27ac`: Human H3K27ac peak file generated with CUT&Run using
  K562 cell-line from [Meers et al.,
  (2019)](https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8581604).
  For more details, run `?CnR_H3K27ac`.

</details>

## Contact

### [Neurogenomics Lab](https://www.neurogenomics.co.uk/inst/report/EpiCompare.html)

UK Dementia Research Institute  
Department of Brain Sciences  
Faculty of Medicine  
Imperial College London  
[GitHub](https://github.com/neurogenomics)  
[DockerHub](https://hub.docker.com/orgs/neurogenomicslab)

## Session Info

<details>

<summary>

👈 <strong>Details</strong>
</summary>

``` r
utils::sessionInfo()
```

    ## R Under development (unstable) (2025-08-22 r88678)
    ## Platform: aarch64-apple-darwin20
    ## Running under: macOS Sequoia 15.7
    ## 
    ## Matrix products: default
    ## BLAS:   /Library/Frameworks/R.framework/Versions/4.6-arm64/Resources/lib/libRblas.0.dylib 
    ## LAPACK: /Library/Frameworks/R.framework/Versions/4.6-arm64/Resources/lib/libRlapack.dylib;  LAPACK version 3.12.1
    ## 
    ## locale:
    ## [1] en_US.UTF-8/en_US.UTF-8/en_US.UTF-8/C/en_US.UTF-8/en_US.UTF-8
    ## 
    ## time zone: Europe/London
    ## tzcode source: internal
    ## 
    ## attached base packages:
    ## [1] stats     graphics  grDevices utils     datasets  methods   base     
    ## 
    ## loaded via a namespace (and not attached):
    ##  [1] gtable_0.3.6        jsonlite_2.0.0      renv_1.1.5         
    ##  [4] dplyr_1.1.4         compiler_4.6.0      BiocManager_1.30.26
    ##  [7] tidyselect_1.2.1    rvcheck_0.2.1       scales_1.4.0       
    ## [10] yaml_2.3.10         fastmap_1.2.0       here_1.0.2         
    ## [13] ggplot2_4.0.0       R6_2.6.1            generics_0.1.4     
    ## [16] knitr_1.50          yulab.utils_0.2.1   tibble_3.3.0       
    ## [19] desc_1.4.3          dlstats_0.1.7       rprojroot_2.1.1    
    ## [22] pillar_1.11.1       RColorBrewer_1.1-3  rlang_1.1.6        
    ## [25] badger_0.2.5        xfun_0.53           fs_1.6.6           
    ## [28] S7_0.2.0            cli_3.6.5           magrittr_2.0.4     
    ## [31] rworkflows_1.0.6    digest_0.6.37       grid_4.6.0         
    ## [34] rstudioapi_0.17.1   rappdirs_0.3.3      lifecycle_1.0.4    
    ## [37] vctrs_0.6.5         evaluate_1.0.5      glue_1.8.0         
    ## [40] data.table_1.17.8   farver_2.1.2        rmarkdown_2.29     
    ## [43] tools_4.6.0         pkgconfig_2.0.3     htmltools_0.5.8.1

</details>

<hr>

#' Group files
#' 
#' Assign group names to each file in a named list based on a series of string
#' searches based on combinations of relevant metadata factors.
#' @param searches A named list of substrings to group \code{peakfiles} by.
#' @inheritParams EpiCompare
#' @returns Named peak files
#' 
#' @export
#' @importFrom stats setNames
#' @examples 
#' data("encode_H3K27ac") # example dataset as GRanges object
#' data("CnT_H3K27ac") # example dataset as GRanges object
#' data("CnR_H3K27ac") # example dataset as GRanges object
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, 
#'                   CnT_H3K27ac=CnT_H3K27ac, 
#'                   encode_H3K27ac=encode_H3K27ac)
#'
#' peaks_grouped <- group_files(peakfiles = peakfiles,
#'                              searches=list(assay=c("H3K27ac"),
#'                                            source=c("Cn","ENCODE")))
group_files <- function(peakfiles, 
                        searches){
    
    check_dep("tidyr")
    combos <- data.frame(expand.grid(searches, stringsAsFactors = FALSE),
                         stringsAsFactors = FALSE)
    rownames(combos) <- tidyr::unite(data = combos, "merged")[,1] 
    peaks_grouped <- mapply(stats::setNames(names(peakfiles),
                                            names(peakfiles)),
        SIMPLIFY = FALSE,                            
        FUN=function(nm){
            rownames(combos)[
                mapply(rownames(combos), 
                       SIMPLIFY = TRUE,
                       FUN = function(rn){
                   all(
                       unlist(
                           lapply(colnames(combos), function(x){ 
                               grepl(pattern = combos[rn,,drop=FALSE][[x]], 
                                     x = nm, 
                                     ignore.case = TRUE) 
                           })
                       )
                   )
                       })
            ] 
        }) 
    return(unlist(peaks_grouped))
}

test_that("group_files works", {
  
    data("encode_H3K27ac") # example dataset as GRanges object
    data("CnT_H3K27ac") # example dataset as GRanges object
    data("CnR_H3K27ac") # example dataset as GRanges object
    peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac,
                      CnT_H3K27ac=CnT_H3K27ac,
                      encode_H3K27ac=encode_H3K27ac)

    peaks_grouped <- group_files(peakfiles = peakfiles,
                                 searches=list(assay=c("H3K27ac"),
                                               source=c("Cn","ENCODE"))) 
    testthat::expect_true(length(peakfiles)==length(peaks_grouped))
    testthat::expect_true(length(unique(peaks_grouped))==2)
    testthat::expect_true(all(c("H3K27ac_Cn","H3K27ac_Cn","H3K27ac_ENCODE")
                              %in% peaks_grouped))
    
    
    peaks_grouped <- group_files(peakfiles = peakfiles,
                                 searches=list(assay=c("H3K27ac")) ) 
    testthat::expect_true(length(peakfiles)==length(peaks_grouped))
    testthat::expect_true(length(unique(peaks_grouped))==1) 
})

#' Remove odd chromosomes from GRanges objects
#'
#' This convenience function removes non-standard, mitochondrial, and/or sex
#' chromosomes from any GRanges object.
#' 
#' This function is adapted from \code{tidyChromosomes} in the
#' \code{BRGenomics} package licensed under the Artistic License 2.0.
#' Original author: Mike DeBerardine <https://github.com/mdeber>
#'
#' @param gr Any GRanges object, or any another object with associated
#'   \code{seqinfo} (or a \code{Seqinfo} object itself). The object should
#'   typically have a standard genome associated with it, e.g. \code{genome(gr)
#'   <- "hg38"}. \code{gr} can also be a list of such GRanges objects.
#' @param keep.X,keep.Y,keep.M,keep.nonstandard Logicals indicating which
#'   non-autosomes should be kept. By default, sex chromosomes are kept, but
#'   mitochondrial and non-standard chromosomes are removed.
#' @param genome An optional string that, if supplied, will be used to set the
#'   genome of \code{gr}.
#'
#' @return A GRanges object in which both ranges and \code{seqinfo} associated
#'   with trimmed chromosomes have been removed.
#'
#' @details Standard chromosomes are defined using the
#'   \code{\link[GenomeInfoDb]{standardChromosomes}} function
#'   from the \pkg{GenomeInfoDb} package.
#'
#' @author Mike DeBerardine
#' @seealso \code{\link[GenomeInfoDb]{standardChromosomes}}
#'
#' @export
#' @importFrom Seqinfo seqlevels sortSeqlevels genome genome<-
#' @importFrom GenomeInfoDb standardChromosomes keepSeqlevels
#' @importFrom methods is
#' @examples
#' # make a GRanges
#' chrom <- c("chr2", "chr3", "chrX", "chrY", "chrM", "junk")
#' gr <- GenomicRanges::GRanges(seqnames = chrom,
#'               ranges = IRanges::IRanges(start = 2*(1:6), end = 3*(1:6)),
#'               strand = "+",
#'               seqinfo = Seqinfo::Seqinfo(chrom))
#' Seqinfo::genome(gr) <- "hg38"
#'
#' gr
#'
#' tidy_chromosomes(gr)
#'
#' tidy_chromosomes(gr, keep.M = TRUE)
#'
#' tidy_chromosomes(gr, keep.M = TRUE, keep.Y = FALSE)
#'
#' tidy_chromosomes(gr, keep.nonstandard = TRUE)
#' 
#' @keywords internal
tidy_chromosomes <- function(gr, keep.X = TRUE, keep.Y = TRUE, keep.M = FALSE,
                            keep.nonstandard = FALSE, genome = NULL) {
    
    if (is.list(gr) || is(gr, "GRangesList"))
        return(lapply(gr, tidy_chromosomes, keep.X, keep.Y, keep.M,
                      keep.nonstandard, genome))
    
    if (!is.null(genome))
        genome(gr) <- genome
    
    chrom <- standardChromosomes(gr)
    
    if (keep.nonstandard) chrom <- seqlevels(gr)
    if (!keep.X)  chrom <- chrom[ chrom != "chrX" ]
    if (!keep.Y)  chrom <- chrom[ chrom != "chrY" ]
    if (!keep.M)  chrom <- chrom[ (chrom != "chrM") & (chrom != "chrMT") ]
    
    if (is(gr, "Seqinfo")) {
        gr <- keepSeqlevels(gr, chrom)
    } else {
        gr <- keepSeqlevels(gr, chrom, pruning.mode = "tidy")
    }
    sortSeqlevels(gr)
}

# This function and its tests were adapted from \code{tidyChromosomes} in the
# \code{BRGenomics} package licensed under the Artistic License 2.0.
# Original author: Mike DeBerardine <https://github.com/mdeber>

library(GenomeInfoDb)  # for dropSeqlevels()

bw <- GRanges(seqnames = c("chr2L", "chrM", "chrX", "chrY", "unassigned"),
              ranges = IRanges(1:5, 2:6))
genome(bw) <- "dm6"

test_that("sex chromosomes", {
    trim_x <- tidy_chromosomes(bw, keep.X = FALSE, keep.M = TRUE,
                              keep.nonstandard = TRUE)
    expect_equal(length(trim_x), length(bw)-1)
    expect_equal(seqinfo(trim_x),
                 sortSeqlevels(dropSeqlevels(seqinfo(bw), "chrX")))
    
    trim_y <- tidy_chromosomes(bw, keep.Y = FALSE, keep.M = TRUE,
                              keep.nonstandard = TRUE)
    expect_equal(length(trim_y), length(bw)-1)
    expect_equal(seqinfo(trim_y),
                 sortSeqlevels(dropSeqlevels(seqinfo(bw), "chrY")))
})

test_that("trim non-standard, keep sex", {
    # default keeps sex, trims M and non-standard
    trim_default <- tidy_chromosomes(bw, keep.X = TRUE, keep.Y = TRUE,
                                    keep.M = FALSE,
                                    keep.nonstandard = FALSE)
    expect_equal(length(trim_default), length(bw)-2)
    expect_equal(seqinfo(trim_default),
                 sortSeqlevels(dropSeqlevels(seqinfo(bw),
                                             c("chrM", "unassigned"))))
})

test_that("use pre-assigned genome", {
    expect_identical(tidy_chromosomes(bw),
                     tidy_chromosomes(bw, genome = "dm6"))
})

test_that("trim from list input", {
    trim_ls <- tidy_chromosomes(list(bw, bw))
    expect_type(trim_ls, "list")
    expect_identical(trim_ls[[1]], tidy_chromosomes(bw))
})

test_that("trim seqinfo object", {
    si <- seqinfo(bw)
    expect_identical(tidy_chromosomes(si), seqinfo(tidy_chromosomes(bw)))
})

data("encode_H3K27ac")
data("CnT_H3K27ac")
data("CnR_H3K27ac")
data("hg19_blacklist")

# test names
test_that("Default names added to peaklist if none provided",{
  # no names are provided
  peaklist <- list(encode_H3K27ac, CnT_H3K27ac, CnR_H3K27ac)
  peaklist <- check_list_names(peaklist)
  expect_equal(names(peaklist), c("sample1", "sample2", "sample3"))
  # one name is provided
  names(peaklist) <- c("encode")
  peaklist <- check_list_names(peaklist)
  expect_equal(names(peaklist), c("encode", "sample2", "sample3"))
})

# test tidy peakfile
test_that("tidy_peakfile() removes peaks in blacklisted
          and non-standard regions",{
  peaklist <- list("encode"=encode_H3K27ac)
  peaklist_tidy <- EpiCompare::tidy_peakfile(peaklist, hg19_blacklist)
  peaklist_chrm_removed <- tidy_chromosomes(peaklist[[1]],
                                            keep.X = TRUE,
                                            keep.Y = TRUE)
  peaklist_chrm_blist_removed <-IRanges::subsetByOverlaps(peaklist_chrm_removed,
                                                          hg19_blacklist,
                                                          invert = TRUE)
  testthat::expect_equal(length(peaklist_tidy[[1]]),
                          length(peaklist_chrm_blist_removed))
})

report_time <- function(t1, 
                        func=NULL,
                        verbose=TRUE){
    messager(if(!is.null(func))paste0(func,"():"),
             "Done in",
             paste0(round(difftime(Sys.time(),t1,units = "s"),1),"s."),
             v=verbose)
}

#' Plot ChromHMM heatmap
#'
#' Creates a heatmap using outputs from ChromHMM using ggplot2.The function
#' takes a list of peakfiles, performs ChromHMM and outputs a heatmap. ChromHMM
#' annotation file must be loaded prior to using this function.
#' ChromHMM annotations are aligned to hg19, and will be automatically lifted
#' over to the \code{genome_build} to match the build of the \code{peaklist}.
#'
#' @param peaklist A named \link[base]{list} of peak files as GRanges object.
#' If list is not named, default names will be assigned.
#' @param chromHMM_annotation ChromHMM annotation list.
#' @param cell_line If not \code{cell_line},
#' will replace \code{chromHMM_annotation}
#'  by importing chromHMM data for a given cell line using
#'  \link[EpiCompare]{get_chromHMM_annotation}.
#' @param genome_build The human genome reference build used to generate
#' peakfiles. "hg19" or "hg38".
#' @param interact Default TRUE. By default, the heatmaps are interactive.
#' If\code{FALSE}, the function generates a static ChromHMM heatmap.
#' @param return_data Return the plot data as in addition to the plot itself.
#' @return ChromHMM heatmap, or a named list.
#'
#' @importMethodsFrom genomation annotateWithFeatures
#' @importFrom genomation heatTargetAnnotation
#' @importFrom GenomicRanges GRangesList
#' @importFrom reshape2 melt
#' @import ggplot2
#' @importMethodsFrom rtracklayer liftOver
#' @importFrom AnnotationHub AnnotationHub
#'
#' @export
#' @examples
#' ### Load Data ###
#' data("CnT_H3K27ac") # example dataset as GRanges object
#' data("CnR_H3K27ac") # example dataset as GRanges object 
#' ### Create Named Peaklist ###
#' peaklist <- list(CnT=CnT_H3K27ac, CnR=CnR_H3K27ac) 
#' ### Run ###
#' my_plot <- plot_chromHMM(peaklist = peaklist,
#'                          cell_line = "K562",
#'                          genome_build = "hg19") 
plot_chromHMM <- function(peaklist,
                          chromHMM_annotation,
                          genome_build,
                          cell_line=NULL,
                          interact=FALSE,
                          return_data=FALSE){
  # devoptera::args2vars(plot_chromHMM)
  State <- Sample <- value <- NULL;

  message("--- Running plot_chromHMM() ---")
  #### Automatically get cell line data ####
  if(!is.null(cell_line) &&
     !is.null(check_cell_lines(cell_lines = cell_line,
                               verbose = FALSE))){
      chromHMM_annotation <- get_chromHMM_annotation(cell_line = cell_line)
  }
  if(missing(chromHMM_annotation)) {
      stp <- "Must supply chromHMM_annotation or cell_line."
      stop(stp)
  }
  if(missing(genome_build)) {
    stp <- "Must supply genome_build."
    stop(stp)
  }
  #### Liftover ####
  ## chromHMM_annotation must be lifted over to match genome build of peaklist.
  chromHMM_annotation <- liftover_grlist(grlist = chromHMM_annotation,
                                         input_build = "hg19",
                                         output_build = genome_build,
                                         as_grangeslist = TRUE)

  # check that peaklist is named, if not, default names assigned
  peaklist <- prepare_peaklist(peaklist = peaklist,
                               remove_empty = TRUE,
                               as_grangeslist = TRUE)
  peaklist <- methods::as(peaklist,"GRangesList")
  # annotate peakfiles with chromHMM annotations
  message("Annotating with features.")
  annotation <- genomation::annotateWithFeatures(
      target = peaklist,
      features = chromHMM_annotation)
  # obtain matrix
  message("Obtaining target annotation matrix.")
  matrix <- genomation::heatTargetAnnotation(annotation, 
                                             plot = FALSE)
  # remove numbers in front of states
  label_corrected <- gsub('X', '', colnames(matrix))
  colnames(matrix) <- label_corrected # set corrected labels
  # convert matrix into molten data frame
  matrix_melt <- reshape2::melt(matrix)
  colnames(matrix_melt) <- c("Sample", "State", "value")
  # order State labels
  sorted_label <- unique(stringr::str_sort(matrix_melt$State,
                                           numeric = TRUE))
  nm <- factor(matrix_melt$State, levels=sorted_label)
  matrix_melt$State <- nm
  # create heatmap
  chrHMM_plot <- ggplot2::ggplot(matrix_melt) +
    ggplot2::geom_tile(ggplot2::aes(x = State,
                                    y = Sample,
                                    fill = value)) +
    ggplot2::ylab("") +
    ggplot2::xlab("") +
    ggplot2::scale_fill_viridis_c() +
    ggplot2::theme_minimal() +
    ggplot2::theme(axis.text = ggplot2::element_text(size = 11)) +
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 315,
                                                       vjust = 0.5,
                                                       hjust = 0))
    #### Make plot interactive ####
    if(isTRUE(interact)){
        chrHMM_plot <- as_interactive(chrHMM_plot) 
    }
    #### Return ####
    if(isTRUE(return_data)){
        message("Returning named list with both plot and data.")
        return(
            list(plot=chrHMM_plot,
                 data=matrix_melt)
        )
    } else{
        message("Returning chrHMM_plot.")
        return(chrHMM_plot)
    }
}

#' Report command
#' 
#' Reconstruct the \link[EpiCompare]{EpiCompare} command 
#' used to generate the current Rmarkdown report.
#' @param params Parameters supplied to the Rmarkdown template.
#' @param peaklist_tidy Post-processed target peaks.
#' @param reference_tidy Post-processed reference peaks.
#' @returns String reconstructing R function call.
#' 
#' @export
#' @examples 
#' # report_command()
report_command <- function(params,
                           peaklist_tidy,
                           reference_tidy){
  cmd <- paste(
    paste0(
      "EpiCompare(peakfiles = list(",
      paste(shQuote(names(peaklist_tidy)),collapse = ","),
      ")"
    ),
    paste0(
      "genome_build =  list(",
      paste(shQuote(params$genome_build), collapse = ","),
      ")"
    ),
    paste0("genome_build_output = ",shQuote(params$genome_build_output)),
    paste0(
      "blacklist = ",if(is.character(params$blacklist)) {
        shQuote(params$blacklist)
      } else if(is.null(params$blacklist)) {
        "NULL"
      } else {
        shQuote("<user_supplied>")
      }
    ),
    paste0("picard_files = list(",
           paste(shQuote(names(params$picard_files)),collapse = ","),
           ")"
    ),
    paste0(
      "reference = ",if(is.null(params$reference)) {
        "NULL"
      } else {
        paste(shQuote(names(reference_tidy)), collapse = ",")
      }
    ),
    paste0("upset_plot = ",params$upset_plot),
    paste0("stat_plot = ",params$stat_plot),
    paste0("chromHMM_plot = ",params$chromHMM_plot),
    paste0("chromHMM_annotation = ",shQuote(params$chromHMM_annotation)),
    paste0("chipseeker_plot = ",params$chipseeker_plot),
    paste0("enrichment_plot = ",params$enrichment_plot),
    paste0("tss_plot = ",params$tss_plot),
    paste0("tss_distance = c(",paste0(params$tss_distance, collapse = ","),")"),
    paste0("precision_recall_plot = ",params$precision_recall_plot),
    paste0("n_threshold = ",params$n_threshold),
    paste0("corr_plot = ",params$corr_plot),
    paste0("interact = ",params$interact),
    paste0("save_output = ",params$save_output),
    paste0(
      "output_dir = ",if(is.null(params$output_dir)){
        "NULL"
      } else {
        shQuote(params$output_dir)
      }
    ),
    paste0("workers = ",params$workers),
    paste0(paste0("error = ",params$error),")"),
    sep = paste0(",\n", paste0(rep(" ",nchar("EpiCompare(")),collapse = "") )
  ) 
  ## <pre> tag is neccesarry for multiline code
  cmd <- paste0("<pre><code class='language-r'>",cmd,"</code></pre>")
  return(cmd)
}

#' Check peaklist is named
#'
#' This function checks whether the peaklist is named.
#' If not, default file names are assigned.
#'
#' @param peaklist A list of peak files as GRanges object.
#' @param default_prefix Default prefix to use when creating names
#'  for \code{peaklist}.
#' @return named peaklist
#' @keywords internal
check_list_names <- function(peaklist,
                             default_prefix="sample"){
  # check that peaklist is named
  # if not, default file names are used
  if(is.null(names(peaklist))){
      names(peaklist) <- paste0(default_prefix, seq_len(length(peaklist)))
  }
  # check for any missing names
  for(i in seq_len(length(peaklist))){
    if(is.na(names(peaklist)[i])){
      names(peaklist)[i] <- paste0(default_prefix, i)
    }
  }
    ####  Check for duplicate names ####
    dup_names <- names(peaklist)[duplicated(names(peaklist))]
    if(length(dup_names)>0){
        message(paste(length(dup_names),"duplicated peaklist names found.",
                      "Forcing unique names with make.unique()."))
        names(peaklist) <- make.unique(names(peaklist))
    }
  return(peaklist)
}

#' Compute precision-recall
#' 
#' Compute precision and recall using each \link[GenomicRanges]{GRanges}
#'  object in \code{peakfiles} as the "query" 
#'  against each \link[GenomicRanges]{GRanges} object in \code{reference}
#'  as the subject.
#'  
#' @param thresholding_cols Depending on which columns are present, 
#'  \link[GenomicRanges]{GRanges} will 
#'  be filtered at each threshold according to one or more of the following:
#' \itemize{
#'  \item{"total_signal" : }{Used by the peak calling software 
#'  \href{https://github.com/FredHutch/SEACR}{SEACR}. 
#'  \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
#'  together or instead of "total_signal".}
#'  \item{"qValue"}{Used by the peak calling software 
#'  \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
#'  Should contain the negative log of the p-values after multiple 
#'  testing correction.}
#'  \item{"Peak Score" : }{
#'  Used by the peak calling software
#'   \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
#' }
#' @param initial_threshold Numeric threshold that was provided to SEACR 
#' (via the parameter \code{--ctrl}) when calling peaks without an IgG 
#' control.
#' @param max_threshold Maximum threshold to test.
#' @param n_threshold Number of thresholds to test. 
#' @param cast Cast the data into a format that's more compatible with 
#' \pkg{ggplot2}.
#' @param save_path File path to save precision-recall results to.
#' @param verbose Print messages.
#' @inheritParams EpiCompare
#' @inheritParams get_bpparam
#' @inheritParams check_workers
#' @inheritDotParams bpplapply
#' @return Overlap
#' 
#' @export
#' @importFrom data.table rbindlist
#' @examples 
#' data("CnR_H3K27ac")
#' data("CnT_H3K27ac")
#' data("encode_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
#' reference <- list("encode_H3K27ac" = encode_H3K27ac)
#'
#' pr_df <- precision_recall(peakfiles = peakfiles,
#'                           reference = reference,
#'                           workers = 1)
precision_recall <- function(peakfiles,
                             reference,
                             thresholding_cols=c("total_signal", 
                                                 "qValue",
                                                 "Peak Score"),
                             initial_threshold=0,
                             n_threshold=20,
                             max_threshold=1,
                             cast=TRUE,
                             workers=1,
                             verbose=TRUE,
                             save_path=
                                 tempfile(fileext = "precision_recall.csv"),
                             ...){ 
    
    precision <- recall <- F1 <- NULL; 
    
    workers <- check_workers(workers = workers)
    threshold_list <- seq(from=initial_threshold, 
                          to=1-(max_threshold/n_threshold), 
                          length.out=n_threshold)
    names(threshold_list) <- threshold_list 
    #### Check which have necessary columns #####
    peakfiles <- check_grlist_cols(grlist = peakfiles, 
                                   target_cols = thresholding_cols)
    ##### Iterate over peakfiles ##### 
    pr_df <- bpplapply(X = threshold_list,
                         workers = workers,
                         FUN = function(thresh){
      if(verbose) message_parallel("Threshold=",thresh,": Filtering peaks")
      peakfiles_filt <- mapply(peakfiles, 
                              FUN=function(gr){  
                                  ## Compute percentiles
                                  gr <- compute_percentiles(
                                      gr = gr, 
                                      thresholding_cols = thresholding_cols,
                                      initial_threshold = initial_threshold
                                  )
                                  ## Filter 
                                  gr <- filter_percentiles(
                                      gr = gr, 
                                      thresh = thresh, 
                                      thresholding_cols = thresholding_cols)
                                  return(gr) 
                              })   
      df <- overlap_percent(peaklist1 = peakfiles_filt, 
                            peaklist2 = reference, 
                            suppress_messages = FALSE,
                            precision_recall = TRUE)
      return(df)
    }, ...) |> data.table::rbindlist(use.names = TRUE, idcol = "threshold")
    #### Recast data ###
    if(isTRUE(cast)){ 
        #### Post-process data ####
        messager("Reformatting precision-recall data.")
        pr_df <- data.table::dcast(
            data = pr_df,
            formula = "peaklist1 + peaklist2 + threshold ~ type", 
            fun.aggregate = mean,
            value.var = c("Percentage","overlap","total")) 
        data.table::setnames(pr_df,
                             c("Percentage_precision","Percentage_recall"), 
                             c("precision","recall"))
        pr_df$threshold <- as.numeric(pr_df$threshold) 
        #### Compute F1 ##### 
        pr_df[,F1:=(2*(precision*recall) / (precision+recall))] 
        pr_df[is.na(F1),]$F1 <- 0
    }
    #### Save ####
    save_path <- save_results(dat = pr_df, 
                              save_path = save_path, 
                              type = "precision-recall")
    #### Return ####
    return(pr_df)
}

#' Is an object of class GRanges
#'
#' Check whether an object is of the class \link[GenomicRanges]{GRanges}.
#' @param obj Any R object.
#' @returns Boolean.
#' @keywords internal
#' @importFrom methods is 
is_granges <- function(obj) {
    methods::is(obj, "GRanges")
}

#' Rebin peaks
#' 
#' Standardise a list of peak files by rebinning them into fixd-width
#'  tiles across the genome.
#' @param as_sparse Return the rebinned peaks as a sparse matrix
#' (default: \code{TRUE}), 
#' which is more efficiently stored than a dense matrix (\code{FALSE}). 
#' @param sep Separator to be used after chromosome name (first item) and 
#' between start/end genomic coordinates (second item). 
#' @inheritParams compute_corr
#' @inheritParams remove_nonstandard_chrom
#' @inheritParams check_workers
#' @inheritDotParams bpplapply
#' @returns Binned peaks matrix
#' 
#' @export
#' @importFrom data.table rbindlist as.data.table dcast
#' @importFrom GenomicRanges tileGenome seqinfo binnedAverage coverage
#' @importFrom GenomicRanges seqnames start end 
#' @importFrom Seqinfo Seqinfo seqnames seqlevels
#' @importFrom BiocGenerics `%in%`
#' @examples 
#' data("CnR_H3K27ac") 
#' data("CnT_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac) 
#' 
#' #increasing bin_size for speed
#' peakfiles_rebinned <- rebin_peaks(peakfiles = peakfiles,
#'                                   genome_build = "hg19",
#'                                   bin_size = 5000,
#'                                   workers = 1)
rebin_peaks <- function(peakfiles,
                        genome_build,
                        intensity_cols=c("total_signal", 
                                         "qValue",
                                         "Peak Score",
                                         "score"), 
                        bin_size=5000,
                        keep_chr=NULL,
                        sep=c(":","-"),
                        drop_empty_chr=FALSE,
                        as_sparse=TRUE,
                        workers=check_workers(),
                        verbose=TRUE,
                        ...){ 
    
    #### check genome build ####
    workers <- check_workers(workers = workers)
    genome_build <- unique(tolower(genome_build))
    gen_build_err <- 
        paste0("All peak files must be of the same genome build.\nUse ",
               "EpiCompare::liftover_grlist on your data first and then rerun.")
    if(length(unique(genome_build))>1) stop(gen_build_err)
    #### get reference gen data ####
    ref_bsgen <- check_genome_build(genome_build = genome_build,
                                    type="bsgen") 
    #### Check which have necessary columns #####
    peakfiles <- check_grlist_cols(grlist = peakfiles, 
                                   target_cols = intensity_cols)
    #### Specify chromosomes ####
    if(isTRUE(drop_empty_chr)){
        present_chr <- lapply(
            peakfiles,
            function(x){unique(GenomicRanges::seqnames(x))}) |> 
            unlist() |> unique() |> as.character() 
        if(is.null(keep_chr)) {
            keep_chr <- present_chr
        } else {
            keep_chr <- keep_chr[keep_chr %in% present_chr] 
        }
    }
    if(!is.null(keep_chr)){
        keep_chr <- intersect(keep_chr, 
                              GenomicRanges::seqnames(ref_bsgen))
    } else {
        keep_chr <- GenomicRanges::seqnames(ref_bsgen)
    }  
    peakfiles <- remove_nonstandard_chrom(grlist = peakfiles,
                                          keep_chr = keep_chr,
                                          verbose = FALSE)
    #### Prepare windows ####
    ## Re-bin desired level,averaging intensity score 
    gr_windows <- 
        GenomicRanges::tileGenome(
            GenomicRanges::seqinfo(ref_bsgen),
            tilewidth=bin_size, 
            cut.last.tile.in.chrom=TRUE) 
    Seqinfo::seqlevels(gr_windows, pruning.mode="coarse") <- keep_chr
    #### Rebin peaks #### 
    messager("Standardising peak files in",
             formatC(length(gr_windows),big.mark = ","),"bins of",
             paste0(formatC(bin_size,big.mark = ",")," bp."), v=verbose)
    rebinned_peaks <- 
        bpplapply(X = peakfiles,
                  workers = workers,
                  FUN = function(gr){
                ## Compute percentiles
                gr <- compute_percentiles(
                    gr = gr, 
                    thresholding_cols = intensity_cols,
                    initial_threshold = 0
                ) 
                # get col name of intensity
                gr_names <- names(GenomicRanges::mcols(gr))
                #can be more than one just arbitrarily take first
                intens_col <-
                    gr_names[gr_names %in% 
                                 paste(intensity_cols,
                                       "percentile",
                                       sep="_")][1] 
                # measure to avg within the bins
                data_cov <-
                    GenomicRanges::coverage(gr,
                                            weight=intens_col)
                rm(gr)
                ## For some reason,
                ## setting na.rm=TRUE drastically increases compute time.
                gr <- GenomicRanges::binnedAverage(bins = gr_windows, 
                                                   numvar = data_cov, 
                                                   varname = "score",
                                                   na.rm = FALSE)
                return(gr$score)
    }, ...)   
    #### Merge data into matrix ####
    messager("Merging data into matrix.",v=verbose)
    rebinned_peaks <- do.call("cbind", rebinned_peaks)
    if(length(sep)==1) sep <- rep(sep,2)
    rownames(rebinned_peaks) <- paste0(
        GenomicRanges::seqnames(gr_windows),sep[1],
        GenomicRanges::start(gr_windows),sep[2],
        GenomicRanges::end(gr_windows))
    #### Convert to sparse ####
    if(isTRUE(as_sparse)){
        messager("Converting to sparse matrix.",v=verbose)
        check_dep("Matrix") 
        rebinned_peaks <- Matrix::Matrix(rebinned_peaks, sparse=TRUE)
    }
    #### Report ####
    if(isTRUE(verbose)){
        messager("Binned matrix size:",
                 paste(formatC(dim(rebinned_peaks),big.mark = ","),
                       collapse = " x ")
        )
        sparsity <- sum(rebinned_peaks == 0, na.rm = TRUE)/
            (nrow(rebinned_peaks)*ncol(rebinned_peaks)) 
        messager("Matrix sparsity:",round(sparsity,4))
    }
    return(rebinned_peaks)
}

test_that("rebin_peaks works", {
    
    data("CnR_H3K27ac")
    data("CnT_H3K27ac")
    peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, 
                      CnT_H3K27ac=CnT_H3K27ac,
                      CnT_missing=CnT_H3K27ac)
    #### Remove the necessary column to compute percentiles ####
    GenomicRanges::mcols(peakfiles$CnT_missing)$qValue <- NULL
    GenomicRanges::mcols(peakfiles$CnT_missing)$score <- NULL
    # Using large bin_size for speed
    #### With selected chroms ####
    peakfiles_rebinned <- EpiCompare::rebin_peaks(peakfiles = peakfiles,
                                                  genome_build = "hg19",
                                                  bin_size = 5000,
                                                  workers = 1)
    testthat::expect_true(methods::is(peakfiles_rebinned,"Matrix"))
    testthat::expect_equal(dim(peakfiles_rebinned),c(647114,2))
    
    #### With selected chroms ####
    keep_chr <- paste0("chr",seq_len(12))
    peakfiles_rebinned2 <- EpiCompare::rebin_peaks(peakfiles = peakfiles,
                                                   keep_chr = keep_chr,
                                                   genome_build = "hg19",
                                                   bin_size = 5000,
                                                   workers = 1)
    testthat::expect_true(methods::is(peakfiles_rebinned2,"Matrix"))
    testthat::expect_equal(dim(peakfiles_rebinned2),c(416959,2))
})

#' Download ChromHMM annotation file(s)
#'
#' Download ChromHMM annotation file(s) for a given cell-line
#'  (returned as a \link[GenomicRanges]{GRanges} object) 
#'  or a list of cell-lines
#'  (returned as a named list of \link[GenomicRanges]{GRanges} objects).
#'  All annotations are aligned to the hg19 genome build.
#'  All data can be found on the UCSC Genome Browser 
#'  \href{https://hgdownload.cse.ucsc.edu/goldenPath/hg19/encodeDCC/wgEncodeBroadHmm/}{
#'  here}. 
#' @param cell_line ChromHMM annotation for user-specified cell-line.
#' Cell-line options are:
#' \itemize{
#'   \item "K562" = K-562 cells
#'   \item "Gm12878" = Cellosaurus cell-line GM12878
#'   \item "H1hesc" = H1 Human Embryonic Stem Cell
#'   \item "Hepg2" = Hep G2 cell
#'   \item "Hmec" = Human Mammary Epithelial Cell
#'   \item "Hsmm" = Human Skeletal Muscle Myoblasts
#'   \item "Huvec" = Human Umbilical Vein Endothelial Cells
#'   \item "Nhek" = Normal Human Epidermal Keratinocytes
#'   \item "Nhlf" = Normal Human Lung Fibroblasts
#' } 
#' @returns Cell-line specific ChromHMM annotation file. Default K562 cell-line.
#' @importFrom genomation readBed
#' @import GenomicRanges
#' @keywords internal
get_chromHMM_annotation <- function(
        cell_line,
        cache=BiocFileCache::BiocFileCache(ask = FALSE)){
  #use bioconductor caching package
  check_dep("BiocFileCache")
  cell_line <- check_cell_lines(cell_lines = cell_line) 
  chromHMM_list_all <- lapply(cell_line, function(x){
      chrHMM_url <- paste0("http://hgdownload.cse.ucsc.edu/goldenPath/hg19/",
                           "encodeDCC/wgEncodeBroadHmm/wgEncodeBroadHmm",
                           x,"HMM.bed.gz")
      chrHMM <- genomation::readBed(chrHMM_url)
      chromHMM_list <- split(chrHMM, chrHMM$name, drop = TRUE)
      msg <- paste0("Adding ",x,"'s chrHMM to local cache,",
                    "future invocations will use local image.")
      if (!checkCache(cache,chrHMM_url)) message(msg)
      path <- BiocFileCache::bfcrpath(cache, chrHMM_url)
      # chrHMM <- genomation::readBed(path)
      return(chromHMM_list)
  }) 
  names(chromHMM_list_all) <- cell_line
  if(length(chromHMM_list_all)==1) {
      return(chromHMM_list_all[[1]])
  } else {
      chromHMM_list_all
  } 
}

#' Plot correlation of peak files
#' 
#' Plot correlation by binning genome and measuring correlation of peak quantile
#' ranking. This ranking is based on p-value or other peak intensity measure 
#' dependent on the peak calling approach.
#' @param show_plot Show the plot. 
#' @inheritParams compute_corr
#' @inheritParams rebin_peaks
#' @inheritParams check_workers 
#' @inheritParams precision_recall_matrix
#' @inheritParams overlap_heatmap
#' @return list with correlation plot (corr_plot) and correlation matrix (data)
#' 
#' @export 
#' @importFrom stringr str_wrap 
#' @examples 
#' data("CnR_H3K27ac")
#' data("CnT_H3K27ac")
#' data("encode_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
#' reference <- list("encode_H3K27ac"=encode_H3K27ac)
#' ## Increasing bin_size for speed here,
#' ## but lower values will give more precise results (and lower correlations)
#' cp <- plot_corr(peakfiles = peakfiles,
#'                 reference = reference,
#'                 genome_build = "hg19",
#'                 bin_size = 5000,
#'                 workers = 1)
plot_corr <- function(peakfiles,
                      reference = NULL,
                      genome_build,
                      bin_size = 5000,
                      keep_chr = NULL,
                      drop_empty_chr = FALSE,
                      method = "spearman",
                      intensity_cols=c("total_signal", 
                                       "qValue",
                                       "Peak Score",
                                       "score"),
                      interact=FALSE,
                      draw_cellnote=TRUE,
                      fill_diag=NA,
                      workers=check_workers(),
                      show_plot=TRUE,
                      save_path = tempfile(fileext = ".corr.csv.gz")){
    
    # devoptera::args2vars(plot_corr); genome_build = "hg19";
    check_dep("ggplot2")
    peakfile1 <- peakfile2 <- corr <- NULL;
    
    workers <- check_workers(workers = workers)
    #### Check deps ####
    fill_diag <- check_heatmap_args(draw_cellnote = draw_cellnote, 
                                    interact = interact, 
                                    fill_diag = fill_diag)
    #### Get corr matrix ####
    corr_mat <- compute_corr(peakfiles = peakfiles,
                             reference = reference,
                             genome_build = genome_build,
                             bin_size = bin_size,
                             keep_chr = keep_chr,
                             drop_empty_chr = drop_empty_chr,
                             method = method,
                             intensity_cols = intensity_cols,
                             fill_diag = fill_diag,
                             workers = workers,
                             save_path = save_path) 
    #### Plot correlation plot ####
    # diag(corr_mat) <- NA
    corr_mat_melt <- reshape2::melt(data = corr_mat,
                                    varnames = c("peakfile1","peakfile2"),
                                    value.name = "corr")
    width <- 80
    corr_mat_melt$peakfile1 <- stringr::str_wrap(corr_mat_melt$peakfile1,
                                                 width = width)
    corr_mat_melt$peakfile2 <- stringr::str_wrap(corr_mat_melt$peakfile2, 
                                                 width = width)
    corr_mat_melt$corr <- round(corr_mat_melt$corr,2)
    plt <- ggplot2::ggplot(corr_mat_melt, 
                    ggplot2::aes(x=peakfile1, 
                                 y=peakfile2, 
                                 fill=corr,
                                 label=corr)) + 
        ggplot2::geom_tile() + 
        ggplot2::scale_fill_viridis_c(option="magma", 
                                      na.value = "grey10",
                                      breaks=seq(0,1,.5),
                                      limits=c(0,1)) + 
        ggplot2::labs(x=NULL, y=NULL, title="Peak correlation matrix") +
        ggplot2::theme_bw() +
        ggplot2::theme(axis.text.x = ggplot2::element_text(angle=45,hjust=1))
    
    if(isTRUE(draw_cellnote)){ 
      plt <- plt + ggplot2::geom_label(fill=ggplot2::alpha("white",.8), 
                                       linewidth = NA,
                                       na.rm = TRUE)
    }
    if(isTRUE(interact)){
      ### Create Interactive Heatmap ###
      if(isTRUE(draw_cellnote)){
        #### heatmaply ####
        plt <- heatmap_heatmaply(X=corr_mat)
      } else {
        #### plotly #### 
        plt <- heatmap_plotly(X=corr_mat)
      } 
      plt <- as_interactive(plt)
    }
    #### With corrplot ####
    # corr_plot <-
    #   corrplot::corrplot(corr_mat, 
    #                      type="lower",
    #                      method="number",
    #                      col=RColorBrewer::brewer.pal(n=8, name="PuOr"),
    #                      tl.col="black", tl.srt=45)
    # #plot isn't returned, work around to record it
    # corr_plot <- grDevices::recordPlot()
    
    #### Show plots ####
    if(isTRUE(show_plot))  methods::show(plt)
    #### Return both the plot and data ####
    return(list(
        data=corr_mat,
        corr_plot=plt
    ))
}

test_that("plot_corr works", {
    
    data("CnR_H3K27ac")
    data("CnT_H3K27ac")
    data("encode_H3K27ac")
    peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
    reference <- list("encode_H3K27ac" = encode_H3K27ac)

    cp_out <- plot_corr(peakfiles = peakfiles,
                        reference = reference,
                        genome_build = "hg19", 
                        workers = 1,
                        bin_size = 5000)
    testthat::expect_equal(nrow(cp_out$data), 3)
    testthat::expect_equal(round(mean(cp_out$data, na.rm=TRUE),2), 0.63)
    testthat::expect_true(all(c("data","corr_plot") %in% names(cp_out)))
    testthat::expect_true(all(c("CnR_H3K27ac","CnT_H3K27ac",
                                  "encode_H3K27ac") %in% colnames(cp_out$data)))
    testthat::expect_true(methods::is(cp_out$corr_plot,"gg"))
})

prepare_output_build <- function(genome_build_output){
    opts <- c("hg19","hg38","mm9","mm10")
    if(is.null(genome_build_output) ||
       missing(genome_build_output) ||
       (!tolower(genome_build_output) %in% opts)){
        stop("Must provide valid genome_build_output: ",
        "'hg19', 'hg38', 'mm9', or 'mm10'")
    }
    return(tolower(genome_build_output)[[1]])
}

#' Summary on fragments
#'
#' This function outputs a summary on fragments using metrics generated by
#' Picard. Provides the number of mapped fragments, duplication rate and number
#' of unique fragments.
#'
#'
#' @param picard_list Named list of duplication metrics generated by Picard
#' as data frame. Data frames must be named and listed using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' To import Picard duplication metrics (.txt file) into R as data frame, use
#' \code{picard <- read.table("/path/to/picard/output", header = TRUE,
#' fill = TRUE)}.
#'
#' @return A table summarizing metrics on fragments.
#' @export
#'
#' @examples
#' ### Load Data ###
#' data(CnT_H3K27ac_picard) # example picard output
#' data(CnR_H3K27ac_picard) # example picard output 
#' ### Import Picard Metrics ###
#' # To import Picard duplication metrics (.txt file) into R as data frame
#' # CnT_H3K27ac_picard <- read.table("/path/to/picard/output.txt",
#' #  header = TRUE,fill = TRUE) 
#' ### Create Named List ###
#' picard_list <- list("CnT_H3K27ac"=CnT_H3K27ac_picard,
#'                "CnR_H3K27ac"=CnR_H3K27ac_picard) 
#' df <- fragment_info(picard_list = picard_list)
fragment_info <- function(picard_list){
  
  message("--- Running fragment_info() ---")
  ### Check List Names ###
  picard_list <- check_list_names(picard_list)

  ### Obtain Metrics ###
  Mapped_Fragments <- c()
  Duplication_Rate <- c()
  Unique_Fragments <- c()

  for(i in seq_len(length(picard_list))){
    Mapped_Fragments <- c(Mapped_Fragments, 
                          picard_list[[i]]$READ_PAIRS_EXAMINED[1])
    Duplication_Rate <- c(Duplication_Rate, 
                          round(picard_list[[i]]$PERCENT_DUPLICATION[1]*100, 2))
    unique <- ((1-picard_list[[i]]$PERCENT_DUPLICATION[1]))*
      as.numeric(picard_list[[i]]$READ_PAIRS_EXAMINED[1])
    Unique_Fragments <- c(Unique_Fragments, round(unique))
  }

  ### Create Data Frame ###
  df_metric <- data.frame(Mapped_Fragments, Duplication_Rate, Unique_Fragments)
  rownames(df_metric) <- names(picard_list) # set row names as sample names
  message("Done.")
  return(df_metric)
}

test_that("fragment_info works", {
   
  data("CnR_H3K27ac_picard")
  picardlist <- list("CnR"=CnR_H3K27ac_picard)
  
  info <- EpiCompare::fragment_info(picardlist)
  testthat::expect_true(methods::is(info,"data.frame"))
  testthat::expect_equal(ncol(info),3)
  testthat::expect_equal(nrow(info),1)
})

#' Read count frequency around TSS
#'
#' This function generates a plot of read count frequency around TSS.
#'
#' @param peaklist A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}
#' If not named, default file names will be assigned.
#' @param txdb A TxDb annotation object from Bioconductor.
#' @param tss_distance A vector specifying the distance upstream and downstream
#' around transcription start sites (TSS). 
#' The default value is \code{c(-3000,3000)}; meaning peak frequency 
#' 3000bp upstream and downstream of TSS will be displayed. 
#' @param conf Confidence interval threshold estimated by bootstrapping
#'  (\code{0.95} means 95%).
#' Argument passed to \link[ChIPseeker]{plotAvgProf}.
#' @param resample Number of bootstrapped iterations to run.
#' Argument passed to \link[ChIPseeker]{plotAvgProf}.
#' @param workers Number of cores to parallelise bootstrapping across.
#' Argument passed to \link[ChIPseeker]{plotAvgProf}. 
#' @inheritParams EpiCompare
#' @returns A named list of profile plots.
#'
#' @importFrom ChIPseeker getPromoters getTagMatrix plotAvgProf
#' @importFrom stats setNames
#' @export
#' @examples
#' ### Load Data ###
#' data("CnT_H3K27ac") # example peaklist GRanges object
#' data("CnR_H3K27ac") # example peaklist GRanges object
#' ### Create Named Peaklist ###
#' peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac) 
#' my_plot <- tss_plot(peaklist = peaklist, 
#'                     tss_distance=c(-50,50),
#'                     workers = 1)
tss_plot <- function(peaklist,
                     txdb = NULL,
                     tss_distance = c(-3000,3000),
                     conf = 0.95,
                     resample = 500,
                     interact = FALSE,
                     workers = check_workers()){
    
  # devoptera::args2vars(tss_plot)
  
  workers <- check_workers(workers = workers)
  message("--- Running tss_plot() ---")  
  #### Check Peaklist Names ####
  peaklist <- check_list_names(peaklist)
  #### Obtain Promoter Ranges ####
  upstream <- abs(tss_distance[1])
  downstream <- abs(tss_distance[2])
  promoters <- ChIPseeker::getPromoters(TxDb = txdb,
                                        upstream = upstream,
                                        downstream = downstream) 
  #### Calculate Tag Matrix ####
  tagMatrixList <- lapply(peaklist,
                          ChIPseeker::getTagMatrix,
                          windows = promoters,
                          upstream = upstream,
                          downstream = downstream,
                          TxDb = txdb,
                          verbose = FALSE)

  #### Generate Profile Plot ####
  plot_list <- lapply(stats::setNames(seq_len(length(tagMatrixList)),
                                      names(tagMatrixList)
                                      ),
                      FUN=function(n){
    ChIPseeker::plotAvgProf(tagMatrixList[[n]],
                            xlim = c(-upstream, downstream),
                            conf = conf,
                            resample = resample,
                            facet = "row",
                            ncpus = workers,
                            verbose = FALSE) +
      ggplot2::ggtitle(names(tagMatrixList)[n])
  }) 
  #### Return ####
  if(isTRUE(interact)){
    plot_list <- lapply(plot_list,as_interactive)
  }
  message("Done.")
  return(plot_list)
}

test_that("tss_plot works", {
 
  ### Load Data ###
  data("CnT_H3K27ac") # example peaklist GRanges object
  data("CnR_H3K27ac") # example peaklist GRanges object
  ### Create Named Peaklist ###
  peaks <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac) 
  txdb <- check_genome_build(genome_build = "hg19")
  my_plot <- tss_plot(peaklist = peaks, 
                      txdb = txdb,
                      tss_distance=c(-50,50),
                      workers = 1)
  testthat::expect_true(methods::is(my_plot$CnT,"gg"))
  testthat::expect_true(methods::is(my_plot$CnR,"gg"))
})

#' Prepare genome builds
#' 
#' Parse the \code{genome_build} argument into
#' \code{peaklist_build} and \code{reference_build}.
#' @inheritParams EpiCompare
#' @returns Named list.
#' @keywords internal
prepare_genome_builds <- function(genome_build,
                                  blacklist = NULL){
    
    item_names <- c("peakfiles","reference","blacklist")
    if(length(genome_build)==0){
        stop("Must provide genome_build.")
    } else if(length(genome_build)==1){
        message(paste("Assuming same genome_build",
                      "for all file inputs:","\n",
                      "'peakfiles': ", genome_build[[1]],"\n",
                      "'reference': ",genome_build[[1]],"\n",
                      "'blacklist': ",genome_build[[1]]
                      ))
        peakfiles_build <- genome_build
        reference_build <- genome_build
        blacklist_build <- genome_build
    } else if(is.null(names(genome_build)) && 
              (length(genome_build)==3)){ 
        peakfiles_build <- genome_build[[1]]
        reference_build <- genome_build[[2]]
        blacklist_build <- genome_build[[3]]
        message(paste("genome_build doesn't contain names.",
                      "Assuming element 1 is 'peakfiles':",peakfiles_build,"\n",
                      "Assuming element 2 is 'reference':",reference_build,"\n",
                      "Assuming element 3 is 'blacklist':",blacklist_build
                      ))
    } else if( all(item_names %in% names(genome_build)) ){
        names(genome_build) <- tolower(names(genome_build))
        peakfiles_build <- genome_build[["peakfiles"]]
        reference_build <- genome_build[["reference"]]
        blacklist_build <- genome_build[["blacklist"]]
        message("Assigning 'peakfiles': ",peakfiles_build)
        message("Assigning 'reference': ",reference_build) 
        message("Assigning 'blacklist': ",blacklist_build) 
    } else if( all(item_names[c(1,2)] %in% names(genome_build)) &&
               is.null(blacklist)){
      names(genome_build) <- tolower(names(genome_build))
      peakfiles_build <- genome_build[["peakfiles"]]
      reference_build <- genome_build[["reference"]]
      blacklist_build <- NULL
    }else {
        stop("Unable to parse genome_build.")
    }
    #### Return ####
    return(list(peaklist=peakfiles_build,
                reference=reference_build,
                blacklist=blacklist_build))
}

#' Prepare peaklist as GRanges
#'
#' @param peaklist A named list of peaks as \link[GenomicRanges]{GRanges}
#'  or paths to BED files.
#' @param remove_empty Remove any empty elements in the list.
#' @param as_grangeslist Convert output to class
#'  \link[GenomicRanges]{GRangesList} before returning. 
#' @returns A list of \link[GenomicRanges]{GRanges} objects
#'
#' @keywords internal
#' @importFrom methods is
#' @importFrom ChIPseeker readPeakFile
#' @importFrom GenomicRanges GRangesList
prepare_peaklist <- function(peaklist,
                             remove_empty=TRUE,
                             as_grangeslist=FALSE){
    message("Preparing peaklist.")
    # check that peaklist is named, if not, default names assigned
    if(methods::is(peaklist,"GRangesList")){
        peaklist <- as.list(peaklist)
    }
    if(!methods::is(peaklist,"list")){
        peaklist <- list(peaklist)
    }
    peaklist <- check_list_names(peaklist = peaklist,
                                 default_prefix = "sample")
    #### Remove empty elements ####
    if(isTRUE(remove_empty)){
        peaklist <- remove_empty_elements(peaklist = peaklist)
    }
    #### Import files if necessary
    ## if path is provided, create GRanges object
    peaklist <- lapply(peaklist, FUN=function(x){
      if(!methods::is(x,"GRanges")){
          x <- ChIPseeker::readPeakFile(x, as = "GRanges")
      }
      x <- clean_granges(gr = x)
      return(x)
  })

  # create GRangeList from GRanges objects
    if(isTRUE(as_grangeslist)){
        peaklist <- GenomicRanges::GRangesList(peaklist,
                                               compress = FALSE)
    }
  if(length(peaklist)==0) stop("peaklist must have a length>0.")
  return(peaklist)
}

#' Summary of Peak Information
#'
#' This function outputs a table summarizing information on the peak files.
#' Provides the total number of peaks and the percentage of peaks in blacklisted
#'  regions.
#'
#' @param peaklist A named list of peak files as GRanges object.
#' Objects listed using \code{list("name1" = peak, "name2" = peak2)}.
#' @param blacklist A GRanges object containing blacklisted regions.
#'
#' @return A summary table of peak information
#'
#' @importMethodsFrom IRanges subsetByOverlaps
#' @export
#'
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peakfile GRanges object
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("hg19_blacklist") # example blacklist GRanges object
#'
#' ### Named Peaklist ###
#' peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)
#'
#' ### Run ###
#' df <- peak_info(peaklist = peaklist,
#'                 blacklist = hg19_blacklist) 
peak_info <- function(peaklist, blacklist){
  message("--- Running peak_info() ---")
  ### Check peaklist names ###
  peaklist <- check_list_names(peaklist)

  ### Obtain Peak Number Before Standardisation ###
  peakN_before_tidy <- mapply(peaklist, FUN=function(file){
    length(file)
  })

  ### Obtain Blacklisted Region Percentage ###
  blacklist_percent <- mapply(peaklist, FUN=function(file){
    options(warn = -1) # silence warning for subsetByOverlaps
    blacklistN <- length(IRanges::subsetByOverlaps(file, blacklist))
    options(warn = 0) # turn warning back on
    blacklistP <- signif(blacklistN/length(file)*100, 3)
  })

  ### Obtain Non-standard Chromosome Percentage ###
  tidy_percent <- mapply(peaklist, FUN=function(file){
    peak_tidy <- tidy_chromosomes(file, keep.X = TRUE, keep.Y = TRUE)
    removedN <- length(file) - length(peak_tidy)
    percentage <- signif(removedN/length(file)*100, 3)
  })

  ### Standardize Peaklist ###
  peaklist_tidy <- tidy_peakfile(peaklist = peaklist,
                                 blacklist = blacklist)

  ### Obtain Peak Number After Standardisation ###
  peakN_after_tidy <- mapply(peaklist_tidy, FUN=function(file){
    length(file)
  })

  ### Create Data Frame ###
  df <- data.frame(peakN_before_tidy,
                   blacklist_percent,
                   tidy_percent,
                   peakN_after_tidy)

  colnames(df) <- c("PeakN Before Tidy",
                    "Blacklisted Peaks Removed (%)",
                    "Non-standard Peaks Removed (%)",
                    "PeakN After Tidy")
  message("Done.")
  return(df)
}

test_that("peak_info works",{
  
  data("encode_H3K27ac")
  data("hg19_blacklist")
  data("CnR_H3K27ac_picard")
  peaklist <- list("encode"=encode_H3K27ac) 
  
  info <- EpiCompare::peak_info(peaklist, hg19_blacklist)
  testthat::expect_true(methods::is(info,"data.frame"))
  testthat::expect_equal(ncol(info),4)
  testthat::expect_equal(nrow(info),1)
})

save_results <- function(dat,
                         save_path,
                         type=NULL,
                         verbose=TRUE,
                         ...){
    if(!is.null(save_path)){
        dir.create(dirname(save_path), showWarnings = FALSE, recursive = TRUE)
        messager("Saving",type,"results ==>",save_path,v=verbose)
        data.table::fwrite(x = dat, 
                           file = save_path,
                           ...)
    }
    return(save_path)
}

heatmap_plotly <- function(X,
                           ...){
  # plot_ly sorts axis labels alphabetically unless they're.
  # Causes problems when trying to add numeric labels to each cell
  requireNamespace("viridis")
  plotly::plot_ly(x = colnames(X),
                  y = rownames(X),
                  z = X,
                  type = "heatmap",
                  colors=viridis::magma(n=256),
                  ...)
}

#' Compute consensus peaks
#' 
#' Compute consensus peaks from a list of \link[GenomicRanges]{GRanges}.
#' 
#' \emph{NOTE:} If you get the error 
#' \code{"Error in serialize(data, node$con) : error writing to connection"},
#' try running \link[base]{closeAllConnections} 
#' and rerun \link[EpiCompare]{compute_consensus_peaks}. 
#' This error can sometimes occur when 
#' \link[EpiCompare]{compute_consensus_peaks}
#' has been disrupted partway through.
#' 
#' @param grlist Named list of \link[GenomicRanges]{GRanges} objects.
#' @param groups A character vector of the same length as \code{grlist} 
#' defining how to group \link[GenomicRanges]{GRanges} objects when 
#' computing consensus peaks.
#' @param method Method to call peaks with:
#' \itemize{
#' \item{"granges" : }{Simple overlap procedure using 
#' \link[GenomicRanges]{GRanges} functions.
#' Faster but less accurate.}
#' \item{"consensusseeker" : }{
#' Uses \link[consensusSeekeR]{findConsensusPeakRegions} to compute consensus
#' peaks. 
#' Slower but more accurate.}
#' }
#' @inheritParams check_genome_build
#' @inheritParams IRanges::slice
#' @inheritParams IRanges::reduce
#' @inheritDotParams consensusSeekeR::findConsensusPeakRegions
#' @returns Named list of consensus peak \link[GenomicRanges]{GRanges}.
#' 
#' @source \href{https://ro-che.info/articles/2018-07-11-chip-seq-consensus}{
#' GenomicRanges tutorial}
#' @source \href{https://doi.org/doi:10.18129/B9.bioc.consensusSeekeR}{
#' consensusSeekeR}
#' @export
#' @importFrom GenomicRanges coverage GRangesList GRanges
#' @importFrom IRanges slice reduce
#' @importFrom Seqinfo seqlevelsInUse
#' @examples 
#' data("encode_H3K27ac") # example dataset as GRanges object
#' data("CnT_H3K27ac") # example dataset as GRanges object
#' data("CnR_H3K27ac") # example dataset as GRanges object 
#' grlist <- list(CnR=CnR_H3K27ac, CnT=CnT_H3K27ac, ENCODE=encode_H3K27ac)
#'
#' consensus_peaks <- compute_consensus_peaks(grlist = grlist,
#'                                            groups = c("Imperial",
#'                                                       "Imperial",
#'                                                       "ENCODE"))
compute_consensus_peaks <- function(grlist,
                                    groups=NULL,
                                    genome_build,
                                    lower=2,
                                    upper=Inf,
                                    min.gapwidth=1L,
                                    method=c("granges","consensusseeker"),
                                    ...){
    method <- tolower(method)[1]
    if(method=="consensusseeker"){
        check_dep("consensusSeekeR")
        if(missing(genome_build)){
            stopper("Must provide genome_build when method='consensusseeker'")
        }
    }
    #### Checking groupings are valid ####
    if(is.null(groups)){
        groups <- "all"
    } else {
        if(length(groups)!=length(grlist)){
            stopper("groups must be the same length as grlist or NULL.")
        }
    }
    #### Remove non-standard chr ####
    grlist <- remove_nonstandard_chrom(grlist = grlist)
    t1 <- Sys.time()
    #### Find consensus peaks in each group ####
    consensus_peaks_grouped <- lapply(unique(groups), function(g){  
        grlist2 <- grlist[which(groups==g)]
        min_peaks <- min(unlist(lapply(grlist2,length)), na.rm=TRUE)
        max_peaks <- max(unlist(lapply(grlist2,length)), na.rm=TRUE)
        messager("Computing consensus peaks for group:",g,
                 paste0("\n - ",length(grlist2)," files ",
                        "\n - ",
                        formatC(min_peaks,big.mark = ","),"-",
                        formatC(max_peaks,big.mark = ","),
                        " peaks each"
                        ))
        if(length(grlist2)<2){
            messager(
                "WARNING:",
                "Cannot compute consensus peaks when group has <2 peak files.",
                "Returning original GRanges object instead.")
            return(grlist2[[1]])
        }
        grlist2 <- GenomicRanges::GRangesList(
            mapply(grlist2,
                   SIMPLIFY = FALSE,
                   FUN=clean_granges)
            ) 
        if(method=="consensusseeker"){ 
            consensus_peaks <- compute_consensus_peaks_consensusseeker(
                grlist = grlist2, 
                genome_build = genome_build,
                ...)
        }else if(method=="granges"){
            consensus_peaks <- compute_consensus_peaks_granges(
                grlist = grlist2,
                upper = upper, 
                lower = lower, 
                min.gapwidth = min.gapwidth)
        } else {
            stopper("Method must be one of:",
                    paste("\n -",c("granges","consensusseeker"),collapse = ""))
        }
        #### Report ####
        messager("Identified",formatC(length(consensus_peaks),big.mark = ","),
                 "consensus peaks from",formatC(length(grlist2),big.mark = ","),
                 "peak files")
        return(consensus_peaks)
    }) 
    names(consensus_peaks_grouped) <- unique(groups)
    #### Report time ####
    t2 <- Sys.time()
    messager("Done computing consensus peaks in",
             round(difftime(t2, t1, units = "min"),2),"min.")
    #### Return ####
    return(consensus_peaks_grouped)
}

test_that("compute_consensus_peaks works", {
  
    data("encode_H3K27ac") # example dataset as GRanges object
    data("CnT_H3K27ac") # example dataset as GRanges object
    data("CnR_H3K27ac") # example dataset as GRanges object
    grlist <- list(CnR=CnR_H3K27ac, CnT=CnT_H3K27ac, ENCODE=encode_H3K27ac)

    consensus_peaks <- compute_consensus_peaks(grlist = grlist,
                                               groups = c("Imperial",
                                                          "Imperial",
                                                          "ENCODE"), 
                                               method = "granges")
    testthat::expect_length(consensus_peaks$Imperial, 1325)
    testthat::expect_length(consensus_peaks$ENCODE, length(grlist$ENCODE)) 
    
    consensus_peaks2 <- compute_consensus_peaks(grlist = grlist,
                                               groups = c("Imperial",
                                                          "Imperial", 
                                                          "ENCODE"), 
                                               genome_build = "hg19",
                                               method = "consensusseeker")
    testthat::expect_length(consensus_peaks2$Imperial, 2982)
    testthat::expect_length(consensus_peaks$ENCODE, length(grlist$ENCODE)) 
})

compute_percentiles <- function(gr,
                                thresholding_cols=c("total_signal", 
                                                    "qValue",
                                                    "Peak Score"),
                                initial_threshold=0){
    check_dep("scales")
    compute_pct <- function(vec,
                            initial_threshold){
        vec2 <- stats::ecdf(vec)(vec)
        if(!is.null(initial_threshold)){
            vec2 <- scales::rescale_pal(range = c(initial_threshold,1))(vec2)
            # vec3 <- scales::rescale(vec2, c(threshold,1))
        }
        return(vec2)
    } 
    GenomicRanges::mcols(gr)$width <- GenomicRanges::width(gr) 
    # GenomicRanges::mcols(gr)$max_signal_width <- sapply(gr$max_signal_region,
    #     function(x){
    #         GenomicRanges::width(GenomicRanges::GRanges(x))
    #     }
    # ) 
    cols <- names( GenomicRanges::mcols(gr))
    for(nm in thresholding_cols){
        if(nm %in% cols){ 
            new_nm <- paste(nm,"percentile",sep="_")
            # messager("Creating new column:",new_nm)
            GenomicRanges::mcols(gr)[[new_nm]] <- compute_pct(
                vec = GenomicRanges::mcols(gr)[[nm]], 
                initial_threshold = initial_threshold)
        }
    } 
    return(gr)
}

#' Compare epigenomic datasets
#'
#' This function compares and analyses multiple epigenomic datasets and outputs
#' an HTML report containing all results of the analysis. The report is mainly
#' divided into three sections: (1) General Metrics on Peakfiles,
#' (2) Peak Overlaps and (3) Functional Annotation of Peaks.
#'
#' @param peakfiles A list of peak files as GRanges object and/or as paths to
#' BED files. If paths are provided, EpiCompare imports the file as GRanges
#' object. EpiCompare also accepts a list containing a mix of GRanges objects
#' and paths.Files must be listed and named using \code{list()}.
#' E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
#' default file names will be assigned.
#' @param genome_build A named list indicating the genome build used to
#' generate each of the following inputs:
#' \itemize{
#' \item{"peakfiles" : }{Genome build for the \code{peakfiles} input.
#' Assumes genome build is the same for each element in the \code{peakfiles}
#' list.}
#' \item{"reference" : }{Genome build for the \code{reference} input.}
#' \item{"blacklist" : }{Genome build for the \code{blacklist} input.}
#' }
#' Example input list:\cr
#'  \code{genome_build = list(peakfiles="hg38",
#'  reference="hg19", blacklist="hg19")}\cr\cr
#' Alternatively, you can supply a single character string instead of a list.
#' This should \emph{only} be done in situations where all three inputs
#' (\code{peakfiles}, \code{reference}, \code{blacklist}) are of the same
#' genome build. For example:\cr
#' \code{genome_build = "hg19"}\cr\cr
#' Supported genome builds are: "hg19", "hg38", "mm9" and "mm10".
#' @param genome_build_output Genome build to standardise all inputs to.
#' Liftovers will be performed automatically as needed.
#' Default: "hg19".\cr\cr
#' \strong{Note:} Cross-species liftovers are supported.
#' @param blacklist A \link[GenomicRanges]{GRanges} object
#'  containing blacklisted genomic regions.
#' Blacklists included in \pkg{EpiCompare} are:
#' \itemize{
#' \item{\code{NULL} (default): }{Automatically selects the appropriate
#'  blacklist based on the \code{genome_build_output} argument.}
#' \item{"hg19_blacklist": }{Regions of hg19 genome that have anomalous
#' and/or unstructured signals. \link[EpiCompare]{hg19_blacklist}}
#' \item{"hg38_blacklist": }{Regions of hg38 genome that have anomalous
#' and/or unstructured signals. \link[EpiCompare]{hg38_blacklist}}
#' \item{"mm10_blacklist": }{Regions of mm10 genome that have anomalous
#' and/or unstructured signals. \link[EpiCompare]{mm10_blacklist}}
#' \item{"mm9_blacklist": }{Blacklisted regions of mm10 genome that have been
#'  lifted over from \link[EpiCompare]{mm10_blacklist}.
#'  \link[EpiCompare]{mm9_blacklist}}
#' \item{\code{<user_input>}: }{A custom user-provided blacklist in
#' \link[GenomicRanges]{GRanges} format.}
#' }
#' @param picard_files A list of summary metrics output from Picard.
#' Files must be in data.frame format and listed using \code{list()}
#' and named using \code{names()}.
#' To import Picard duplication metrics (.txt file)
#'  into R as data frame, use:\cr
#' \code{picard <- read.table("/path/to/picard/output",
#'  header = TRUE, fill = TRUE)}.
#' @param reference A named list containing reference peak file(s) as GRanges
#'  object. Please ensure that the reference file is listed and named
#' i.e. \code{list("reference_name" = reference_peak)}. If more than one
#' reference is specified, individual reports for each reference will be
#' generated. However, please note that specifying more than one reference can
#' take awhile. If a reference is specified, it enables two analyses: (1) plot
#' showing statistical significance of overlapping/non-overlapping peaks; and
#' (2) ChromHMM of overlapping/non-overlapping peaks.
#' @param upset_plot Default FALSE. If TRUE, the report includes upset plot of
#' overlapping peaks.
#' @param stat_plot Default FALSE. If TRUE, the function creates a plot showing
#' the statistical significance of overlapping/non-overlapping peaks.
#' Reference peak file must be provided.
#' @param precision_recall_plot Default is FALSE. If TRUE,
#' creates a precision-recall curve plot and an F1 plot using
#' \link[EpiCompare]{plot_precision_recall}.
#' @param corr_plot Default is FALSE. If TRUE, creates a correlation plot across
#' all peak files using
#' \link[EpiCompare]{plot_corr}.
#' @param chromHMM_plot Default FALSE. If TRUE, the function outputs ChromHMM
#' heatmap of individual peak files. If a reference peak file is provided,
#' ChromHMM annotation of overlapping and non-overlapping peaks
#' is also provided.
#' @param chromHMM_annotation ChromHMM annotation for ChromHMM plots.
#' Default K562 cell-line. Cell-line options are:
#' \itemize{
#'   \item "K562" = K-562 cells
#'   \item "Gm12878" = Cellosaurus cell-line GM12878
#'   \item "H1hesc" = H1 Human Embryonic Stem Cell
#'   \item "Hepg2" = Hep G2 cell
#'   \item "Hmec" = Human Mammary Epithelial Cell
#'   \item "Hsmm" = Human Skeletal Muscle Myoblasts
#'   \item "Huvec" = Human Umbilical Vein Endothelial Cells
#'   \item "Nhek" = Normal Human Epidermal Keratinocytes
#'   \item "Nhlf" = Normal Human Lung Fibroblasts
#' }
#' @param chipseeker_plot Default FALSE. If TRUE, the report includes a barplot
#' of ChIPseeker annotation of peak files.
#' @param enrichment_plot Default FALSE. If TRUE, the report includes dotplots
#' of KEGG and GO enrichment analysis of peak files.
#' @param tss_plot Default FALSE. If TRUE, the report includes peak count
#' frequency around transcriptional start site. Note that this can take awhile.
#' @param interact Default TRUE. By default, plots are interactive.
#' If set FALSE, all plots in the report will be static.
#' @param add_download_button Add download buttons for each plot or dataset.
#' @param save_output Default FALSE. If TRUE, all outputs (tables and plots) of
#' the analysis will be saved in a folder (EpiCompare_file).
#' @param output_filename Default EpiCompare.html. If otherwise, the html report
#'  will be saved in the specified name.
#' @param output_timestamp Default FALSE. If TRUE, date will be included in the
#' file name.
#' @param output_dir Path to where output HTML file should be saved.
#' @param display After completion, automatically display the HTML report file
#'  in one of the following ways:
#' \itemize{
#' \item{"browser" : }{Display the report in your default web browser.}
#' \item{"rsstudio" : }{Display the report in Rstudio.}
#' \item{NULL (default) : }{Do not display the report.}
#' }
#' @param run_all Convenience argument that enables all plots/features
#' (without specifying each argument manually)
#'  by overriding the default values.
#'  Default: \code{FALSE}.
#' @param error If \code{TRUE}, the Rmarkdown report will continue to render
#' even when some chunks encounter errors (default: \code{FALSE}).
#' Passed to \link[knitr]{opts_chunk}.
#' @param debug Run in debug mode, where are messages and warnings
#' are printed within the HTML report (default: \code{FALSE}).
#' @inheritParams plot_precision_recall
#' @inheritParams plot_corr
#' @inheritParams tss_plot
#' @inheritParams check_workers
#' @inheritParams rmarkdown::render
#' @return Path to one or more HTML report files.
#'
#' @export
#' @importFrom rmarkdown render
#' @importFrom methods show is
#' @importFrom utils browseURL packageVersion
#'
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example dataset as GRanges object
#' data("CnT_H3K27ac") # example dataset as GRanges object
#' data("CnR_H3K27ac") # example dataset as GRanges object
#' data("CnT_H3K27ac_picard") # example Picard summary output
#' data("CnR_H3K27ac_picard") # example Picard summary output
#'
#' #### Prepare Input ####
#' # create named list of peakfiles
#' peakfiles <- list(CnR=CnR_H3K27ac, CnT=CnT_H3K27ac)
#' # create named list of picard outputs
#' picard_files <- list(CnR=CnR_H3K27ac_picard, CnT=CnT_H3K27ac_picard)
#' # reference peak file
#' reference <- list("ENCODE" = encode_H3K27ac)
#'
#' ### Run EpiCompare ###
#' output_html <- EpiCompare(peakfiles = peakfiles,
#'            genome_build = list(peakfiles="hg19",
#'                                reference="hg19"),
#'            picard_files = picard_files,
#'            reference = reference,
#'            output_filename = "EpiCompare_test",
#'            output_dir = tempdir())
#' # utils::browseURL(output_html)
EpiCompare <- function(peakfiles,
                       genome_build,
                       genome_build_output = "hg19",
                       blacklist = NULL,
                       picard_files = NULL,
                       reference = NULL,
                       upset_plot = FALSE,
                       stat_plot = FALSE,
                       chromHMM_plot = FALSE,
                       chromHMM_annotation = "K562",
                       chipseeker_plot = FALSE,
                       enrichment_plot = FALSE,
                       tss_plot = FALSE,
                       tss_distance = c(-3000, 3000),
                       precision_recall_plot = FALSE,
                       n_threshold = 20,
                       corr_plot = FALSE,
                       bin_size = 5000,
                       interact = TRUE,
                       add_download_button = FALSE,
                       save_output = FALSE,
                       output_filename = "EpiCompare",
                       output_timestamp = FALSE,
                       output_dir,
                       display = NULL,
                       run_all = FALSE,
                       workers = 1,
                       quiet = FALSE,
                       error = FALSE,
                       debug = FALSE) {
  # devoptera::args2vars(EpiCompare)
  #### time ####
  t1 <- Sys.time()
  #### Check that essential args are not missing ####
  force(output_dir)
  force(genome_build)
  #### Set all args to true ####
  if (isTRUE(run_all)) {
    upset_plot <- stat_plot <- chromHMM_plot <-
      chipseeker_plot <- enrichment_plot <- tss_plot <-
      precision_recall_plot <- corr_plot <- TRUE
    
    save_output <- TRUE
    
    if (is.null(output_dir))
      output_dir <- tempdir()
  }
  #### Check for ComplexUpset compatibility with ggplot 4.4.0 ####
  if (packageVersion("ggplot2") >= "4.0.0" &&
      packageVersion("ComplexUpset") <= "1.3.6" &&
      isTRUE(upset_plot)) {
    messager(
      "NOTE: The installed version of ComplexUpset package is not yet",
      "compatible with ggplot2 >= v4.0.0.",
      "Please downgrade to ggplot2 v3 to use this feature.",
      "EpiCompare will proceed without generating upset plot. \n "
    )
    upset_plot <- FALSE
  }
  #### Report which features are NOT being used ####
  check_unused_args(
    upset_plot = upset_plot,
    stat_plot = stat_plot,
    chromHMM_plot = chromHMM_plot,
    chipseeker_plot = chipseeker_plot,
    enrichment_plot = enrichment_plot,
    tss_plot = tss_plot,
    precision_recall_plot = precision_recall_plot,
    corr_plot = corr_plot,
    add_download_button = add_download_button
  )
  #### Display HTML after it's been rendered ####
  if (!is.null(display))
    display <- tolower(display)[1]
  ### Output Filename ###
  if (isTRUE(output_timestamp)) {
    date <- format(Sys.Date(), '%b_%d_%Y')
    output_filename <- paste0(output_filename, "_", date)
  }
  #### Check args ####
  if (is.null(reference)) {
    if (isTRUE(precision_recall_plot)) {
      messager("WARNING:",
               "precision-recall curves cannot be generated when reference=NULL.")
    }
  }
  ### Parse Parameters Into Markdown & Render HTML ###
  html_file <- paste0(output_filename, ".html")
  ### Locate Rmd ###
  markdown_path <- system.file("markdown", "EpiCompare.Rmd", package = "EpiCompare")
  ### Multiple Reference Files ###
  if (methods::is(reference, "list") &&
      length(reference) > 1) {
    output_html <- lapply(names(reference), function(nm) {
      message("\n", "======>> ", nm, " <<======")
      #### Create subfolder for each run ####
      output_dir <- file.path(output_dir, nm)
      dir.create(output_dir,
                 showWarnings = FALSE,
                 recursive = TRUE)
      #### Parse Parameters Into Markdown & Render HTML ####
      rmarkdown::render(
        input = markdown_path,
        output_dir = output_dir,
        output_file = output_filename,
        quiet = quiet,
        params = list(
          peakfiles = peakfiles,
          genome_build = genome_build,
          genome_build_output = genome_build_output,
          blacklist = blacklist,
          picard_files = picard_files,
          reference = reference[nm],
          upset_plot = upset_plot,
          stat_plot = stat_plot,
          chromHMM_plot = chromHMM_plot,
          chromHMM_annotation = chromHMM_annotation,
          chipseeker_plot = chipseeker_plot,
          enrichment_plot = enrichment_plot,
          tss_plot = tss_plot,
          tss_distance = tss_distance,
          precision_recall_plot = precision_recall_plot,
          n_threshold = n_threshold,
          corr_plot = corr_plot,
          bin_size = bin_size,
          interact = interact,
          add_download_button = add_download_button,
          save_output = save_output,
          output_dir = output_dir,
          workers = workers,
          error = error,
          debug = debug
        )
      )
      return(file.path(output_dir, html_file))
    }) |> unlist()
  } else{
    dir.create(output_dir,
               showWarnings = FALSE,
               recursive = TRUE)
    #### Parse Parameters Into Markdown & Render HTML ####
    rmarkdown::render(
      input = markdown_path,
      output_dir = output_dir,
      output_file = output_filename,
      quiet = quiet,
      params = list(
        peakfiles = peakfiles,
        genome_build = genome_build,
        genome_build_output = genome_build_output,
        blacklist = blacklist,
        picard_files = picard_files,
        reference = reference,
        upset_plot = upset_plot,
        stat_plot = stat_plot,
        chromHMM_plot = chromHMM_plot,
        chromHMM_annotation = chromHMM_annotation,
        chipseeker_plot = chipseeker_plot,
        enrichment_plot = enrichment_plot,
        tss_plot = tss_plot,
        tss_distance = tss_distance,
        precision_recall_plot = precision_recall_plot,
        n_threshold = n_threshold,
        corr_plot = corr_plot,
        bin_size = bin_size,
        interact = interact,
        add_download_button = add_download_button,
        save_output = save_output,
        output_dir = output_dir,
        workers = workers,
        error = error,
        debug = debug
      )
    )
    output_html <- file.path(output_dir, html_file)
  }
  ### Show Timer ###
  t2 <- Sys.time()
  methods::show(paste("Done in", round(difftime(t2, t1, units = "min"), 2), "min."))
  ### Display results ###
  messager("All outputs saved to:", output_dir)
  #### Return path only ####
  if (is.null(display)) {
    return(output_html)
    #### Launch in web browser ####
  } else if (display == "browser") {
    for (x in output_html) {
      utils::browseURL(x)
    }
    #### Launch in Rstudio ####
  } else if (display == "rstudio") {
    for (x in output_html) {
      file.show(x)
    }
  }
  #### Return paths to html reports ####
  return(output_html)
}

testthat::test_that("EpiCompare works",{
  #### (run_all=T, interact=T, reference=NULL) ####
  {
    data("encode_H3K27ac")
    data("CnT_H3K27ac")
    data("CnR_H3K27ac")
    data("CnT_H3K27ac_picard")
    data("CnR_H3K27ac_picard")
    # create named peaklist, reference and picard list
    peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
    reference <- list("ENCODE"=encode_H3K27ac)
    picard_list <- list("CnT"=CnT_H3K27ac_picard, "CnR"=CnR_H3K27ac_picard)
  }
  
  outpath <- file.path(tempdir(),"t1")
  html_file <- EpiCompare::EpiCompare(peakfiles = peaklist,
                                      genome_build = "hg19", 
                                      picard_files = picard_list,
                                      reference = NULL,
                                      interact = TRUE,
                                      tss_distance = c(-50,50),
                                      run_all = TRUE,
                                      output_dir = outpath)
  # browseURL(html_file)
  files <- list.files(file.path(outpath,"EpiCompare_file"))
  testthat::expect_gte(length(files), 10)
  testthat::expect_true(is.element("peak_info.csv", files))
  testthat::expect_true(is.element("fragment_info.csv", files))
  testthat::expect_true(is.element("width_plot.html", files))
  testthat::expect_true(is.element("samples_percent_overlap.html", files))
  ## save_output() throws an error for ComplexUpset. Skip saving for now. 
  # testthat::expect_true(is.element("upset_plot.png", files))
  testthat::expect_false(is.element("stat_plot.png", files))
  testthat::expect_true(is.element("samples_ChromHMM.html", files))
  testthat::expect_false(is.element("sample_in_ref_ChromHMM.html", files))
  testthat::expect_false(is.element("ref_in_sample_ChromHMM.html", files))
  testthat::expect_false(is.element("ref_not_in_sample_ChromHMM.html", files))
  testthat::expect_false(is.element("sample_not_in_ref_ChromHMM.html", files))
  testthat::expect_true(is.element("chipseeker_annotation.html", files))
  testthat::expect_true(is.element("KEGG_analysis.html", files))
  testthat::expect_true(is.element("GO_analysis.html", files))

  #### All options FALSE, correct outputs generated ####
  # outpath <- file.path(tempdir(),"t2")
  # html_file <- EpiCompare::EpiCompare(peakfiles = peaklist,
  #                                      genome_build = "hg19", 
  #                                      picard_files = picard_list,
  #                                      reference = NULL,
  #                                      upset_plot = FALSE,
  #                                      stat_plot = FALSE,
  #                                      chromHMM_plot = FALSE,
  #                                      chipseeker_plot = FALSE,
  #                                      enrichment_plot = FALSE,
  #                                      tss_plot = FALSE,
  #                                      interact = FALSE,
  #                                      save_output = TRUE,
  #                                      output_dir = outpath)
  # 
  # files <- list.files(file.path(outpath,"EpiCompare_file"))
  # testthat::expect_gte(length(files), 4)
  # testthat::expect_true(is.element("peak_info.csv", files))
  # testthat::expect_true(is.element("fragment_info.csv", files))
  # testthat::expect_true(is.element("processed_peakfiles_hg19", files))
})

data("encode_H3K27ac")
data("CnT_H3K27ac")
data("CnR_H3K27ac")
data("hg19_blacklist")
# create peaklist
peaklist <- list(CnT_H3K27ac, CnR_H3K27ac)

testthat::test_that("outputs are saved in EpiCompare_file", {
  outpath <- file.path(tempdir(),"t1") 
  EpiCompare::EpiCompare(peakfiles = peaklist,
                         genome_build = "hg19",
                         blacklist = hg19_blacklist,
                         save_output = TRUE,
                        output_dir = outpath)
  files <- list.files(file.path(outpath,"EpiCompare_file"))
  testthat::expect_true(length(files)>1)
  testthat::expect_true(is.element("peak_info.csv", files))
  testthat::expect_true(file.exists(file.path(outpath,"EpiCompare.html")))
})

test_that("outputs are saved in EpiCompare_file", {
  outpath <- file.path(tempdir(),"t2") 
  EpiCompare::EpiCompare(peakfiles = peaklist,
                         genome_build = "hg19",
                         blacklist = hg19_blacklist,
                         save_output = FALSE,
                         output_dir = outpath )
  testthat::expect_true(file.exists(file.path(outpath,"EpiCompare.html")))
})

test_that("output filename set by users",{
  outpath <- file.path(tempdir(),"t3") 
  EpiCompare::EpiCompare(peakfiles = peaklist,
                         genome_build = "hg19",
                         blacklist = hg19_blacklist,
                         save_output = TRUE,
                         output_dir = outpath,
                         output_filename = "testthat_example",
                         output_timestamp = TRUE)
  date <- format(Sys.Date(), '%b_%d_%Y')
  name <- "testthat_example"
  filename <- paste0(name,"_",date)
  files <- list.files(outpath)
  testthat::expect_true(file.exists(file.path(outpath,paste0(filename,".html"))))
})

filter_percentiles <- function(gr, 
                               thresh,
                               thresholding_cols){
    # message("Filtering percentiles.")
    for(nm in paste(thresholding_cols,"percentile",sep="_")){
        if(nm %in% names(GenomicRanges::mcols(gr))){
            gr <- gr[GenomicRanges::mcols(gr)[[nm]]>=thresh,]
        }
    }
    return(gr)
}

#' Write example peaks
#'
#' Write example peaks datasets to disk.
#'
#' @param dir Directory to save peak files to.
#' @param datasets Example datasets from \pkg{EpiCompare} to write.
#'
#' @returns Named vector of paths to saved peak files.
#'
#' @export 
#' @importFrom utils data
#' @examples
#' save_paths <- EpiCompare::write_example_peaks()
write_example_peaks <- function(dir = file.path(tempdir(),
                                                "processed_results"),
                                datasets = c("encode_H3K27ac",
                                             "CnT_H3K27ac",
                                             "CnR_H3K27ac")){
  check_dep("plyranges")
  dir.create(dir, showWarnings = FALSE, recursive = TRUE)
  save_paths <- vapply(datasets, function(x){
    save_path <- file.path(dir,paste0(x,".narrowPeaks.bed"))
    message("Writing ==> ",save_path)
    #save in bed format - rtracklayer doesn't work for some reason
    #use get(x) to get the dataset from the package named char stored in x
    #rtracklayer::export.bed(get(x),con=save_path)
    #rtracklayer::export(get(x),con=save_path,format="narrowPeak",
    #                    extraCols = extraCols_narrowPeak)
    utils::data(list = x, package = "EpiCompare")
    plyranges::write_narrowpeaks(get(x),file=save_path)
    return(save_path)
  }, character(1))
  names(save_paths) <- datasets
  return(save_paths)
}

test_that("import_narrowPeak works", {

  path <- EpiCompare::write_example_peaks(datasets="encode_H3K27ac")
  peaks <- rtracklayer::import(path, format = "narrowPeak")
  testthat::expect_true(methods::is(peaks,"GRanges"))
  testthat::expect_length(peaks,5142)
  testthat::expect_equal(ncol(GenomicRanges::mcols(peaks)),6)
})

test_that("gather_files works", {

  #### Make example files ####
  save_paths <- EpiCompare::write_example_peaks()
  dir <- unique(dirname(save_paths))
  datasets <- eval(formals(write_example_peaks)$datasets)
  testthat::expect_equal(length(save_paths), length(datasets))

  #### Import files ####
  peaks <- EpiCompare::gather_files(dir=dir, 
                                    type="*.narrowPeaks.bed$",
                                    workers = 1)
  testthat::expect_equal(length(peaks), length(datasets)) 
  for(peak in peaks){
    testthat::expect_true(methods::is(peak,'GRanges'))
  }
  
  #### Gather files ####
  peaks2 <- EpiCompare::gather_files(dir=dir, 
                                     type="peaks.narrow",
                                     return_paths = TRUE,
                                     workers = 1)
  testthat::expect_equal(length(peaks2), length(datasets)) 
  for(peak in peaks2){
      testthat::expect_true(file.exists(peak))
  }
  
  #### Gather NO files ####
  peaks3 <- EpiCompare::gather_files(dir="./", 
                                     type="peaks.narrow",
                                     return_paths = TRUE,
                                     workers = 1)
  testthat::expect_null(peaks3)
})

#' Peak width boxplot
#'
#' This function creates boxplots showing the distribution of widths in
#' each peak file.
#'
#' @param peaklist A list of peak files as GRanges object.
#' Files must be named and listed using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}
#' @inheritParams EpiCompare
#' @return A boxplot of peak widths.
#'
#' @importMethodsFrom GenomicRanges width
#' @importFrom ggplot2 ggplot 
#' @importFrom data.table data.table rbindlist
#'
#' @export
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peaklist GRanges object
#' data("CnT_H3K27ac") # example peaklist GRanges object  
#' peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)  
#' my_plot <- width_boxplot(peaklist = peaklist) 
width_boxplot <- function(peaklist,
                          interact = FALSE){
  
  width <- NULL;
  message("--- Running width_boxplot() ---")
  #### Check Peaklist Names ####
  peaklist <- check_list_names(peaklist) 
  #### Obtain widths, create data.table #### 
  df <- lapply(seq_len(length(peaklist)), function(i){
    sample <- names(peaklist)[i]
    width <- GenomicRanges::width(peaklist[[i]])
    data.table::data.table(sample, width, name=peaklist[[i]]$name)
  }) |> data.table::rbindlist(fill=TRUE)
  #### Create Boxplot ####
  boxplot <- ggplot2::ggplot(df, ggplot2::aes(x = sample, 
                                              y = width, 
                                              fill = sample)) +
             ggplot2::geom_boxplot(outlier.shape = NA, 
                                   na.rm = TRUE) +
             ggplot2::scale_y_continuous(
               trans="log10",
               limits = quantile(df$width,c(0.1, 0.9))) +
             # ggplot2::scale_fill_viridis_d(alpha = .9, option = "mako") +
             ggplot2::labs(x="",y="width (bp)") +
             ggplot2::coord_flip() +
             ggplot2::theme_bw() +
             ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 315,
                                                                vjust = 0.5,
                                                                hjust = 0)) 
  message("Done.")
  if(isTRUE(interact)){ 
    boxplot <- as_interactive(boxplot) 
  }
  #### Return ####
  return(list(plot=boxplot,
              data=df)) 
}

test_that("width_boxplot works",{
  
  data("encode_H3K27ac") 
  peaklist <- list("encode"=encode_H3K27ac)
  
  p <- EpiCompare::width_boxplot(peaklist)
  testthat::expect_true(methods::is(p$plot,"ggplot"))
  testthat::expect_true(methods::is(p$data,"data.frame"))
})

#' Clean GRanges
#' 
#' Remove columns from the metadata (\code{GenomicRanges::mcols}) that
#' conflicts with \link[GenomicRanges]{GRanges} conventions.
#' 
#' @param gr A \link[GenomicRanges]{GRanges} object.
#' @param nono_cols Problematic columns to search for and remove (if present).
#' @keywords internal
#' @importFrom GenomicRanges mcols
#' @returns Cleaned \link[GenomicRanges]{GRanges} object. 
clean_granges <- function(gr,
                          nono_cols = c("seqnames", 
                                         "ranges",
                                         "strand",
                                         "seqlevels", 
                                         "seqlengths", 
                                         "isCircular",
                                         "start", 
                                         "end", 
                                         "width", 
                                         "element")){  
    cnames <- colnames(GenomicRanges::mcols(gr))
    rm_cols <- cnames[cnames %in% nono_cols]
    if(length(rm_cols)>0){
        for(rcol in rm_cols){
            GenomicRanges::mcols(gr)[rcol] <- NULL
        } 
    }
    return(gr)
}

#' Read peaks
#' 
#' Read peak files.
#' @param path Path to peak file.
#' @param verbose Print messages.
#' @inheritParams gather_files
#' @return \link[GenomicRanges]{GRanges}
#' 
#' @keywords internal
#' @importFrom methods is
#' @importFrom GenomicRanges makeGRangesFromDataFrame mcols
#' @importFrom rtracklayer import
#' @importFrom data.table fread 
#' @importFrom ChIPseeker readPeakFile
read_peaks <- function(path,
                       type,
                       verbose=TRUE){
    
   
    if(startsWith(type,"peaks")){
        messager("Reading peaks.",v=verbose)
        dat <- ChIPseeker::readPeakFile(path, as = "GRanges")
        if(type=="peaks.stringent" && 
           (ncol(GenomicRanges::mcols(dat))==3)){
            ## Colnames are not included in SEACR output,
            ## but can be found in the SEACR documentation.
            colnames(GenomicRanges::mcols(dat))  <- c("total_signal",
                                                      "max_signal",
                                                      "max_signal_region")
        }
    } else if(grepl("narrowPeak",path,ignore.case = TRUE)){
        messager("Reading narrowPeaks",v=verbose)
        dat <- rtracklayer::import(path, 
                                   format = "narrowPeak")
    } else {
        messager("Reading peaks with data.table.",v=verbose)
        dat <- data.table::fread(path)
    }
    #### Ensure GRanges format ####
    if(length(dat)==0){
        messager("WARNING: File contains 0 rows.",v=verbose)
        return(NULL)
    }
    if(!methods::is(dat,"GRanges")) {
        ## Try to convert to GRanges, but if it fails,
        ## just return the data.table
        dat <- tryCatch({
            GenomicRanges::makeGRangesFromDataFrame(
                df = dat,
                keep.extra.columns = TRUE)
        }, error = function(e) {message(e); NULL}) 
    } 
    dat@metadata <- list(path=path)
    return(dat)
}

#' Stop messages
#' 
#' Conditionally print stop messages.
#' Allows developers to easily control verbosity of functions, 
#'  and meet Bioconductor requirements that dictate the stop message 
#'  must first be stored to a variable before passing to \link[base]{stop}. 
#' @param v Whether to print messages or not.
#' 
#' @return Null 
#' @keywords internal 
stopper <- function(..., v = TRUE) {
    msg <- paste(...)
    if (v) {
        stop(msg)
    } else {
        stop()
    }
}

#' Wrapper for \link[BiocParallel]{bplapply}
#' 
#' Wrapper function for \link[BiocParallel]{bplapply} that automatically
#' handles issues with \pkg{BiocParallel} related to different OS platforms.
#' @param apply_fun Iterator function to use. 
#' @param verbose Print messages. 
#' @inheritParams get_bpparam 
#' @inheritParams BiocParallel::bplapply
#' @inheritDotParams BiocParallel::bplapply
#' @returns (Named) list.
#' 
#' @export
#' @examples
#' X <- stats::setNames(seq_len(length(letters)), letters)
#' out <- bpplapply(X, print) 
bpplapply <- function(X, 
                      FUN, 
                      apply_fun=parallel::mclapply,
                      workers=check_workers(), 
                      progressbar=workers>1,
                      verbose=workers==1,
                      use_snowparam=TRUE,
                      register_now=FALSE,
                      ...){
    
    #### Select method ####
    if(any(attr(apply_fun,"package")=="BiocParallel")){
        check_dep("BiocParallel")
        BPPARAM <- get_bpparam(workers = workers,
                               progressbar = progressbar,
                               use_snowparam = use_snowparam,
                               register_now = register_now)
        if(isFALSE(verbose)) FUN <- function(FUN){suppressMessages(FUN)}
        apply_fun(X = X, 
                  FUN = FUN, 
                  BPPARAM = BPPARAM, 
                  ...)
    } else {
        if(environmentName(environment(apply_fun))=="parallel"){
            check_dep("parallel")
        }
        if(isFALSE(verbose)) FUN <- function(FUN){suppressMessages(FUN)}
        apply_fun(FUN = FUN,  
                  X,
                  ...)
    } 
}

test_that("bpplapply works", { 
    
    X <- stats::setNames(seq_len(length(letters)), letters)
    
    #### BiocParallel ####
    out1 <- bpplapply(X, print, workers = 1) 
    testthat::expect_true(is.list(out1))
    testthat::expect_equal(names(out1), names(X)) 
    
    #### BiocParallel ####
    out2 <- bpplapply(X, print, workers = 2, use_snowparam = FALSE) 
    testthat::expect_true(is.list(out2))
    testthat::expect_equal(names(out2), names(out2))
    
    #### lapply ####
    out3 <- bpplapply(X, print, workers = 1, apply_fun = lapply) 
    testthat::expect_true(is.list(out3))
    testthat::expect_equal(names(out3), names(X))
    
    #### mapply ####
    out4 <- bpplapply(X, print, workers = 1, apply_fun = mapply) 
    testthat::expect_true(is.vector(out4))
    testthat::expect_equal(names(out4), names(X))
})

remove_empty_elements <- function(peaklist){
    is_empty <- unlist(lapply(peaklist, function(x){length(x)==0}))
    if(sum(is_empty)>0) {
        message("Removing ",sum(is_empty)," empty elements in peaklist")
        peaklist <- peaklist[!is_empty] 
    } 
    return(peaklist)
}

#' Save output
#'
#' This function saves data frames and plots generated by EpiCompare.
#'
#' @param save_output Default FALSE. If TRUE, outputs are saved.
#' @param file Tables and plots to be saved.
#' @param file_type Type of file to be saved.
#' "data.frame", "ggplot", "image"
#' @param filename Name of file.
#' @param outpath Outpath
#' @param interactive Default FALSE. If TRUE, interactive plots are saved as
#' html.
#' @param verbose Print messages.
#'
#' @importFrom htmltools save_html
#' @importFrom data.table fwrite
#' @importFrom ggplot2 ggsave
#'
#' @return Saved data frames and plots.
#' @keywords internal 
save_output <- function(save_output = FALSE,
                        file,
                        file_type,
                        filename,
                        outpath,
                        interactive = FALSE,
                        verbose = TRUE){

  if(isTRUE(save_output)){
    dir.create(outpath, showWarnings = FALSE, recursive = TRUE)
    if(file_type == "data.frame"){
      messager("Saving CSV file ==>",file,v=verbose)
      data.table::fwrite(x = file,
                         file = file.path(
                           outpath,
                           paste0(gsub("\\.txt|\\.csv","",basename(filename)),
                                  ".csv")),
                         sep = "\t")
    }else if(file_type == "ggplot"){ 
      ## Save interactive HTML
      if(isTRUE(interactive)){
        messager("Saving HTML  ==>",file,v=verbose)
        htmltools::save_html(html = file,
                             file = file.path(
                               outpath,paste0(basename(filename),".html")
                               )
                             )
      ## Save static PNG
      } else{
        messager("Saving PNG file ==>",file,v=verbose) 
        ggplot2::ggsave(filename = file.path(outpath,
                                             paste0(basename(filename),".png")
                                             ),
                        plot = file,
                        device = "png",
                        path = outpath)
        # unlink(paste0(filename,".png"))
      }
    }else if(file_type == "image"){
      messager("Saving PNG file ==>",file,v=verbose)
      check_dep("grDevices")
      grDevices::png(file.path(outpath, paste0(basename(filename),".png")))
      file
      grDevices::dev.off()
    }
  }
}

#' Print messages 
#' 
#' Conditionally print messages.
#'  Allows developers to easily control verbosity of functions, 
#'  and meet Bioconductor requirements that dictate the message 
#'  must first be stored to a variable before passing to \link[base]{message}. 
#'  
#' 
#' @param v Whether to print messages or not.
#' @param parallel Whether to enable message print when wrapped 
#' in parallelised functions.
#' 
#' @return Null 
#' @keywords internal 
messager <- function(..., v = TRUE, parallel = FALSE) {
    if(parallel){
        if(v) try({message_parallel(...)})
    } else {
        msg <- paste(...)
        if (v) try({message(msg)})
    }
}

#' Liftover peak list
#' 
#' Perform genome build liftover to one or more
#'  \link[GenomicRanges]{GRanges} objects at once.
#' @param grlist A named list of \link[GenomicRanges]{GRanges} objects,
#'  or simply a single unlisted \link[GenomicRanges]{GRanges} object.
#'  Can perform liftover within species or across species.
#' @param input_build The genome build of \code{grlist}.
#' @param output_build Desired genome build for
#'  \code{grlist} to be lifted over to.
#' @param as_grangeslist Return as a \link[GenomicRanges]{GRangesList}. 
#' @param merge_all Merge all \link[GenomicRanges]{GRanges} into a single
#'  \link[GenomicRanges]{GRanges} object.
#' @param style Chromosome style,
#'  set by \link[GenomeInfoDb]{seqlevelsStyle}.
#'  \itemize{
#'  \item{"UCSC" : }{Uses the chromosome style "chr1".}
#'  \item{"NCBI" : }{Uses the chromosome style "1"}
#'  }
#' @param verbose Print messages.
#' @inheritParams remove_nonstandard_chrom
#' 
#' @returns Named list of lifted \link[GenomicRanges]{GRanges} objects.
#' @export
#' @importFrom AnnotationHub AnnotationHub
#' @importFrom rtracklayer liftOver
#' @importFrom GenomicRanges GRangesList
#' @examples 
#' grlist <- list("gr1"=GenomicRanges::GRanges("4:1-100000"),
#'                "gr2"=GenomicRanges::GRanges("6:1-100000"),
#'                "gr3"=GenomicRanges::GRanges("8:1-100000"))
#'
#' grlist_lifted <- liftover_grlist(grlist = grlist,
#'                                  input_build = "hg19",
#'                                  output_build="hg38")
liftover_grlist <- function(grlist,
                            input_build,
                            output_build="hg19",
                            style="UCSC",
                            keep_chr = paste0(
                                "chr",c(seq_len(22),"X","Y")
                            ),
                            as_grangeslist=FALSE,
                            merge_all=FALSE,
                            verbose=TRUE){ 
    
    input_build <- translate_genome(genome = input_build, 
                                    style = "UCSC")
    output_build <- translate_genome(genome = output_build, 
                                     style = "UCSC")
    #### No liftover necessary ####
    if(input_build==output_build){
        messager("grlist is already in the output_build format.",
                 "Skipping liftover.",v=verbose)
      if(isTRUE(as_grangeslist)){
        messager("Converting to GRangesList.",v=verbose)
        grlist <- GenomicRanges::GRangesList(grlist,
                                             compress = FALSE) 
      }
        return(grlist)
    } 
    #### Check if it's a single element ####
    tmp_list <- FALSE
    if(is_granges(grlist)){
        grlist <- list(gr1=grlist)
        tmp_list <- TRUE
    }
    #### Chain files ####
    grlist_chain <- get_chain_file(grlist = grlist,
                                   input_build = input_build, 
                                   output_build = output_build, 
                                   verbose = verbose)
    grlist <- grlist_chain$grlist
    chain <- grlist_chain$chain
    #### Liftover iteratively ####
    grlist_lifted <- mapply(grlist, FUN = function(gr){
        gr <- clean_granges(gr = gr)
        suppressMessages(suppressWarnings(
            GenomeInfoDb::seqlevelsStyle(gr) <- "UCSC"
        ))
        gr2 <- rtracklayer::liftOver(x = gr, 
                                     chain = chain) 
        suppressMessages(suppressWarnings(
            GenomeInfoDb::seqlevelsStyle(gr) <- style
        ))
        return(unlist(gr2))
    })    
    #### Clean up chroms ####
    grlist_lifted <- remove_nonstandard_chrom(grlist = grlist_lifted, 
                                              keep_chr = keep_chr,
                                              verbose = verbose)
    #### Merge list into one ####
    if(isTRUE(merge_all)) as_grangeslist <- TRUE
    if(isTRUE(as_grangeslist)){
        messager("Converting to GRangesList.",v=verbose)
        grlist_lifted <- GenomicRanges::GRangesList(grlist_lifted, 
                                                    compress = FALSE) 
    }
    if(isTRUE(merge_all)){
        messager("Merging all GRanges into one.",v=verbose)
        grlist_lifted <- unlist(grlist_lifted)
    }
    if(isTRUE(tmp_list)) grlist_lifted <- grlist_lifted[[1]]
    return(grlist_lifted)
}

test_that("liftover_grlist works", {
  
    data("encode_H3K27ac") # example dataset as GRanges object
    data("CnT_H3K27ac") # example dataset as GRanges object
    data("CnR_H3K27ac") # example dataset as GRanges object
    grlist <- list("encode_H3K27ac"=encode_H3K27ac,
                   "CnT_H3K27ac"=CnT_H3K27ac,
                   "CnR_H3K27ac"=CnR_H3K27ac)

    grlist_lifted <- liftover_grlist(grlist = grlist,
                                     input_build = "hg19",
                                     output_build="hg38")
    testthat::expect_length(grlist_lifted,length(grlist))
    testthat::expect_true(all(unlist(lapply(grlist_lifted,
                                            methods::is,"GRanges"))))
    
    nm <- names(grlist)[1]
    gr1 <- grlist[[nm]] 
    gr2 <- grlist_lifted[[nm]] 
    testthat::expect_gt(length(gr2), length(gr1))
    
    
    #### hg38 to mm9 #### 
    grlist_lifted2 <- liftover_grlist(grlist = grlist_lifted,
                                     input_build = "hg38",
                                     output_build = "mm9")
    nm <- names(grlist)[1]
    gr1 <- grlist[[nm]] 
    gr2 <- grlist_lifted2[[nm]] 
    testthat::expect_gt(length(gr2), length(gr1))
})

#' Dynamic Figure Length Generator
#'
#' This function calculates the appropriate figure height depending on the
#' number of items.
#'
#' @param default_size The default figure length. Must be numeric.
#' @param number_of_items Number of peak files, or terms.
#' @param max_items Maximum number of peak files, or terms.
#'
#' @return Figure height/width. A number.
#' @keywords internal

fig_length <- function(default_size, number_of_items ,max_items){
  if(number_of_items > max_items){
    extra_N <- number_of_items - max_items
    length <- default_size + extra_N*0.25
  }else{
    length <- default_size
  }
  return(length)
}

#' Generate heatmap of percentage overlap
#'
#' This function generates a heatmap showing percentage of overlapping peaks
#' between peak files.
#'
#' @param peaklist A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names will be assigned.
#' @param interact Default TRUE. By default heatmap is interactive.
#' If FALSE, heatmap is static.
#' @param draw_cellnote Draw the numeric values within each heatmap cell.
#' @param verbose Print messages.
#' @inheritParams precision_recall_matrix
#' @returns An interactive heatmap
#'
#' @import ggplot2
#' @importFrom reshape2 melt
#' @importFrom plotly plot_ly
#'
#' @export
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peakfile GRanges object
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' ### Create Named List ###
#' peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)
#' ### Run ###
#' my_heatmap <- overlap_heatmap(peaklist = peaklist) 
overlap_heatmap <- function(peaklist,
                            interact=TRUE,
                            draw_cellnote=TRUE,
                            fill_diag=NA,
                            verbose=TRUE){

  ### Variables ###
  Var1 <- Var2  <- value <- NULL;
  t1 <- Sys.time()
  messager("--- Running overlap_heatmap() ---",v=verbose) 
  #### Check deps ####
  fill_diag <- check_heatmap_args(draw_cellnote = draw_cellnote, 
                                  interact = interact, 
                                  fill_diag = fill_diag, 
                                  verbose = verbose)
  ### Check Peaklist Names ###
  peaklist <- check_list_names(peaklist) 
  ### Calculate Overlap Percentage ### 
  overlap_matrix <- precision_recall_matrix(peaklist = peaklist,
                                            fill_diag = fill_diag,
                                            verbose = verbose)
  ### Create Static Heatmap ###
  if(isFALSE(interact)){
    mtx_melt <- reshape2::melt(overlap_matrix)
    plt <- ggplot2::ggplot(
        data = mtx_melt,
        ggplot2::aes(x=Var1, y=Var2,
                     fill=value, 
                     label=round(value,2))) +
        ggplot2::geom_tile() + 
        ggplot2::labs(x="Recall", y="Precision",
                      fill=gsub(" ","\n","% overlap")) +
        ggplot2::scale_fill_viridis_c(na.value = "transparent") +
        ggplot2::theme_bw()
    if(isTRUE(draw_cellnote)){ 
        plt <- plt + ggplot2::geom_label(fill=ggplot2::alpha("white",.8), 
                                         linewidth = NA,
                                         na.rm = TRUE)
    }
  } else{
      ### Create Interactive Heatmap ###
      if(isTRUE(draw_cellnote)){
        #### heatmaply ####
        plt <- heatmap_heatmaply(X=overlap_matrix)
      } else {
        #### plotly #### 
        plt <- heatmap_plotly(X=overlap_matrix)
      }  
  }
  report_time(t1 = t1,
              func="overlap_heatmap",
              verbose = verbose)
  #### Return ####
  return(list(plot=plt,
              data=overlap_matrix))
}

test_that("overlap_heatmap works", {
                      
  data("encode_H3K27ac")
  data("CnT_H3K27ac")
  peaklist <- list("encode"=encode_H3K27ac,
                   "CnT"=CnT_H3K27ac)
  
  p_interact <- EpiCompare::overlap_heatmap(peaklist = peaklist, 
                                            interact = TRUE)
  testthat::expect_true(methods::is(p_interact$plot,"plotly"))
  
  p <- EpiCompare::overlap_heatmap(peaklist = peaklist, 
                                   interact = FALSE)
  testthat::expect_true(methods::is(p$plot,"ggplot"))
}) 

#' Plot precision-recall curves
#'
#' Plot precision-recall curves (and optionally F1 plots) by
#' iteratively testing for peak overlap across a series of
#' thresholds used to filter \code{peakfiles}.
#' Each \link[GenomicRanges]{GRanges}
#'  object in \code{peakfiles} will be used as the "query"
#'  against each \link[GenomicRanges]{GRanges} object in \code{reference}
#'  as the subject.
#'  Will automatically use any columns that are
#'  specified with \code{thresholding_cols} and present within each
#'  \link[GenomicRanges]{GRanges} object
#'  to create percentiles  for thresholding.
#' \emph{NOTE} : Assumes that all \link[GenomicRanges]{GRanges} in
#' \code{peakfiles} and \code{reference} are already
#' aligned to the same genome build.
#' @param show_plot Show the plot.
#' @param plot_f1 Generate a plot with the F1 score vs. threshold as well.
#' @param subtitle Plot subtitle.
#' @param color Variable to color data points by.
#' @param shape Variable to set data point shapes by.
#' @param verbose Print messages.
#' @inheritParams precision_recall
#' @inheritParams EpiCompare
#' @inheritParams check_workers
#' @inheritParams get_bpparam
#' @inheritParams ggplot2::aes
#' @inheritParams ggplot2::facet_grid
#' @return list with data and precision recall and F1 plots
#'
#' @export
#' @importFrom methods show
#' @importFrom data.table data.table dcast :=
#' @examples
#' data("CnR_H3K27ac")
#' data("CnT_H3K27ac")
#' data("encode_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
#' reference <- list("encode_H3K27ac" = encode_H3K27ac)
#'
#' pr_out <- plot_precision_recall(peakfiles = peakfiles,
#'                                 reference = reference,
#'                                 workers = 1)
plot_precision_recall <- function(peakfiles,
                                  reference,
                                  thresholding_cols = c("total_signal", "qValue", "Peak Score"),
                                  initial_threshold = 0,
                                  n_threshold = 20,
                                  max_threshold = 1,
                                  workers = check_workers(),
                                  plot_f1 = TRUE,
                                  subtitle = NULL,
                                  color = "peaklist1",
                                  shape = color,
                                  rows = "peaklist2",
                                  cols = NULL,
                                  interact = FALSE,
                                  show_plot = TRUE,
                                  save_path =
                                    tempfile(fileext = "precision_recall.csv"),
                                  verbose = TRUE) {
  check_dep("ggplot2")
  workers <- check_workers(workers = workers)
  # #### Gather precision-recall data ####
  plot_dat <- precision_recall(
    peakfiles = peakfiles,
    reference = reference,
    thresholding_cols = thresholding_cols,
    initial_threshold = initial_threshold,
    max_threshold = max_threshold,
    n_threshold = n_threshold,
    workers = workers,
    cast = TRUE,
    save_path = save_path,
    verbose = verbose
  )
  #### Plot precision-recall ####
  gg <- plot_precision_recall_prcurve(
    plot_dat = plot_dat,
    color = color,
    shape = shape,
    cols = cols,
    rows = rows,
    subtitle = subtitle,
    interact = interact
  )
  #### Plot F1 #####
  if (isTRUE(plot_f1)) {
    ggf1 <- plot_precision_recall_f1(
      plot_dat = plot_dat,
      color = color,
      shape = shape,
      rows = rows,
      cols = cols,
      initial_threshold = initial_threshold,
      interact = interact
    )
  } else {
    ggf1 <- NULL
  }
  #### Show plots ####
  if (isTRUE(show_plot)) {
    methods::show(gg)
    methods::show(ggf1)
  }
  #### Return both the plot and data ####
  return(list(
    data = plot_dat,
    precision_recall_plot = gg,
    f1_plot = ggf1
  ))
}

test_that("plot_precision_recall works", {
  
    data("CnR_H3K27ac")
    data("CnT_H3K27ac")
    data("encode_H3K27ac")
    peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
    reference <- list("encode_H3K27ac" = encode_H3K27ac)

    pr_out <- plot_precision_recall(peakfiles = peakfiles,
                                    reference = reference,
                                    workers = 1)
    testthat::expect_equal(nrow(pr_out$data), 20*length(peakfiles))
    testthat::expect_true(all(c("precision","recall","F1") %in% names(pr_out$data)))
    testthat::expect_true(methods::is(pr_out$precision_recall_plot,"gg"))
    testthat::expect_true(methods::is(pr_out$f1_plot,"gg"))
    
    #### Prediction precision-recall #####
    res <- predict_precision_recall(pr_df = pr_out$data)
    testthat::expect_equal(nrow(res),20*length(peakfiles))
    testthat::expect_equal(round(max(res[predicted_var=="precision",]$predicted)),
                           96)
    testthat::expect_equal(round(max(res[predicted_var=="recall",]$predicted)),
                           53)
    # ggplot(res[predicted_var=="recall",], 
    #        aes(x=predicted, y=input, color=peaklist1)) +
    #     geom_point()
})

check_unused_args <- function(upset_plot,
                              stat_plot,
                              chromHMM_plot,
                              chipseeker_plot,
                              enrichment_plot,
                              tss_plot,
                              precision_recall_plot,
                              corr_plot,
                              add_download_button,
                              verbose=TRUE){
  #### Check which arguments are boolean ####
  # boolean_args <- names(formals(EpiCompare::EpiCompare))[
  #   sapply(formals(EpiCompare), is.logical)
  # ]
  if(isTRUE(verbose)){
    bool_args <- c(upset_plot=upset_plot, 
                   stat_plot=stat_plot, 
                   chromHMM_plot=chromHMM_plot, 
                   chipseeker_plot=chipseeker_plot, 
                   enrichment_plot=enrichment_plot, 
                   tss_plot=tss_plot, 
                   precision_recall_plot=precision_recall_plot, 
                   corr_plot=corr_plot,
                   add_download_button=add_download_button) 
    args_not_used <- names(bool_args)[bool_args==FALSE]
    if(length(args_not_used)>0){
      messager("NOTE: The following EpiCompare features are NOT being used:",
               paste0("\n - ",args_not_used,"=",collapse = ""),
               v=verbose) 
    } else{
      messager("All EpiCompare features are being used.",v=verbose)
    }
  } 
}

read_bam <- function(path,
                     verbose=TRUE){
    
    check_dep("GenomicAlignments")
    messager("Parsing BAM file.",v=verbose)
    bam <- GenomicAlignments::readGAlignments(file = path)
    return(bam)
}

check_heatmap_args <- function(draw_cellnote,
                               interact,
                               fill_diag,
                               verbose=TRUE){
  if(isTRUE(draw_cellnote) && isTRUE(interact)){
    check_dep("heatmaply")
    if(!is.null(fill_diag)){
      messager("Warning: fill_diag must =NULL",
               "due to a known bug in the package 'heatmaply'.",v=verbose)
      fill_diag <- NULL
    } 
  }
  return(fill_diag)
}

#' Prepare referemce as GRanges
#'
#' @param reference A named list of \link[GenomicRanges]{GRanges} objects, 
#' or a single \link[GenomicRanges]{GRanges} object to be converted into a 
#' named list.
#' @param max_elements Max number of elements to use within the list.
#' Set to \code{NULL} (default) to use all elements. 
#' @param remove_empty Remove any empty elements in the list.
#' @param as_list Return as a list.
#' @param as_grangeslist Return as a \link[GenomicRanges]{GRangesList}
#' (overrides \code{as_list}).
#'
#' @return A list of \link[GenomicRanges]{GRanges} objects
#'
#' @keywords internal
#' @importFrom methods is as
prepare_reference <- function(reference,
                              max_elements=NULL,
                              remove_empty=TRUE,
                              as_list=TRUE,
                              as_grangeslist=FALSE){
    message("Preparing reference.")
    if(is.null(reference)) {
        message("No reference provided. Returning NULL.")
        return(NULL)
    }
     
    if(isTRUE(as_list) | isTRUE(as_grangeslist)){
      if(methods::is(reference,"GRangesList") &&
         isFALSE(as_grangeslist)){
        reference <- as.list(reference)
      }
      if(!methods::is(reference,"list")){
        if(methods::is(reference,"GRangesList")){
          reference <- as.list(reference)
        } else {
          reference <- list(reference) 
        }
      }
      if(isTRUE(as_grangeslist) ){
        reference <- methods::as(reference,"GRangesList") 
      }
    }
    reference <- check_list_names(peaklist = reference, 
                                  default_prefix = "reference") 
    #### Remove empty elements ####
    if(isTRUE(remove_empty)){
        reference <- remove_empty_elements(peaklist = reference)
    }
    #### Limit the number of elements ####
    if((!is.null(max_elements))){
        if(length(reference)>max_elements){
            msg <- paste("Warning:",length(reference),
                         "elements were found in reference list",
                         "Only using first element.")
            message(msg)
            reference <- reference[seq_len(max_elements)]
        } else if(length(reference)==1 && 
                  max_elements==1 &&
                  isFALSE(as_list)) {
            msg <- paste("Extracting only the first GRanges object from list.")
            message(msg)
            reference <- reference[[1]]
        } 
    } 
    return(reference)
}

#' Calculate percentage of overlapping peaks
#'
#' This function calculates the percentage of overlapping peaks and outputs
#' a table or matrix of results.
#'
#' @param peaklist1 A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names will be assigned.
#' @param peaklist2 peaklist1 A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' @param suppress_messages Suppress messages.
#' @param precision_recall Return percision-recall results for all combinations 
#' of \code{peaklist1} (the "query") and \code{peaklist2} (the "subject"). 
#' See \link[IRanges]{subsetByOverlaps} for more details on this terminology. 
#' @inheritParams IRanges::findOverlaps
#'
#' @return data frame
#' @importMethodsFrom IRanges subsetByOverlaps
#' @importFrom data.table data.table rbindlist :=
#' @export
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peakfile GRanges object
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("CnR_H3K27ac") # example peakfile GRanges object
#'
#' ### Create Named Peaklist ###
#' peaks <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
#' reference_peak <- list("ENCODE"=encode_H3K27ac)
#'
#' ### Run ###
#' overlap <- overlap_percent(peaklist1=peaks,
#'                            peaklist2=reference_peak)
overlap_percent <- function(peaklist1,
                            peaklist2,
                            invert=FALSE, 
                            precision_recall=TRUE,
                            suppress_messages=TRUE){
    #### Suppress messages ####
    f <- if(suppress_messages) suppressMessages else function(x){x}    
    #### Clean up chroms ####
    peaklist1 <- remove_nonstandard_chrom(peaklist1, 
                                          verbose = FALSE)
    peaklist2 <- remove_nonstandard_chrom(peaklist2,
                                          verbose = FALSE)
  ### Calculate Overlap ###
  # When peaklist1 is the reference
  if(length(peaklist1)==1 && isFALSE(precision_recall)){
    percent_list <- mapply(peaklist2, 
                           SIMPLIFY = FALSE,
                           FUN=function(y){
      overlap <- f(
          IRanges::subsetByOverlaps(x = peaklist1[[1]], # query
                                    ranges = y, # subject
                                    invert = invert)
      )
      percent <- length(overlap)/length(peaklist1[[1]])*100
      data.table::data.table(Percentage=signif(percent,3)) 
    }) |> data.table::rbindlist(use.names = TRUE, idcol = "query")
    #### Reformat to work with EpiCompare ####
    df <- data.frame(Percentage = percent_list$Percentage, 
                     row.names = percent_list$query) 
    
  ### Calculate Overlap ###
  # When peaklist2 is the reference
  } else if(length(peaklist2)==1 && isFALSE(precision_recall)){ 
    percent_list <- mapply(peaklist1, 
                           SIMPLIFY = FALSE,
                           FUN=function(x){
      overlap <- f(
          IRanges::subsetByOverlaps(x = x, # query
                                    ranges = peaklist2[[1]], # subject
                                    invert = invert)
      )
      percent <- length(overlap)/length(x)*100
      data.table::data.table(Percentage=signif(percent,3)) 
  }) |> data.table::rbindlist(use.names = TRUE, idcol = "query")
    #### Reformat to work with EpiCompare ####
    df <- data.frame(Percentage = percent_list$Percentage, 
                     row.names = percent_list$query)

    ### Calculate Overlap ###
    # When peaklist1 abd peaklist2 have multiple elements
  } else { 
      messager("Computing precision-recall results.")
      # if subject or ranges is an IntegerRanges object, 
      # query or x can be an integer vector 
      # to be converted to length-one ranges. 
      #### peaklist1 as the query, peaklist2 as the subject ####
      percent_list1 <- mapply(peaklist1, 
                             SIMPLIFY = FALSE,
                             FUN = function(x){
          mapply(peaklist2,
                 SIMPLIFY = FALSE,
                 FUN=function(y){
                     overlap <- f(
                         IRanges::subsetByOverlaps(x = x, # query
                                                   ranges = y, # subject
                                                   invert = invert)
                     )
                     percent <- length(overlap)/length(x)*100
                     data.table::data.table(overlap=length(overlap),
                                            total=length(x),
                                            Percentage=signif(percent,3)) 
         }) |> data.table::rbindlist(use.names = TRUE, idcol = "subject")
      }) |> data.table::rbindlist(use.names = TRUE, idcol = "query") 
      percent_list1$type <- "precision"
      
      #### peaklist2 as the query, peaklist1 as the subject ####
      percent_list2 <- mapply(peaklist2, 
                              SIMPLIFY = FALSE,
                              FUN = function(x){
          mapply(peaklist1,
                 SIMPLIFY = FALSE,
                 FUN=function(y){
                     overlap <- f(
                         IRanges::subsetByOverlaps(x = x, # query 
                                                   ranges = y, # subject
                                                   invert = invert)
                     )
                     percent <- length(overlap)/length(x)*100
                     data.table::data.table(overlap=length(overlap),
                                            total=length(x),
                                            Percentage=signif(percent,3)) 
                 }) |> data.table::rbindlist(use.names = TRUE, idcol = "subject")
    }) |> data.table::rbindlist(use.names = TRUE, idcol = "query")
      percent_list2$type <- "recall"
      df <- rbind(percent_list1, percent_list2)
      id <- type <- query <- subject <- NULL;
      df[,peaklist1:=ifelse(type=="precision",query,subject)] 
      df[,peaklist2:=ifelse(type=="precision",subject,query)] 
  }
  ### Return ###
  return(df)
}

compute_consensus_peaks_consensusseeker <- function(grlist,
                                                    genome_build,
                                                    ...){ 
    bsgen <- check_genome_build(genome_build = genome_build,
                                type = "bsgen")
    chrInfo <- GenomicRanges::seqinfo(bsgen)[
        Seqinfo::seqlevelsInUse(grlist)
    ]  
    consensus_peaks <- consensusSeekeR::findConsensusPeakRegions( 
        peaks = unlist(grlist),
        chrInfo = chrInfo, 
        ...
    )$consensusRanges
    return(consensus_peaks)
}

get_chain_file <- function(grlist,
                           input_build,
                           output_build,
                           verbose = TRUE){
    messager("Preparing chain file.",v=verbose)
    ah <- AnnotationHub::AnnotationHub()
    # chainfiles <- AnnotationHub::query(ah , c("hg38", "hg19", "chainfile"))
    # chainfiles <- AnnotationHub::query(ah , c("hg38", "mm10", "chainfile"))
    # chainfiles <- AnnotationHub::query(ah , c("hg19", "mm10", "chainfile")) 
    ## hg38 <--> mm9 chain file is not available ###
    # chainfiles <- AnnotationHub::query(ah , c("hg19", "mm9", "chainfile"))
    # chainfiles <- AnnotationHub::query(ah , c("mm10", "mm9", "chainfile"))
    # AH14108 | hg38ToHg19.over.chain.gz                     
    # AH14150 | hg19ToHg38.over.chain.gz                     
    # AH78915 | Chain file for Homo sapiens rRNA hg19 to hg38
    # AH78916 | Chain file for Homo sapiens rRNA hg38 to hg19
    
    #### get chain  ####
    ## Intra-species 
    #### hg38 <--> hg19 ####
    if ((input_build=="hg38") && 
        (output_build=="hg19")){ 
        chain <- ah[["AH14108"]]  
    } else if((input_build=="hg19") && 
              (output_build=="hg38")){
        chain <- ah[["AH14150"]] 
        #### mm10 <--> mm9 ####
    } else if ((input_build=="mm10") && 
               (output_build=="mm9")){ 
        chain <- ah[["AH14535"]]
    } else if((input_build=="mm9") && 
              (output_build=="mm10")){
        chain <- ah[["AH14596"]]
        ## Inter-species 
        #### mm10 ####
    } else if((input_build=="hg38") && 
              (output_build=="mm10")){
        chain <- ah[["AH14109"]] 
    } else if((input_build=="mm10") && 
              (output_build=="hg38")){
        chain <- ah[["AH14528"]]  
    } else if((input_build=="hg19") && 
              (output_build=="mm10")){
        chain <- ah[["AH14156"]] 
    } else if((input_build=="mm10") && 
              (output_build=="hg19")){
        chain <- ah[["AH14527"]] 
        #### mm9 ####
    } else if((input_build=="mm9") && 
              (output_build=="hg19")){
        chain <- ah[["AH14592"]] 
    } else if((input_build=="hg19") && 
              (output_build=="mm9")){
        chain <- ah[["AH14155"]] 
        #### use recursion for hg38 <-- hg19 --> mm9 ####
        ## There is no chain file mapping hg38 to mm9 directly (or vice versa), 
        ## so we must first convert to hg19 as an intermediate step.
    } else if((input_build=="hg38") && 
              (output_build=="mm9")){
        grlist <- liftover_grlist(grlist = grlist, 
                                  input_build = input_build, 
                                  output_build = "hg19", 
                                  verbose = verbose)
        input_build <- "hg19"
        chain <- ah[["AH14155"]] 
    } else if((input_build=="mm9") && 
              (output_build=="hg38")){
        grlist <- liftover_grlist(grlist = grlist, 
                                  input_build = input_build, 
                                  output_build = "hg19", 
                                  verbose = verbose)
        input_build <- "hg19"
        chain <- ah[["AH14150"]] 
    } else {
        stopper(input_build,"-->",output_build,"not currently supported.")
    }
    #### liftover ####
    messager("Performing liftover: ",
             input_build," --> ",output_build,
             v=verbose) 
    return(list(grlist=grlist,
                chain=chain))
}

#' Predict precision-recall
#' 
#' Predict specific values of precision or recall by fitting a model to
#' a precision-recall curve. 
#' Predictions that are <0 will automatically be set to 0.
#' Predictions that are >100 will automatically be set to 100.
#' @param pr_df Precision-recall data.frame generated by 
#' \link[EpiCompare]{precision_recall}.
#' @param fun Function to fit the data with.
#' @param precision Precision values to predict recall from.
#' @param recall Recall values to predict precision from. 
#' @returns A named list of fitted models and predictions.
#' @source \href{https://stackoverflow.com/a/27796497}{
#' Fix for producing NAs from loess fun.}
#' 
#' @export
#' @importFrom data.table rbindlist
#' @examples 
#' data("CnR_H3K27ac")
#' data("CnT_H3K27ac")
#' data("encode_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
#' reference <- list("encode_H3K27ac" = encode_H3K27ac)
#' pr_df <- precision_recall(peakfiles = peakfiles,
#'                           reference = reference)
#' predictions <- predict_precision_recall(pr_df = pr_df)           
predict_precision_recall <- function(pr_df,
                                     fun=stats::loess,
                                     precision=seq(10,100,10),
                                     recall=seq(10,100,10)){
    # devoptera::args2vars(EpiCompare::predict_precision_recall) 
    
    peaklist1 <- peaklist2 <- NULL;
    
     res <- lapply(stats::setNames(unique(pr_df$peaklist1),
                                   unique(pr_df$peaklist1)),
                   function(p1){ 
         messager("Making predictions for peaklist1:",p1)
         lapply(stats::setNames(unique(pr_df$peaklist2),
                                unique(pr_df$peaklist2)),
                function(p2){ 
             messager("Making predictions for peaklist2:",p2)
             pr_sub <- pr_df[peaklist1==p1 & peaklist2==p2,]
             data.table::rbindlist(
                 #### Predict recall from precision ####
                list(
                    predict_values(df = pr_sub,
                                   fun = fun,
                                   values = precision, 
                                   input_var = "precision", 
                                   predicted_var = "recall"),
                    
                    #### Predict precision from recall  ####
                    predict_values(df = pr_sub, 
                                   fun = fun,
                                   values = recall, 
                                   input_var = "recall", 
                                   predicted_var = "precision")
                )
            )
         }) |> data.table::rbindlist(use.names = TRUE, idcol = "peaklist2") 
     }) |> data.table::rbindlist(use.names = TRUE, idcol = "peaklist1")
    #### Return ####
    return(res)
}

#' Check genome build
#' 
#' Check that the genome build is valid and 
#' require specific reference datasets to be installed.
#' @param genome_build Genome build name.
#' @param type whether to fetch the txdb or bsgen reference data
#' @keywords internal
#' @returns txdb or bsgen
check_genome_build <- function(genome_build,
                               type="txdb"){
    genome_build <- translate_genome(genome = genome_build)
    if(type=="txdb"){
      if(genome_build == "hg19"){
        check_dep("TxDb.Hsapiens.UCSC.hg19.knownGene")
          txdb <- 
            TxDb.Hsapiens.UCSC.hg19.knownGene::TxDb.Hsapiens.UCSC.hg19.knownGene
      }else if(genome_build == "hg38"){
        check_dep("TxDb.Hsapiens.UCSC.hg38.knownGene")
          txdb <- 
            TxDb.Hsapiens.UCSC.hg38.knownGene::TxDb.Hsapiens.UCSC.hg38.knownGene
      }else if(genome_build == "mm9"){
        check_dep("TxDb.Mmusculus.UCSC.mm9.knownGene")
          txdb <- 
            TxDb.Mmusculus.UCSC.mm9.knownGene::TxDb.Mmusculus.UCSC.mm9.knownGene
      }else if(genome_build == "mm10"){
        check_dep("TxDb.Mmusculus.UCSC.mm10.knownGene")
          txdb <- 
            TxDb.Mmusculus.UCSC.mm10.knownGene::TxDb.Mmusculus.UCSC.mm10.knownGene
      }else {
          msg <- paste("genome_build must be one of:",
                       "\n- 'hg19'",
                       "\n- 'hg38'",
                       "\n- 'mm9'",
                       "\n- 'mm10'")
          stop(msg)
      }
      return(txdb)
    }else{ #BSgenome
      if(genome_build == "hg19"){
        check_dep("BSgenome.Hsapiens.UCSC.hg19")
        bsgen <- BSgenome.Hsapiens.UCSC.hg19::Hsapiens
      }else if(genome_build == "hg38"){
        check_dep("BSgenome.Hsapiens.UCSC.hg38")
        bsgen <- BSgenome.Hsapiens.UCSC.hg38::Hsapiens
      }else if(genome_build == "mm9"){
        check_dep("BSgenome.Mmusculus.UCSC.mm9")
        bsgen <- BSgenome.Mmusculus.UCSC.mm9::Mmusculus
      }else if(genome_build == "mm10"){
        check_dep("BSgenome.Mmusculus.UCSC.mm10")
        bsgen <- BSgenome.Mmusculus.UCSC.mm10::Mmusculus
      }else {
        msg <- paste("genome_build must be one of:",
                     "\n- 'hg19'",
                     "\n- 'hg38'",
                     "\n- 'mm9'",
                     "\n- 'mm10'")
        stop(msg)
      }
      return(bsgen)
    }
}

#' Check cache
#' 
#' Quick function to check if object is already saved.
#' @param cache BiocFileCache.
#' @param url Path to cached file.  
#' @keywords internal
#' @returns path 
checkCache <- function(cache=BiocFileCache::BiocFileCache(ask = FALSE),
                       url) {
  
    check_dep("BiocFileCache")
    cached <- BiocFileCache::bfcinfo(cache)$rname
    return(url %in% cached)
}

compute_consensus_peaks_granges <- function(grlist,
                                            upper,
                                            lower,
                                            min.gapwidth){
    peak_coverage <- GenomicRanges::coverage(grlist)
    consensus_peaks <- IRanges::slice(x = peak_coverage, 
                                      upper = upper,
                                      lower = lower, 
                                      rangesOnly = TRUE)
    consensus_peaks <- GenomicRanges::GRanges(consensus_peaks)
    #### Merge nearby peaks ####
    consensus_peaks <- GenomicRanges::reduce(x = consensus_peaks, 
                                             min.gapwidth = min.gapwidth)
    return(consensus_peaks)
}

#' As interactive
#' 
#' Convert a \link[ggplot2]{ggplot} object to \link[plotly]{plotly}, 
#' and enable it to be plotted within an Rmarkdown HTML file.
#' @source \href{https://github.com/yihui/knitr/issues/926}{
#' GitHub Issue to check whether knitting}
#' @param plt ggplot object.
#' @param to_widget Convert to a widget so it works within Rmarkdown HTML files.
#' By default, this will be only be set to \code{TRUE} when being run within 
#' the context of \pkg{knitr} rendering.
#' @param add_boxmode Add extra \link[plotly]{layout} to enable dodged boxplots.
#' @returns A \link[plotly]{plotly} object or a \link[htmltools]{tagList} 
#' wrapping the \link[plotly]{plotly} object.
#' 
#' @keywords internal 
#' @importFrom plotly ggplotly layout as_widget
#' @importFrom htmltools tagList
as_interactive <- function(plt, 
                           to_widget=isTRUE(getOption('knitr.in.progress')),
                           add_boxmode=FALSE){
  # plotly::renderPlotly()
  if(is.null(plt)) return(NULL)
  if(!methods::is(plt,"plotly")){
    plt <- plotly::ggplotly(plt)
  } 
  if(isTRUE(add_boxmode)){
    plt <- plt |> plotly::layout(boxmode = "group")
  }
  if(isTRUE(to_widget)){
    plt <- htmltools::tagList(plotly::as_widget(plt))
  } 
  return(plt)
}

#' Translate genome
#' 
#' Translate the name of a genome build from one format to another.
#' @param omit_subversion Omit any subversion suffixes after the ".". 
#' @param default_genome Default genome build when \code{genome} is \code{NULL}.
#' @inheritParams GenomeInfoDb::mapGenomeBuilds
#' @returns Standardized genome build name as a character string. 
#' 
#' @export
#' @importFrom GenomeInfoDb mapGenomeBuilds
#' @importFrom utils tail
#' @examples 
#' genome <- translate_genome(genome="hg38", style="Ensembl")
#' genome2 <- translate_genome(genome="mm10", style="UCSC")
translate_genome <- function(genome,
                             style=c("UCSC", "Ensembl","NCBI"),
                             default_genome=NULL,
                             omit_subversion=TRUE){
    #### Check genome ####
    force(genome)
    if(is.null(genome)){
        if(!is.null(default_genome)){
            msg <- paste(
                "WARNING: genome=NULL. Setting to default build:",
                default_genome
            )
            messager(msg)
            genome <- default_genome
        } else {
            stp <- paste(
                "Must provide a valid genome build",
                "to translate_genome(genome=)."
            )
            stop(stp)
        }
    }
    #### Check style ####
    if(length(style)>1){
        style <- style[1]
        # messager("WARNING: >1 style supplied. Only using the first:",style)
    }
    #### Map genome build synonyms ####
    if(toupper(style)=="NCBI"){
        style <- "Ensembl"
        use_ncbi <- TRUE
    } else {
        use_ncbi <- FALSE
    }
    dat <- GenomeInfoDb::mapGenomeBuilds(genome = genome,
                                         style = style)
    
    if(nrow(dat)==0){
        stopper("Could not recognize genome.")
    }
    #### Use latest version (bottom) #### 
    if(use_ncbi){
        genome2 <- tail(grep("NCBI",dat$ensemblID, value = TRUE),1)
    } else{
        if(toupper(style)=="UCSC"){
            
            genome2 <- utils::tail(dat$ucscID,1)
        } else {
            genome2 <- utils::tail(grep("GRCh",dat$ensemblID, value = TRUE),1)
        } 
    }  
    if(omit_subversion){
        genome2 <- strsplit(genome2,"[.]")[[1]][1]
    }
    if(is.na(genome2) || length(genome2)==0){
        stopper("Could not identify genome translation.")
    }
    return(genome2)
}

check_dep <- function(dep){
  if(!requireNamespace(dep, quietly = TRUE)){
    stp <- paste("Package",shQuote(dep),
                 "must be installed to use this function.")
    stop(stp,
         call. = FALSE)
  }
}

#' Set min/max
#' 
#' Set the min/max values in a data.frame.
#' @param df data.frame
#' @param colname Column name to check.
#' @param min_val Minimum value.
#' @param max_val Maximum value.
#' @returns data.frame
#' 
#' @keywords internal 
#' @importFrom data.table data.table :=
set_min_max <- function(df,
                        colname,
                        min_val=0,
                        max_val=100){ 
    
    df <- data.table::data.table(df)
    df[,(colname):=ifelse(get(colname)<min_val,min_val,get(colname)),]
    df[,(colname):=ifelse(get(colname)>max_val,max_val,get(colname)),]
    return(df)
}

#' Read bowtie
#' 
#' Read a bowtie file.
#' @param path Path to bowtie file.
#' @param verbose Print messages.
#' @returns \link[data.table]{data.table}
#' 
#' @keywords internal
#' @importFrom data.table fread :=
read_bowtie <- function(path,
                        verbose=TRUE){
    
    metric <- NULL;
    if(endsWith("idxstats",path)){
        # Header info from: http://www.htslib.org/doc/samtools-idxstats.html
        messager("Reading bowtie2 idxstats.",v=verbose) 
        dat <- data.table::fread(path)|>
            `colnames<-`(c("chrom","sequence_length",
                           "mapped_read_segments",
                           "unmapped_read_segments"))
    } else {
        messager("Reading bowtie2 stats.",v=verbose)
        l <- readLines(path)
        dat <- data.table::fread(text=grep("^SN",l,value = TRUE), 
                                 sep = "\t",
                                 fill = TRUE)[,-1] |>
            `colnames<-`(c("metric","value","comment"))
        dat[,metric:=gsub(
            "[%]","pct",
            gsub("[()]|[)]","",
                 gsub("[ ]","_",trimws(metric,whitespace = ":"))))]
    } 
    dat[,path:=path]
    return(dat)
}

#' Create a precision-recall matrix
#' 
#' Converts a list of peak files to a symmetric matrix where 
#' the y-axis indicates precision and the x-axis indicates recall.
#' 
#' @param fill_diag Fill the diagonal of the overlap matrix.
#' @param verbose Print messages.
#' @returns matrix
#' 
#' @keywords internal
#' @importMethodsFrom IRanges subsetByOverlaps
precision_recall_matrix <- function(peaklist,
                                    fill_diag=NA,
                                    verbose=TRUE){
  
    messager("Genreating precision-recall matrix.",v=verbose)
    overlap_list <- lapply(peaklist, function(mainfile){
        lapply(peaklist, function(subfile){
            # overlapping peaks
            overlap <- IRanges::subsetByOverlaps(x = subfile, 
                                                 ranges = mainfile)
            # calculate percentage overlap
            percent <- length(overlap)/length(subfile)*100
            return(percent)
        }) 
    })
    ### Create Matrix ###
    overlap_matrix <- matrix(unlist(overlap_list),
                             ncol = max(lengths(overlap_list)),
                             byrow = FALSE, 
                             dimnames = list(names(peaklist),
                                             names(peaklist)))  
    #### Fill diagonal ####
    if(!is.null(fill_diag)){
        diag(overlap_matrix) <- fill_diag
    }
    return(overlap_matrix)
}

#' Example ChIP-seq peak file
#'
#' Human H3K27ac peak file generated with ChIP-seq using K562 cell-line.
#' Human genome build hg19 was used.
#' The peak file (.BED) was obtained from ENCODE project
#' (\url{https://www.encodeproject.org/files/ENCFF044JNJ/}).
#' The BED file was then imported as an GRanges object.
#' Peaks located on chromosome 1 were subsetted to reduce the dataset size.
#'
#' @source
#' The code to prepare the .Rda file from the raw peak file is:
#'
#' \code{# dataset was directly downloaded from} \cr
#' \code{# https://www.encodeproject.org/files/ENCFF044JNJ/}
#' \code{encode_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
#' \code{encode_H3K27ac <- 
#' encode_H3K27ac[seqnames(encode_H3K27ac) == "chr1"]} \cr
#' \code{my_label <-
#'  c("name","score","strand","signalValue","pValue","qValue","peak")} \cr
#' \code{colnames(GenomicRanges::mcols(encode_H3K27ac)) <- my_label} \cr
#' \code{usethis::use_data(encode_H3K27ac, overwrite = TRUE)} \cr
#'
#' @usage data("encode_H3K27ac")
"encode_H3K27ac"

#' Example CUT&Tag peak file
#'
#' Human H3K27ac peak file generated with CUT&Tag using K562 cell-line from
#' Kaya-Okur et al., (2019). Human genome build hg19 was used.
#' Raw peak file (.BED) was obtained from GEO
#' (\url{https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8383507}).
#' Peak calling was performed by Leyla Abbasova using MACS2.
#' The peak file was then imported as an GRanges object.
#' Peaks located on chromosome 1 were subsetted to reduce the dataset size.
#'
#' @source
#' The code to prepare the .Rda file from the raw peak file is:
#'
#' \code{# sequences were directly downloaded from 
#' https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8383507} \cr
#' \code{# and peaks (BED file) were generated by Leyla Abbasova 
#' (Neurogenomics Lab, Imperial College London)} \cr
#' \code{CnT_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
#' \code{CnT_H3K27ac <- CnT_H3K27ac[seqnames(CnT_H3K27ac)== "chr1"]} \cr
#' \code{my_label <- 
#' c("name","score","strand","signalValue","pValue","qValue","peak")} \cr
#' \code{colnames(GenomicRanges::mcols(CnT_H3K27ac)) <- my_label} \cr
#' \code{usethis::use_data(CnT_H3K27ac)} \cr
#' @usage data("CnT_H3K27ac")
"CnT_H3K27ac"

#' Example CUT&Run peak file
#'
#' Human H3K27ac peak file generated with CUT&Run using K562 cell-line from
#' Meers et al., (2019). Human genome build hg19 was used.
#' Raw peak file (.BED) was obtained from GEO
#' (\url{https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8581604}).
#' Peak calling was performed by Leyla Abbasova using MACS2.
#' The peak file was then processed into GRanges object.
#' Peaks located on chromosome 1 were subsetted to reduce the dataset size.
#'
#' @source
#' The code to prepare the .Rda file from the raw peak file is:
#'
#' \code{# sequences were directly downloaded from
#'  https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8581604} \cr
#' \code{# and peaks (BED file) were generated by Leyla Abbasova 
#' (Neurogenomics Lab, Imperial College London)} \cr
#' \code{CnR_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
#' \code{CnR_H3K27ac <- CnR_H3K27ac[seqnames(CnR_H3K27ac)== "chr1"]} \cr
#' \code{my_label <- 
#' c("name","score","strand","signalValue","pValue","qValue","peak")}\cr
#' \code{colnames(GenomicRanges::mcols(CnR_H3K27ac)) <- my_label} \cr
#' \code{usethis::use_data(CnR_H3K27ac, overwrite = TRUE)} \cr
#' @usage data("CnR_H3K27ac")
"CnR_H3K27ac"

#' Example Picard duplication metrics file 1
#'
#' Duplication metrics output of CUT&Tag H3K27ac file 
#' (sample accession: SRR8581604).
#' Raw sequences were aligned to hg19 genome and Picard 
#' was performed by Leyla Abbasova.
#' The duplication summary output generated by Picard was processed 
#' to reduce the size of data.
#'
#' @source
#' The code to prepare the .Rda file is:
#'
#' \code{picard <- read.table("path/to/picard/duplication/output", 
#' header = TRUE, fill = TRUE)]} \cr
#' \code{CnT_H3K27ac_picard <- picard[1,]} \cr
#' \code{usethis::use_data(CnT_H3K27ac_picard, overwrite = TRUE)} \cr
#' @usage data("CnT_H3K27ac_picard")
"CnT_H3K27ac_picard"

#' Example Picard duplication metrics file 2
#'
#' Duplication metrics output on CUT&Run H3K27ac file 
#' (sample accession: SRR8581604).
#' Raw sequences were aligned to hg19 genome and after, 
#' Picard was performed by Leyla Abbasova.
#' The duplication summary output generated by Picard 
#' was processed to reduce the size of data.
#'
#' @source
#' The code to prepare the .Rda file is:
#'
#' \code{picard <- read.table("path/to/picard/duplication/output", 
#' header = TRUE, fill = TRUE)} \cr
#' \code{CnR_H3K27ac_picard <- picard[1,]} \cr
#' \code{usethis::use_data(CnR_H3K27ac_picard, overwrite = TRUE)} \cr
#' @usage data("CnR_H3K27ac_picard")
"CnR_H3K27ac_picard"

#' Human genome hg19 blacklisted regions
#'
#' Obtained from \url{https://www.encodeproject.org/files/ENCFF001TDO/}.
#' The ENCODE blacklist includes regions of the hg19 genome that have anomalous
#' and/or unstructured signals independent of the cell-line or experiment.
#' Removal of ENCODE blacklist is recommended for quality measure.
#'
#' @source
#' The code to prepare the .Rda file is:
#'
#' \code{# blacklisted regions were directly downloaded} \cr
#' \code{# from https://www.encodeproject.org/files/ENCFF001TDO/} \cr
#' \code{hg19_blacklist <- 
#' ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
#' \code{usethis::use_data(hg19_blacklist, overwrite = TRUE)} \cr
#' @usage data("hg19_blacklist")
"hg19_blacklist"

#' Human genome hg38 blacklisted regions
#'
#' Obtained from \url{https://www.encodeproject.org/files/ENCFF356LFX/}.
#' The ENCODE blacklist includes regions of the hg38 genome that have 
#' anomalous and/or unstructured signals 
#' independent of the cell-line or experiment. Removal
#' of ENCODE blacklist is recommended for quality measure.
#'
#' @source
#' The code to prepare the .Rda file is:
#'
#' \code{## blacklisted regions were directly downloaded} \cr
#' \code{## from https://www.encodeproject.org/files/ENCFF356LFX/} \cr
#' \code{hg38_blacklist <- 
#' ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
#' \code{usethis::use_data(hg38_blacklist, overwrite = TRUE)} \cr
#' @usage data("hg38_blacklist")
"hg38_blacklist"

#' Mouse genome mm10 blacklisted regions
#'
#' Obtained from \url{https://www.encodeproject.org/files/ENCFF547MET/}.
#' The ENCODE blacklist includes regions of the mm10 genome that have anomalous 
#' and/or unstructured signals independent of the cell-line or experiment. 
#' Removal of ENCODE blacklist is recommended for quality measure.
#'
#' @source
#' The code to prepare the .Rda file is:
#'
#' \code{## blacklisted regions were directly downloaded} \cr
#' \code{## from https://www.encodeproject.org/files/ENCFF547MET/} \cr
#' \code{mm10_blacklist <- 
#' ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
#' \code{usethis::use_data(mm10_blacklist, overwrite = TRUE)} \cr
#' @usage data("mm10_blacklist")
"mm10_blacklist"


#' Mouse genome mm9 blacklisted regions
#'
#' Blaklisted regions of the mm9 genome build 
#' btained by lifting over the \code{mm10_blacklist}.
#' @source
#' \code{
#' tmp <- base::get("mm10_blacklist", asNamespace("EpiCompare"))
#' mm9_blacklist <- liftover_grlist(grlist = tmp,
#'                                  input_build = "mm10",
#'                                  output_build = "mm9", 
#'                                  keep_chr = NULL)
#' usethis::use_data(mm9_blacklist, overwrite = TRUE)                                
#' }
#' @usage data("mm9_blacklist")
"mm9_blacklist"

#' Compute correlation matrix
#' 
#' Compute correlation matrix on all peak files.
#' @param genome_build The build of **all** peak and reference files to 
#' calculate the correlation matrix on. If all peak and reference files are not 
#' of the same build use 
#' \link[EpiCompare]{liftover_grlist} to convert them all before running. Genome
#' build should be one of hg19, hg38, mm9, mm10.
#' @param bin_size Default of 100. Base-pair size of the bins created to measure
#' correlation. Use smaller value for higher resolution but longer run time and 
#' larger memory usage.
#' @param drop_empty_chr Drop chromosomes that are not present in any of the 
#' \code{peakfiles} (default: \code{FALSE}). 
#' @param method Default spearman (i.e. non-parametric). A character string 
#' indicating which correlation coefficient (or covariance) is to be computed. 
#' One of "pearson", "kendall", or "spearman": can be abbreviated.
#' @param intensity_cols Depending on which columns are present, this
#' value will be used to get quantiles and ultimately calculate the 
#' correlations:
#' \itemize{
#'  \item{"total_signal" : }{Used by the peak calling software 
#'  \href{https://github.com/FredHutch/SEACR}{SEACR}. 
#'  \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
#'  together or instead of "total_signal".}
#'  \item{"qValue"}{Used by the peak calling software 
#'  \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
#'  Should contain the negative log of the p-values after multiple 
#'  testing correction.}
#'  \item{"Peak Score" : }{
#'  Used by the peak calling software
#'   \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
#' }
#' @param return_bins If \code{TRUE}, returns a named list
#'  with both the rebinned 
#' (standardised) peaks ("bin") and the correlation matrix ("cor").
#' If \code{FALSE} (default), returns only the correlation matrix (unlisted). 
#' @param save_path Path to save a table of correlation results to.
#' @inheritParams EpiCompare
#' @inheritParams get_bpparam
#' @inheritParams check_workers
#' @inheritParams remove_nonstandard_chrom
#' @inheritParams precision_recall_matrix
#' @return correlation matrix
#' 
#' @export
#' @importFrom stats cor
#' @examples 
#' data("CnR_H3K27ac")
#' data("CnT_H3K27ac")
#' data("encode_H3K27ac")
#' peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
#' reference <- list("encode_H3K27ac"=encode_H3K27ac)
#' 
#' #increasing bin_size for speed but lower values will give more granular corr
#' corr_mat <- compute_corr(peakfiles = peakfiles,
#'                          reference = reference,
#'                          genome_build = "hg19",
#'                          bin_size = 200000, 
#'                          workers = 1)
compute_corr <- function(peakfiles,
                         reference = NULL,
                         genome_build,
                         keep_chr = NULL,
                         drop_empty_chr = FALSE,
                         bin_size = 5000,
                         method = "spearman",
                         intensity_cols=c("total_signal", 
                                          "qValue",
                                          "Peak Score",
                                          "score"),
                         return_bins = FALSE,
                         fill_diag = NA,
                         workers = check_workers(),
                         save_path = tempfile(fileext = ".corr.csv.gz")){
    # devoptera::args2vars(EpiCompare::compute_corr)
    
    t1 <- Sys.time() 
    workers <- check_workers(workers = workers)
    #### append all peak files since all to be compared ####
    #make sure reference not already added to peakfiles
    if(!is.null(reference)){
        all_peaks <- append(reference[!names(reference) %in% names(peakfiles)],
                            peakfiles)
    } else {
        all_peaks <- peakfiles
    } 
    #sense check
    if(length(all_peaks)<=1)
      stop("Need more than one peak file to create correlation matrix.") 
    #### Re-bin data so comparisons made same regions ####
    gr_mat <- rebin_peaks(peakfiles = all_peaks, 
                          genome_build = genome_build, 
                          keep_chr = keep_chr,
                          drop_empty_chr = drop_empty_chr,
                          intensity_cols = intensity_cols,
                          bin_size = bin_size, 
                          ## stats::cor can only take dense matrices.
                          as_sparse = FALSE,
                          workers = workers)
    #### Run all pairwise correlations #####
    messager("Calculating correlation matrix.")
    cor_mat <- stats::cor(gr_mat, method=method)
    #### Fill diagonal ####
    if(!is.null(fill_diag)) diag(cor_mat) <- fill_diag
    #### Report time ####
    t2 <- Sys.time()
    messager("Done computing correlations in",
             round(difftime(t2,t1,units = "s"),0),"seconds.") 
    #### Save ####
    save_path <- save_results(
        dat = data.table::data.table(cor_mat, keep.rownames = TRUE), 
        save_path = save_path, 
        type = "correlation")
    #### Return ####
    if(isTRUE(return_bins)){ 
       return(
           list(bins = gr_mat,
                cor = cor_mat)
       )
    } else {
        return(cor_mat)
    } 
}

test_that("compute_corr works", {
  
    data("CnR_H3K27ac")
    data("CnT_H3K27ac")
    data("encode_H3K27ac")
    peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
    reference <- list("encode_H3K27ac" = encode_H3K27ac)

    #increasing bin_size for speed but lower values will give more granular corr
    bins <- c(500000,100000,10000,5000
              # 1000,500,400,200,100,50
              )
    cor_mats <- mapply(stats::setNames(bins,bins),
                       SIMPLIFY = FALSE,
                       FUN=function(bin_size){
        compute_corr(peakfiles = peakfiles,
                     reference = reference,
                     genome_build = "hg19",
                     workers = 1,
                     bin_size = bin_size)
    })
    cor_mats2 <- mapply(cor_mats, 
                       SIMPLIFY = FALSE,
                       FUN=function(x){diag(x)<-NA;x}) 
    cor_mean <- mapply(cor_mats2, FUN=mean, na.rm=TRUE)
    testthat::expect_equal(round(mean(cor_mean),2),.75)
    #### Larger bin size strongly predicts great inter-sample correlation ####
    testthat::expect_gte(
        cor(as.numeric(names(cor_mean)), cor_mean), .85
    )
    # saveRDS(cor_mats, file = "~/Downloads/compute_corr.cor_mats.rds")
})

#' Check \link[GenomicRanges]{GRanges} list columns
#' 
#' Check that at least one of the required columns is in 
#' a list of \link[GenomicRanges]{GRanges} objects.
#' Elements that do not meet this criterion will be dropped from the list.
#' 
#' @param grlist Named list of \link[GenomicRanges]{GRanges} objects.
#' @param target_cols A character vector of column names to search for.
#' 
#' @keywords internal
#' @importFrom GenomicRanges mcols
#' @returns Named list of \link[GenomicRanges]{GRanges} objects.
check_grlist_cols <- function(grlist,
                              target_cols){ 
    nms <- names(grlist)
    grlist <- lapply(seq_len(length(grlist)),  
                     FUN=function(i){
                         gr <- grlist[[i]]
                         if(all(!target_cols %in%
                                names(GenomicRanges::mcols(gr))) ){
                             msg <- paste(
                                 "WARNING:",
                                 paste0("'",names(grlist)[i],"'"),
                                 "missing threshold col,",
                                 "will be",
                                 "excluded from precision-recall",
                                 "calculations.")
                             message(msg)
                             return(NULL)
                         } else {
                             return(gr)
                         } 
                     }) 
    names(grlist) <- nms
    #### Remove NULL items ####
    grlist <- grlist[!unlist(lapply(grlist, is.null))]
    return(grlist)
}

read_picard <- function(path,
                        verbose=TRUE){
    
    messager("Reading picard report.",v=verbose)
    dat <- data.table::fread(path,
                              skip = "LIBRARY",
                              fill = TRUE,
                              nrows = 1)
    dat[,path:=path]
    return(dat)
}

#' Tidy peakfiles in GRanges
#'
#' This function filters peak files by removing peaks in blacklisted regions
#' and in non-standard chromosomes. It also checks that the input list of
#' peakfiles is named. If no names are provided, default file names will be
#' used.
#'
#' @param peaklist A named list of peak files as GRanges object.
#' Objects must be named and listed using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}
#' If not named, default names are assigned.
#' @param blacklist Peakfile specifying blacklisted regions as GRanges object.
#'
#' @return list of GRanges object
#' @export
#'
#' @importMethodsFrom IRanges subsetByOverlaps
#'
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peakfile GRanges object
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("hg19_blacklist") # blacklist region for hg19 genome
#'
#' ### Create Named Peaklist ###
#' peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)
#'
#' ### Run ###
#' peaklist_tidy <- tidy_peakfile(peaklist = peaklist,
#'                                blacklist = hg19_blacklist)
#'
tidy_peakfile <- function(peaklist, blacklist){
  ### check peaklist names ###
  peaklist <- check_list_names(peaklist)
  ### standardise peakfiles ###
  peaklist_tidy <- mapply(peaklist, FUN = function(file){
    # remove non-standard chromosomes
    sample <- tidy_chromosomes(file, keep.X = TRUE, keep.Y = TRUE)
    # remove blacklisted regions
    IRanges::subsetByOverlaps(sample,
                              blacklist,
                              invert = TRUE)
  })
  return(peaklist_tidy)
}

## ----setup_vignette-----------------------------------------------------------
library(EpiCompare)

## -----------------------------------------------------------------------------
data("encode_H3K27ac") # ENCODE ChIP-seq
data("CnT_H3K27ac") # CUT&Tag
data("CnR_H3K27ac") # CUT&Run
data("hg19_blacklist") # hg19 genome blacklist 
data("CnT_H3K27ac_picard") # CUT&Tag Picard summary output
data("CnR_H3K27ac_picard") # CUT&Run Picard summary output

## -----------------------------------------------------------------------------
peaklist <- list(CnT_H3K27ac, CnR_H3K27ac) # create list of peakfiles 
names(peaklist) <- c("CnT", "CnR") # set names 

## -----------------------------------------------------------------------------
# create list of Picard summary
picard <- list(CnT_H3K27ac_picard, CnR_H3K27ac_picard) 
names(picard) <- c("CnT", "CnR") # set names 

## -----------------------------------------------------------------------------
reference_peak <- list("ENCODE_H3K27ac" = encode_H3K27ac)

## -----------------------------------------------------------------------------
EpiCompare(peakfiles = peaklist,
           genome_build = "hg19",
           blacklist = hg19_blacklist,
           picard_files = picard,
           reference = reference_peak,
           upset_plot = FALSE,
           stat_plot = FALSE,
           chromHMM_plot = FALSE,
           chromHMM_annotation = "K562",
           chipseeker_plot = FALSE,
           enrichment_plot = FALSE,
           tss_plot = FALSE,
           interact = FALSE,
           save_output = FALSE,
           output_filename = "EpiCompare_test",
           output_timestamp = FALSE,
           output_dir = tempdir())

## ----Session_Info_vignette----------------------------------------------------
utils::sessionInfo()

#' Generate enrichment analysis plots
#'
#' This function runs KEGG and GO enrichment analysis of peak files
#' and generates dot plots.
#'
#' @param peaklist A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names will be assigned.
#' @param txdb A TxDb annotation object from Bioconductor.
#' @param pvalueCutoff P-value cutoff,
#'  passed to \link[clusterProfiler]{compareCluster}.
#' @param verbose Print messages.
#' @inheritParams EpiCompare
#' @inheritParams tss_plot
#' @return KEGG and GO dot plots
#'
#' @importFrom ChIPseeker annotatePeak
#' @import ggplot2
#' @export
#' @examples
#' ### Load Data ###
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("CnR_H3K27ac") # example peakfile GRanges object 
#' ### Create Named Peaklist ###
#' peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac) 
#' enrich_res <- plot_enrichment(peaklist = peaklist, pvalueCutoff=1,
#'                               tss_distance = c(-50,50)) 
plot_enrichment <- function(peaklist, 
                            txdb = NULL,
                            tss_distance = c(-3000, 3000),
                            pvalueCutoff = 0.05,
                            interact = FALSE,
                            verbose = TRUE){
  # devoptera::args2vars(plot_enrichment) 
  
  messager("--- Running plot_enrichment() ---",v=verbose)
  t1 <- Sys.time()
  ### Check deps ### 
  check_dep("clusterProfiler")  
  check_dep("org.Hs.eg.db")
  ### Check Peaklist Names ###
  peaklist <- check_list_names(peaklist)  
  ### Adjust Font Size ###
  font_size <- 11
  if(length(peaklist) > 6){
    font_size <- 8
  }
  ### Annotate Peak ###
  peak_annotated <- lapply(peaklist,
                           ChIPseeker::annotatePeak,
                           TxDb = txdb,
                           tssRegion = tss_distance,
                           verbose = FALSE)
  genes <- lapply(peak_annotated, function(i) as.data.frame(i)$geneId)
  ### KEGG ### 
  messager("+ Running clusterProfiler::compareCluster for KEGG.",v=verbose)
  compKEGG <- clusterProfiler::compareCluster(geneCluster = genes,
                                              fun = "enrichKEGG",
                                              pvalueCutoff = pvalueCutoff,
                                              pAdjustMethod = "BH")
  #### Check res ####
  if(is.null(compKEGG)){
    messager("WARNING: No enrichment identified with enrichKEGG.",
             "Skipping plot creation.",
             v=verbose)
    kegg_plot <- NULL
  } else {
    ### Generate KEGG Dotplot ###
    kegg_plot <- clusterProfiler::dotplot(compKEGG) +
      ggplot2::labs(x="") +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                         vjust = 1,
                                                         hjust = 1,
                                                         size = font_size))
    # Remove new line
    sample_names <- gsub('\n([0-9]*)','',kegg_plot$data$Cluster)
    kegg_plot$data$Cluster <- sample_names
  } 
  ### GO ###
  messager("+ Running clusterProfiler::compareCluster for GO.",v=verbose)
  compGO <- clusterProfiler::compareCluster(geneCluster = genes,
                                           OrgDb = "org.Hs.eg.db",
                                           fun = "enrichGO",
                                           pvalueCutoff = pvalueCutoff,
                                           pAdjustMethod = "BH")
  #### Check res ####
  if(is.null(compGO)){
    messager("WARNING: No enrichment identified with enrichKEGG.",
             "Skipping plot creation.",
             v=verbose)
    go_plot <- NULL
  } else {
    ### Generate GO Dotplot ###
    go_plot <- clusterProfiler::dotplot(compGO) +
      ggplot2::labs(x="") +
      ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                         vjust = 1,
                                                         hjust=1,
                                                         size = font_size))
    # Remove new line
    sample_names <- gsub('\n([0-9]*)','',go_plot$data$Cluster)
    go_plot$data$Cluster <- sample_names
  } 
  #### Report time #### 
  report_time(t1 = t1,
              func = "plot_enrichment",
              verbose = verbose)
  #### Return ####
  if(isTRUE(interact)){
    return(list(kegg_plot=as_interactive(kegg_plot), 
                kegg_data=compKEGG,
                go_plot=as_interactive(go_plot),
                go_data=compGO)
           )
  } else {
    return(list(kegg_plot=kegg_plot, 
                kegg_data=compKEGG,
                go_plot=go_plot,
                go_data=compGO)
           )
  }
}

#' Check cell lines
#' 
#' Check whether a list of cell lines matches any of those that are made
#' available through \code{EpiCompare}.
#' 
#' @param cell_lines A character vector of cell line names.
#' If \code{NULL} (default), will return names of all cell lines.
#' @param verbose Print messages.
#' @keywords internal
#' @returns Character vector, or NULL.
check_cell_lines <- function(cell_lines=NULL,
                             verbose=TRUE){
    opts <- c("K562",
      "Gm12878",
      "H1hesc",
      "Hepg2",
      "Hmec",
      "Hsmm",
      "Huvec",
      "Nhek",
      "Nhlf"
    )
    if(is.null(cell_lines)){
        ## Return all options
        return(opts)
    } else{
        ## return only matching options
        cl_match <- opts[tolower(opts) %in% tolower(cell_lines)]
        if(length(cl_match)>0){
            msg <- paste("Returning",length(cl_match),"matching cell line(s).")
            if(verbose) message(msg)
            return(cl_match)
        }else {
            smsg <- paste("cell_lines did not match any available options.",
                         "Must be one of:",
                         paste("\n -",paste0("'",opts,"'"),collapse = ""))
            if(verbose) message(smsg)
            return(NULL)
        }
    }
}

## ----setup, include=FALSE-----------------------------------------------------
#### Package name ####
pkg <- read.dcf("../DESCRIPTION", fields = "Package")[1]
library(pkg, character.only = TRUE)
## Docker containers must be lowercase
pkg <- tolower(pkg)
#### Username of DockerHub account ####
docker_user <- "neurogenomicslab"

## ----Session Info-------------------------------------------------------------
utils::sessionInfo()

#' Create ChIPseeker annotation plot
#'
#' This function annotates peaks using \code{ChIPseeker::annotatePeak}.
#' It outputs functional annotation of each peak file in a barplot.
#'
#' @param peaklist A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names will be assigned.
#' @param txdb A TxDb annotation object from Bioconductor.
#' @inheritParams EpiCompare
#' @inheritParams tss_plot
#' @returns ggplot barplot
#'
#' @export
#' @importFrom ChIPseeker annotatePeak
#' @importMethodsFrom ChIPseeker plotAnnoBar
#' @examples
#' ### Load Data ###
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("CnR_H3K27ac") # example peakfile GRanges object 
#' peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
#' my_plot <- plot_ChIPseeker_annotation(peaklist = peaklist, 
#'                                       tss_distance = c(-50,50))
plot_ChIPseeker_annotation <- function(peaklist, 
                                       txdb = NULL,
                                       tss_distance = c(-3000, 3000),
                                       interact = FALSE){
  # devoptera::args2vars(plot_ChIPseeker_annotation)
  message("--- Running plot_ChIPseeker_annotation() ---")
  ### Check Peaklist Names ###
  peaklist <- check_list_names(peaklist) 
  ### Annotate Features ###
  peak_annotated <- lapply(peaklist,
                           ChIPseeker::annotatePeak,
                           TxDb = txdb,
                           tssRegion = tss_distance,
                           verbose = FALSE)
  ### Create Plot ###
  anno_barplot <- ChIPseeker::plotAnnoBar(x = peak_annotated,
                                          xlab = "Percentage (%)")
  if(isTRUE(interact)){
    anno_barplot <- as_interactive(anno_barplot)
  }
  message("Done.")
  return(anno_barplot)
}

heatmap_heatmaply <- function(X,
                              draw_cellnote=TRUE,
                              cellnote_textposition="middle center",
                              ...){
  requireNamespace("heatmaply")
  requireNamespace("viridis")
  # https://github.com/talgalili/heatmaply/issues/11 
  heatmaply::heatmaply(X, 
                       colors=viridis::magma(n=256),
                       draw_cellnote=draw_cellnote,
                       cellnote_textposition=cellnote_textposition,
                       ...)
}

plot_precision_recall_prcurve <- function(plot_dat,
                                          color,
                                          shape,
                                          rows,
                                          cols,
                                          subtitle,
                                          interact) {
  messager("Plotting precision-recall curve.")
  gg <- ggplot2::ggplot(
    data = plot_dat,
    ggplot2::aes(
      x = .data$recall,
      y = .data$precision,
      group = .data$peaklist1,
      color = color
    )
  ) +
    ggplot2::geom_point(ggplot2::aes(size = 1 - .data$threshold, shape =
                                       shape), alpha = .8) +
    ggplot2::geom_line() +
    ggplot2::facet_grid(rows = rows, cols = cols) +
    ggplot2::ylim(0, 100) +
    ggplot2::xlim(0, 100) +
    ggplot2::labs(
      title = "precision-recall curves",
      subtitle = subtitle,
      x = "Recall\n(% reference peaks in sample peaks)",
      y = "Precision\n(% sample peaks in reference peaks)"
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      strip.background = ggplot2::element_rect(fill = "grey20"),
      strip.text = ggplot2::element_text(color = "white"),
      plot.margin = ggplot2::margin(.5, .5, .5, .5)
      # legend.title = ggplot2::element_text(size=7),
      # legend.spacing.y = ggplot2::unit(.001, units = "npc"),
      # legend.text=ggplot2::element_text(size=7)
    )
  if (isTRUE(interact))
    gg <- as_interactive(gg)
  return(gg)
}

read_trimgalore <- function(path,
                            verbose=TRUE){
    
    messager("Parsing trimgalore report.",v=verbose)
    l <- readLines(path, )
    lstart <- grep("=== Summary ===",l)
    lend <- grep("=== Adapter 1 ===",l)
    ldat <- grep("^$",l[seq(lstart+1,lend-1)], 
                 invert = TRUE, value = TRUE)
    ldat <- stringr::str_split(gsub("[ ]+"," ",ldat),":", simplify = TRUE) 
    dat_vals <- stringr::str_split(gsub("[%]|[)]|bp|[,]|[ ]","",ldat[,2]),
                                   "[(]", simplify = TRUE)
    dat <- data.table::data.table(
        metric=tolower(gsub("[ ]|[-]","_",gsub("[(]|[)]","",ldat[,1]))),
        value=as.integer(dat_vals[,1]),
        percent=as.numeric(dat_vals[,2])
    )
    dat[,path:=path]
    return(dat)
}

#' Generate Upset plot for overlapping peaks
#'
#' This function generates upset plot of overlapping peaks files
#' using the 
#' \href{https://cran.r-project.org/web/packages/ComplexUpset}{
#' \pkg{ComplexUpset}} package.
#' @param peaklist A named list of peak files as GRanges object.
#' Objects must be listed and named using \code{list()}.
#' e.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names are assigned.
#' @param verbose Print messages
#' @returns Upset plot of overlapping peaks.
#'
#' @importMethodsFrom IRanges findOverlaps
#' @importFrom GenomicRanges elementMetadata
#' @importFrom IRanges to from 
#'
#' @export
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # load example data
#' data("CnT_H3K27ac") # load example data 
#' peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac) 
#' my_plot <- overlap_upset_plot(peaklist = peaklist)
overlap_upset_plot <- function(peaklist,
                               verbose=TRUE){
    
  value <- NULL;
  
  t1 <- Sys.time()
  
  font_size <- 1
  messager("--- Running overlap_upset_plot() ---",v=verbose)
  #### Check package is available ####
  check_dep("ComplexUpset")
  check_dep("tidyr") 
  ### Check Peaklist Names ###
  peaklist <- check_list_names(peaklist)
  ### Set Metadata Colnames ###
  # So it doesn't interfere
  for(i in seq_len(length(peaklist))){
    my_label <- make.unique(rep("name",
                           ncol(GenomicRanges::elementMetadata(peaklist[[i]])))
                           )
    colnames(GenomicRanges::elementMetadata(peaklist[[i]])) <- my_label
  }
  ### Erase Names ###
  peaklist_names <- names(peaklist)
  names(peaklist) <- NULL
  ### Create Merged Dataset ###
  merged_peakfile <- do.call(c, peaklist)
  ### Calculate Overlap & Create Data Frame ###
  overlap_df <- NULL
  for(i in seq_len(length(peaklist))){
   overlap <- IRanges::findOverlaps(merged_peakfile, peaklist[[i]])
   sample_name <- rep(peaklist_names[i], length(IRanges::to(overlap)))
   df <- data.frame(peak=IRanges::from(overlap), sample=sample_name)
   unique_df <- unique(df)
   overlap_df <- rbind(overlap_df, unique_df)
  }
  ### Adjust Font Size ###  
  if(length(peaklist)>6){
    font_size <- 0.65
  }
  #### Create Upset Plot ###
  overlap_df$value <- 1
  overlap_df <- tidyr::spread(data = overlap_df, 
                              key = sample, 
                              value = value, 
                              fill=0) 
  
  base_annotations <- list(
    'Intersection size'=ComplexUpset::intersection_size(),
    'Intersection ratio'=ComplexUpset::intersection_ratio(
      text_mapping=ggplot2::aes(label=!!ComplexUpset::upset_text_percentage())
    )
  )
  plt <- ComplexUpset::upset(data = overlap_df,
                             intersect = peaklist_names,
                             base_annotations = base_annotations)
  # upset_plot <- UpSetR::upset(data = overlap_df, 
  #                             order.by = "freq",
  #                             mb.ratio = c(0.60, 0.40),
  #                             sets = peaklist_names,
  #                             number.angles = 30,
  #                             text.scale = c(1, 1, 1, font_size,
  #                                            font_size+0.15, font_size))
  report_time(t1 = t1,
              func="overlap_upset_plot",
              verbose = verbose)
  return(list(plot=plt,
              data=overlap_df))
}

#' Make file names
#'
#' Support function for \link[EpiCompare]{gather_files}.
#'
#' @keywords internal
#' @param paths Character vector of file paths.
#' @param verbose Print messages.
#' @returns Named character vector. 
#' @inheritParams gather_files
#' @importFrom stringr str_split
#' @importFrom stats setNames
gather_files_names <- function(paths,
                               type,
                               nfcore_cutandrun,
                               verbose=TRUE){
  messager("Constructing file names.",v=verbose)
  if(isTRUE(nfcore_cutandrun)){
    if(startsWith(type,"peaks")){
      # paths <- grep("03_peak_calling", paths, value = TRUE)
      list_names <- paste(
        basename(
          dirname(
            dirname(
              dirname(
                ## Some versions of nfcore/cutandrun include a seacr subdir
                gsub(file.path("","seacr",""),file.path("",""),paths)
              )
            )
          )
        ),
        stringr::str_split(basename(paths),"[.]",
                           simplify = TRUE)[,1],
        sep=".")
    } else if(type=="picard"){
      list_names <- paste(
        basename(
          dirname(
            dirname(
              dirname(
                dirname(
                  dirname(
                    dirname(
                      paths
                    )
                  )
                )
              )
            )
          )
        ),
        stringr::str_split(basename(paths),"[.]",
                           simplify = TRUE)[,1],
        sep=".")
    } else if(type=="multiqc"){
        list_names <- basename(dirname(dirname(paths)))
    }else if(startsWith(type,"bowtie")){
        list_names <- paste(
            basename(dirname(dirname(dirname(dirname(paths))))),
            stringr::str_split(basename(paths),"[.]",
                               simplify = TRUE)[,1],
            sep="."
        ) 
    }else if(type=="trimgalore"){
        list_names <- paste(
            basename(dirname(dirname(dirname(paths)))),
            stringr::str_split(basename(paths),"[.]",
                               simplify = TRUE)[,1],
            sep="."
        )
    }else if(startsWith(type,"bam")){
        list_names <- paste(
            basename(dirname(dirname(dirname(dirname(paths))))),
            stringr::str_split(basename(paths),"[.]",
                               simplify = TRUE)[,1],
            sep="."
        )
    }else{
        list_names <- make.unique(basename(paths))
    }
  } else {
    list_names <- make.unique(basename(paths))
  }
  return(stats::setNames(paths,list_names))
}

#' Download local file
#' 
#' Save an object as RDS and create a download button that can be rendered to
#'  Rmarkdown HTML pages.
#' Uses the package \pkg{downloadthis}.
#' @source \href{https://github.com/fmmattioni/downloadthis/issues/31}{
#' csv2 Issue.}
#' @source \href{https://github.com/fmmattioni/downloadthis/issues/16}{
#' Plotly Issue}
#' @param filename Name of the file to save.
#' @param outfile_dir Directory to save the file to.
#' @param verbose Print messages.
#' @inheritParams EpiCompare
#' @inheritParams base::saveRDS
#' @inheritParams downloadthis::download_file
#' @inheritParams downloadthis::download_this
#' @returns Download button as HTML text.
#' 
#' @export
#' @importFrom downloadthis download_file download_this
#' @examples 
#' button <- download_button(object=mtcars)
download_button <- function(object,
                            save_output = FALSE,
                            outfile_dir = NULL,
                            filename = NULL,
                            button_label = paste0("Download: ",
                                                  "<code>",filename,"</code>"),
                            output_extension = ".rds",
                            icon = "fa fa-save",
                            button_type = "success",
                            self_contained = TRUE,
                            add_download_button = TRUE,
                            verbose = TRUE){
  # devoptera::args2vars(download_button)
  
  if(isFALSE(add_download_button)) {
    messager("Download button disabled.",v=verbose)
    return(NULL)
  }
  #### Make sure object is a list ####
  if(!is.list(object)){
    object <- list(object) 
  } else if(methods::is(object,"plotly")){
    object <- list(plot=object)
  }
  #### With saving file ####
  if(isTRUE(save_output)){
    #### Make file path ####
    file <- if(!is.null(filename) && 
               exists("outfile_dir")){
      if(is.null(outfile_dir)) outfile_dir <- tempdir()
      file.path(outfile_dir,filename)
    } else {
      tempfile(fileext = ".rds") 
    }  
    #### Save file ####
    if(!file.exists(file)){
      messager("Saving file ==>",file,v=verbose)
      saveRDS(object = object,
              file = file)
    }
    #### Make button ####
    downloadthis::download_file(
      path = file,
      output_name = basename(file),
      button_label = button_label,
      button_type = button_type,
      has_icon = TRUE,
      icon = icon,
      self_contained = self_contained)
  #### Without saving file ####
  } else {
    #### Get output file name ####
    output_name <- if(is.null(filename)){
      "EpiCompare_download" 
    } else {
      gsub("\\.rds$","",basename(filename),ignore.case = TRUE)
    }
    #### Make button ####
    downloadthis::download_this(
      object,
      output_name = output_name,
      output_extension = output_extension,
      button_label = button_label,
      button_type = button_type,
      has_icon = TRUE,
      icon = icon,
      csv2 = FALSE,  
      self_contained = self_contained)
  } 
}

#' Remove non-standard chromosomes
#' 
#' Remove non-standard chromosomes from a list of 
#' \link[GenomicRanges]{GRanges} objects.
#' @param grlist Named list of \link[GenomicRanges]{GRanges} objects.
#' @param keep_chr Which chromosomes to keep.
#' @param verbose Print messages.
#' @returns Named list of \link[GenomicRanges]{GRanges} objects.
#' @keywords internal
#' @importFrom Seqinfo seqnames sortSeqlevels seqlevelsInUse
#' @importFrom GenomeInfoDb seqlevelsStyle
#' @importFrom BiocGenerics %in% 
remove_nonstandard_chrom <- function(grlist,
                                     keep_chr = paste0(
                                         "chr",c(seq_len(22),"X","Y")
                                     ),
                                     verbose = TRUE){
    #### Check if keep_chr is NULL ####
    if(is.null(keep_chr)) return(grlist)
    messager("Removing non-standard chromosomes.",v=verbose)
    #### Check if it's a single element ####
    tmp_list <- FALSE
    if(is_granges(grlist)){
        grlist <- list(gr1=grlist)
        tmp_list <- TRUE
    }
    grlist <- lapply(grlist, function(gr){ 
               suppressMessages(suppressWarnings(
                   GenomeInfoDb::seqlevelsStyle(gr) <- "UCSC"
               ))
               gr <- gr[BiocGenerics::`%in%`(Seqinfo::seqnames(gr),
                                             keep_chr)]
               Seqinfo::seqlevels(gr) <- Seqinfo::sortSeqlevels(
                   Seqinfo::seqlevelsInUse(gr)
               )
               #update seq lengths
               Seqinfo::seqlevels(
                   gr, pruning.mode="coarse") <- keep_chr
               return(gr)
           })
    if(tmp_list) grlist <- grlist[[1]]
    return(grlist)
}

#' Check workers
#' 
#' Assign parallel worker cores.
#' @param workers Number of cores to parallelise across 
#' (in applicable functions).
#' If \code{NULL}, will set to the total number of available cores minus 1.
#' @returns Integer
#' 
#' @export
#' @importFrom parallel detectCores
#' @examples 
#' workers <- check_workers()
check_workers <- function(workers=NULL){
  if(is.null(workers)) workers <- parallel::detectCores()-1
  return(workers)
}

#' Predict values 
#' 
#' Fit a model and make predictions from it.
#' @param df data.frame
#' @param values Values to make predictions from.
#' @param input_var Input variable column name.
#' @param predicted_var Predicted variable name.
#' @inheritParams predict_precision_recall
#' @returns data.frame
#' 
#' @keywords internal
#' @importFrom stats loess predict loess.control
predict_values <- function(df,
                           fun,
                           values, 
                           input_var,
                           predicted_var){
    
    if(length(values)>0){
        messager("+ Predicting",predicted_var,"from",
                 formatC(length(values),big.mark = ","),
                 input_var,"values.")
        #### Fit model ####
        mod <- fun(formula = paste(predicted_var,"~",input_var),
                   control=stats::loess.control(surface="direct"),
                   data = df) 
        #### Make predictions ####
        df2 <- data.frame(input_var=input_var,
                          predicted_var=predicted_var,
                          input=values)
        predicted_col <- "predicted"
        df2[[predicted_col]] <- stats::predict(object = mod, 
                                               df2$input) 
        #### Prevent negative predictions ####
        df2 <- set_min_max(df = df2,
                           colname = predicted_col)
    } else {
        mod <- NULL
        df2 <- NULL
    }
    return(df2)
}

#' Statistical significance of overlapping peaks
#'
#' This function calculates the statistical significance of overlapping/
#' non-overlapping peaks against a reference peak file. If the reference peak
#' file has the BED6+4 format (peak called by MACS2), the function generates a
#' series of box plots showing the distribution of q-values for sample peaks 
#' that are overlapping and non-overlapping with the reference. 
#' If the reference peak file does not have the BED6+4 format, the function uses 
#' \link[ChIPseeker]{enrichPeakOverlap}
#' from \pkg{ChIPseeker} package to calculate the statistical significance of
#' overlapping peaks only. In this case, please provide an annotation file as
#' a TxDb object.
#' @param reference A reference peak file as GRanges object.
#' @param peaklist A list of peak files as GRanges object.
#' Files must be listed and named using \code{list()}.
#' E.g. \code{list("name1"=file1, "name2"=file2)}.
#' If not named, default file names will be assigned.
#' @param txdb A TxDb annotation object from Bioconductor. This is
#' required only if the reference file does not have BED6+4 format.
#' @inheritParams EpiCompare
#' @inheritParams check_workers
#' @inheritParams base::signif
#' @inheritParams ChIPseeker::enrichPeakOverlap
#' @returns A named list. 
#' \itemize{
#' \item{"plot"}{boxplot/barplot showing the statistical significance of
#' overlapping/non-overlapping peaks.}
#' \item{"data"}{Plot data.}
#' } 
#'
#' @importMethodsFrom IRanges subsetByOverlaps
#' @import GenomicRanges
#' @importFrom methods is
#' @importFrom stats quantile
#' @importFrom data.table data.table rbindlist
#' @importFrom ChIPseeker enrichPeakOverlap
#' @import ggplot2
#'
#' @export
#' @examples
#' ### Load Data ###
#' data("encode_H3K27ac") # example peakfile GRanges object
#' data("CnT_H3K27ac") # example peakfile GRanges object
#' data("CnR_H3K27ac") # example peakfile GRanges object 
#' ### Create Named Peaklist & Reference ###
#' peaklist <- list('CnT'=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
#' reference <- list("ENCODE"=encode_H3K27ac) 
#' out <- overlap_stat_plot(reference = reference,
#'                          peaklist = peaklist,
#'                          workers = 1) 
overlap_stat_plot <- function(reference,
                              peaklist,
                              txdb = NULL,
                              interact = FALSE,
                              nShuffle = 50,
                              digits = 4,
                              workers = check_workers()){
  
  # devoptera::args2vars(overlap_stat_plot)
  message("--- Running overlap_stat_plot() ---")
  workers <- check_workers(workers = workers)
  # define variables
  qvalue <- tSample <- p.adjust <- NULL;
  # check that peaklist is named, if not, default names assigned
  peaklist <- check_list_names(peaklist)
  #### Validate reference list ####
  reference <- prepare_reference(reference = reference,
                                 max_elements = 1, 
                                 as_list = FALSE)
  reference <- clean_granges(reference)
  
  #### check if the file has BED6+4 format ####
  if(ncol(GenomicRanges::elementMetadata(reference)) %in% c(6,7)){
    main_df <- NULL
    # for each peakfile, obtain overlapping and unique peaks
    for (i in seq_len(length(peaklist))){
      # reference peaks found in sample peaks
      overlap <- IRanges::subsetByOverlaps(x = reference,
                                           ranges = peaklist[[i]])
      # reset names of metadata
      n <- 4
      my_label <- NULL
      for (l in seq_len(ncol(GenomicRanges::elementMetadata(overlap)))){
        label <- paste0("V",n)
        my_label <- c(my_label, label)
        n <- n + 1
      }
      colnames(GenomicRanges::mcols(overlap)) <- my_label

      # reference peaks not found in sample peaks
      unique <- IRanges::subsetByOverlaps(x = reference,
                                          ranges = peaklist[[i]],
                                          invert = TRUE)
      # reset names of metadata
      n <- 4
      my_label <- NULL
      for (l in seq_len(ncol(GenomicRanges::elementMetadata(unique)))){
        label <- paste0("V",n)
        my_label <- c(my_label, label)
        n <- n + 1
      }
      colnames(GenomicRanges::mcols(unique)) <- my_label
      # if no overlap, set q-value as 0 to avoid error
      # else, obtain q-value from field V9
      overlap_qvalue <- overlap$V9
      if (length(overlap) == 0) overlap_qvalue <- 0
      # create data frame of q-values for overlapping peaks
      sample <- names(peaklist)[i]
      group <- "overlap"
      overlap_df <- data.table::data.table(overlap_qvalue, sample, group)
      colnames(overlap_df) <- c("qvalue", "sample", "group")
      #
      unique_qvalue <- unique$V9
      if(length(unique) == 0){
        unique_qvalue <- 0
      }
      # create data frame of q-values for unique peaks
      group <- "unique"
      unique_df <- data.table::data.table(unique_qvalue, sample, group)
      colnames(unique_df) <- c("qvalue", "sample", "group")
      # combine two data frames
      sample_df <- data.table::rbindlist(list(overlap_df, unique_df))
      main_df <- data.table::rbindlist(list(main_df, sample_df))
    }
    # find value at 95th percentile
    max_val <- stats::quantile(main_df$qvalue, 0.95)
    # remove values greater than 95th quantile
    main_df <- main_df[qvalue<max_val,]

    # create paired boxplot for each peak file (sample)
    plt <- ggplot2::ggplot(main_df,
                                   ggplot2::aes(x=sample,
                                                y=qvalue,
                                                fill=group)) +
      ggplot2::geom_boxplot(outlier.shape = NA) +
      # ggplot2::scale_fill_viridis_d(alpha = .75, option = "magma") +
      ggplot2::labs(x="Sample",y="-log10(q-value)",fill="Sample") +
      ggplot2::theme_bw() +
      ggplot2::theme(axis.text.x =ggplot2::element_text(angle = 270,
                                                        vjust = 0,
                                                        hjust=0)) +
      ggplot2::coord_flip()
    if(isTRUE(interact)){
      plt <- as_interactive(plt, add_boxmode = TRUE)
    } 
    message("Done.")
    return(list(plot=plt, 
                data=main_df))
    
    #### for files not in BED6+4 format ####
    } else{
      # calculate significance of overlapping peaks using enrichPeakOverlap() 
      overlap_result <- ChIPseeker::enrichPeakOverlap(queryPeak = reference,
                                                      targetPeak = peaklist,
                                                      TxDb = txdb,
                                                      nShuffle = nShuffle,
                                                      pAdjustMethod = "BH",
                                                      chainFile = NULL,
                                                      mc.cores = workers,
                                                      verbose = FALSE)
      overlap_result$tSample <- names(peaklist) # set names with sample names
      percent_overlap <- c()
      # for each peakfile, calculate percentage overlap
      for (i in seq_len(nrow(overlap_result))){
        percent <- overlap_result[i,5]/overlap_result[i,3]*100
        percent_overlap <- c(percent_overlap, percent)
      }
      # add percentage overlap to a column
      overlap_result$percent_overlap <- percent_overlap 
      # create bar plot showing percentage overlap
      # and statistical significance of overlapping peaks
      plt <- ggplot2::ggplot(data=overlap_result,
                              ggplot2::aes(x=tSample,
                                           y=percent_overlap, 
                                           fill=p.adjust)) +
          ggplot2::geom_bar(stat="identity") + 
          ggplot2::theme_bw() +
          ggplot2::labs(x="Sample",y="Percentage overlap (%)",
                        fill="q-value") +
          ggplot2::scale_fill_viridis_c(alpha = .75, option = "magma") +
          ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45,
                                                             vjust = 1,
                                                             hjust = 1)) +
          ggplot2::ylim(0,100) +
          ggplot2::coord_flip()
      #### Add labels ####
      if(isFALSE(interact)){
        ## Only add labels when not interactive,
        ## to avoid plotly warning message 
        ## about not being able to translate the label/text geom
        plt <- plt + ggplot2::geom_label(
          ggplot2::aes(label=paste0("q-value=",
                                    signif(p.adjust,digits = digits))),
          position=ggplot2::position_dodge(width=0.9),
          fill=ggplot2::alpha("black",.8),
          color="white")
      }
      # return both plot and data frame
      message("Done.")
      if(isTRUE(interact)){
        plt <- as_interactive(plt, add_boxmode = TRUE)
      } 
      return(list(plot=plt, 
                  data=overlap_result))
  }
}

test_that("overlap_stat_plot works", {
  
  ### Load Data ###
  data("encode_H3K27ac") # example peakfile GRanges object
  data("CnT_H3K27ac") # example peakfile GRanges object
  data("CnR_H3K27ac") # example peakfile GRanges object
  ### Create Named Peaklist & Reference ###
  peaklist <- list('CnT'=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
  reference <- list("ENCODE"=encode_H3K27ac)
  out <- overlap_stat_plot(reference = reference,
                           peaklist = peaklist, 
                           workers = 1)
  testthat::expect_true(methods::is(out$plot,"gg"))
  testthat::expect_true(methods::is(out$data,"data.frame"))
  
  out <- EpiCompare::overlap_stat_plot(reference = reference, 
                                       peaklist = peaklist) 
  testthat::expect_true(methods::is(out$plot,"ggplot"))
  testthat::expect_equal(unique(out$data$sample),names(peaklist))
  testthat::expect_equal(unique(out$data$group),c("overlap","unique"))
})

#' Gather files
#'
#' Recursively find peak/picard files stored within subdirectories and import
#' them as a list of \link[GenomicRanges]{GRanges} objects.
#' 
#' For "peaks.stringent" files called with 
#' \href{https://github.com/FredHutch/SEACR}{SEACR}, column names will be
#' automatically added:
#' \itemize{
#' \item{total_signal : }{Total signal contained within denoted coordinates.}
#' \item{max_signal}{Maximum bedgraph signal attained at any base pair 
#'  within denoted coordinates.}
#' \item{max_signal_region}{ Region representing the farthest upstream 
#' and farthest downstream bases within the denoted coordinates 
#'  that are represented by the maximum bedgraph signal.}
#' }
#' 
#' @param dir Directory to search within.
#' @param type File type to search for. Options include:
#' \itemize{
#' \item{"<pattern>"}{Finds files matching an arbitrary regex pattern
#' specified by user.}
#' \item{"peaks.stringent"}{Finds files ending in "*.stringent.bed$"}
#' \item{"peaks.consensus"}{Finds files ending in "*.consensus.peaks.bed$"}
#' \item{"peaks.consensus.filtered"}{
#' Finds files ending in"*.consensus.peaks.filtered.awk.bed$"}
#' \item{"picard"}{Finds files ending in
#' "*.target.markdup.MarkDuplicates.metrics.txt$"}
#' }
#' @param nfcore_cutandrun Whether the files were generated by the
#' \href{https://nf-co.re/cutandrun}{nf-core/cutandrun} Nextflow pipeline.
#'  If \code{TRUE}, can use the standardised folder structure to
#'  automatically generate more descriptive file names with sample IDs.
#' @param return_paths Return only the file paths without actually reading them 
#' in as \link[GenomicRanges]{GRanges}. 
#' @param rbind_list Bind all objects into one.
#' @param verbose Print messages. 
#' @inheritParams check_workers
#' @inheritParams BiocParallel::MulticoreParam 
#' @returns A named list of \link[GenomicRanges]{GRanges} objects.
#'
#' @export 
#' @importFrom stats setNames
#' @importFrom stringr str_split
#' @importFrom data.table fread := rbindlist
#' @importFrom GenomicRanges GRangesList
#' @examples
#' #### Make example files ####
#' save_paths <- EpiCompare::write_example_peaks()
#' dir <- unique(dirname(save_paths))
#' #### Gather/import files ####
#' peaks <- EpiCompare::gather_files(dir=dir, 
#'                                   type="peaks.narrow",
#'                                   workers = 1)
gather_files <- function(dir,
                         type = "peaks.stringent",
                         nfcore_cutandrun = FALSE,
                         return_paths = FALSE,
                         rbind_list = FALSE,
                         workers = check_workers(),
                         verbose = TRUE){

  #### Parse type arg ####
  workers <- check_workers(workers = workers)
  type_key <- c(
    ## peak files
    "peaks.stringent"="*.stringent.bed$",
    "peaks.consensus"="*.consensus.peaks.bed$",
    "peaks.consensus.filtered"="*.consensus.peaks.filtered.awk.bed$",
    "peaks.pooled"="pooledPeak",
    "peaks.narrow"="narrowPeak",
    "peaks.broad"="broadPeak",
    ## picard files 
    # "picard"= "*.target.markdup.MarkDuplicates.metrics.txt$",
    "picard"= "^multiqc_picard*",
    ## multiQC files 
    # "multiqc"=c("^meta_table"),
    "multiqc"="multiqc_general_stats.txt$|meta_table_ctrl.csv",
    ## trimgalore files
    "trimgalore"="*.fastq.gz_trimming_report.txt$",
    ## bowtie files
    "bowtie.stats"="*\\.stats$",
    "bowtie.idxstats"="*\\.idxstats$",
    "bowtie.target.stats"=".*\\.stats$",
    "bowtie.target.idxstats"=".*\\.idxstats$",
    ## bam files
    "bam"="*.bam$",
    "bam.dedup"="*.dedup.bam$",
    "bam.dedup.sorted"="*.dedup.sorted.bam$",
    "bam.dedup.sorted.target"="*.target.dedup.sorted.bam$"
  )
  #### Check for known file types ####
  pattern <- if(all(type %in% names(type_key))) {
      type_key[tolower(type)]
  } else {
      type
  }
  if(all(is.na(pattern))){
    stop("type must be at least one of:\n",
         paste("-",c(names(type_key),"<regex query>"), collapse = "\n"))
  }
  #### Search for files recursively ####
  messager("Searching for",type,"files...",v=verbose)
  paths <- list.files(path = dir,
                      pattern = paste(unname(pattern), collapse = "|"),
                      recursive = TRUE,
                      full.names = TRUE, 
                      ignore.case = TRUE)
  #### Remove any R scripts ####
  paths <- grep("\\.R", paths, value = TRUE, invert = TRUE) 
  #### Omit duplicate files ####
  ## nfcore creates duplicates of same peak files 
  ## in different subfolders: "04_reporting" and "04_called_peaks".
  ## Omit one of these subfolders.
  if(isTRUE(nfcore_cutandrun) && 
     (type=="peaks")){
      paths <- paths[!grepl("04_reporting",paths)]
  }
  #### Report files found ####
  if(length(paths)==0) {
      msg <- "0 matching files identified. Returning NULL."
      messager(msg,v=verbose)
      return(NULL)
  }
  messager(formatC(length(paths),big.mark = ","),
          "matching files identified.",v=verbose)
  #### Construct names ####
  paths <- gather_files_names(paths=paths,
                              type=type,
                              nfcore_cutandrun=nfcore_cutandrun)
  if(isTRUE(return_paths)){
      messager("Returning paths.",v=verbose)
      return(paths)
  }
  #### Import files ####
  messager("Importing files.",v=verbose)  
  files <- bpplapply(X = paths, 
                     workers = workers,
                     FUN = function(x){ 
        if(type=="picard"){
            dat <- read_picard(path = x,
                               verbose = verbose)
        } else if(type=="multiqc"){
            dat <- read_multiqc(path = x,
                                verbose = verbose)
        } else if(startsWith(type,"bowtie")){
            dat <- read_bowtie(path = x,
                               verbose = verbose)
        } else if(type=="trimgalore"){
            dat <- read_trimgalore(path = x,
                                   verbose = verbose)
        } else if(startsWith(type,"bam")){
            dat <- read_bam(path = x,
                            verbose = verbose)
        } else {
            dat <- read_peaks(path = x,
                              type = type,
                              verbose = verbose)
        }
        return(dat) 
  }) 
  #### rbind into one object ####
  if(isTRUE(rbind_list)){
      messager("Binding all files into one.",v=verbose)
      if(startsWith(type,"peaks")){
          files <- unlist(GenomicRanges::GRangesList(files))
      } else {
          files <- data.table::rbindlist(files,
                                         use.names = TRUE,
                                         idcol = "batch",
                                         fill = TRUE)
      }
  }
  #### Report ####
  messager(formatC(length(files),big.mark = ","),"files retrieved.",v=verbose)
  return(files)
}

plot_precision_recall_f1 <- function(plot_dat,
                                     color,
                                     shape,
                                     rows,
                                     cols,
                                     initial_threshold,
                                     interact) {
  messager("Plotting F1.")
  ggf1 <- ggplot2::ggplot(data = plot_dat,
                          ggplot2::aes(
                            x = .data$threshold,
                            y = .data$F1,
                            group = .data$peaklist1,
                            color = color
                          )) +
    ggplot2::geom_point(ggplot2::aes(size = 1 - .data$threshold, shape =
                                       shape), alpha = .8) +
    ggplot2::facet_grid(rows = rows, cols = cols) +
    ggplot2::ylim(0, 100) +
    ggplot2::xlim(initial_threshold, 1) +
    ggplot2::labs(title = "F1 plot", y = "F1\n2 * (precision * recall) / (precision + recall)") +
    ggplot2::geom_line() +
    ggplot2::theme_bw() +
    ggplot2::theme(
      strip.background = ggplot2::element_rect(fill = "grey20"),
      strip.text = ggplot2::element_text(color = "white")
    )
  if (isTRUE(interact))
    ggf1 <- as_interactive(ggf1)
  return(ggf1)
}

#' Get \link[BiocParallel]{BiocParallel} parameters
#' 
#' Get (and optionally register) \link[BiocParallel]{BiocParallel} parameter 
#' (\code{BPPARAM}). \link[BiocParallel]{SnowParam} is the default function
#' as it tends to be more robust. However, because it doesn't work on Windows, 
#' this function automatically detected the Operating System and switches 
#' to  \link[BiocParallel]{SerialParam} as needed. 
#' @keywords internal
#' @param workers Number of threads to parallelize across. 
#' @param register_now Register the cores now with
#' \link[BiocParallel]{register} (\code{TRUE}),
#'  or simply return the \code{BPPARAM object} (default: \code{FALSE}). 
#' @param use_snowparam Whether to use
#'  \link[BiocParallel]{SnowParam} (default: \code{TRUE}) or
#'  \link[BiocParallel]{MulticoreParam} (\code{FALSE})
#'   when parallelising across multiple \code{workers}.
#' @inheritParams BiocParallel::SnowParam
#' @returns BPPARAM
get_bpparam <- function(workers,
                        progressbar=workers>1,
                        use_snowparam=TRUE,
                        register_now=FALSE){
    
    check_dep("BiocParallel")
    if(.Platform$OS.type == "windows"){
        BPPARAM <-  BiocParallel::SerialParam()
    } else {
        if(isTRUE(use_snowparam)){
            BPPARAM <- BiocParallel::SnowParam(workers = workers,
                                               progressbar = progressbar)
        } else{
            BPPARAM <- BiocParallel::MulticoreParam(workers = workers,
                                                    progressbar = progressbar)
        } 
    } 
    if(register_now){
        BiocParallel::register(BPPARAM = BPPARAM)
    } 
    return(BPPARAM)
}

#' Report header
#' 
#' Generate a header for \link[EpiCompare]{EpiCompare} reports generated using 
#' the \emph{EpiCompare.Rmd} template.
#' @returns Header string to be rendering within Rmarkdown file.
#' 
#' @export
#' @examples 
#' report_header()
report_header <- function(){ 
  paste0(
    "<div class='report-header'>",
    "<a href=",shQuote("https://github.com/neurogenomics/EpiCompare", type = "sh"),
    " target='_blank'>",
    "<code>EpiCompare</code>",
    "</a>",
    "<code>Report</code>",
    "<a href=",shQuote("https://doi.org/doi:10.18129/B9.bioc.EpiCompare", type = "sh"),
    " target='_blank'>",
    "<img src=",
    shQuote(system.file('hex','hex.png',package = 'EpiCompare'), type = "sh"),
    " height='100'",">",
    "</a>",
    "</div>"
  ) 
}

#' Prepare blacklist as GRanges
#'
#' Selects the appropriate blacklist in a variety of conditions.
#' @param blacklist_build Genome build of the blacklist.
#' Only used when \code{blacklist} is a user-supplied 
#' \link[GenomicRanges]{GRanges} object.
#' @inheritParams liftover_grlist
#' @returns A \link[GenomicRanges]{GRanges} objects of blacklisted 
#' genomic regions from the relevant genome build.
#'
#' @keywords internal
#' @importFrom methods is
prepare_blacklist <- function(blacklist,
                              output_build,
                              blacklist_build = NULL,
                              verbose = TRUE){ 
  
  #### Select blacklist ####
  if(is.null(blacklist)){
    blacklist <- if(output_build=="hg19"){
      messager("Auto-selecting blacklist:","hg19",v=verbose)
      get_pkg_data(name = "hg19_blacklist")
    } else if(output_build=="hg38"){
      messager("Auto-selecting blacklist:","hg38",v=verbose) 
      get_pkg_data(name = "hg38_blacklist")
    } else if(output_build=="mm10"){
      messager("Auto-selecting blacklist:","mm10",v=verbose)
      get_pkg_data(name = "mm10_blacklist") 
    } else if(output_build=="mm9"){
      messager("Auto-selecting blacklist:","mm9",v=verbose)  
      get_pkg_data(name = "mm9_blacklist") 
    } else {
      stopper("When blacklist is NULL, output_build must be one of:",
              paste("\n -",shQuote(c("hg19","hg38","mm10","mm9")),
                    collapse = ""))
    }
  ##### User-specified ####
  } else if(is.character(blacklist)){
    blacklist <- if(blacklist=="hg19_blacklist"){
      messager("Using blacklist:","hg19_blacklist",v=verbose) 
      get_pkg_data(name = "hg19_blacklist")
    } else if(blacklist=="hg38_blacklist"){
      messager("Using blacklist:","hg38_blacklist",v=verbose) 
      get_pkg_data(name = "hg38_blacklist")
    } else if(blacklist=="mm10_blacklist"){
      messager("Using blacklist:","mm10_blacklist",v=verbose) 
      get_pkg_data(name = "mm10_blacklist")
    } else if(blacklist=="mm9_blacklist"){
      messager("Using blacklist:","mm9_blacklist",v=verbose) 
      get_pkg_data(name = "mm9_blacklist")
    } else {
      stopper(
        "When blacklist is a character, output_build must be one of:",
              paste("\n -",shQuote(c("hg19_blacklist","hg38_blacklist",
                                     "mm10_blacklist","mm9_blacklist")),
                    collapse = ""))
    }
  #### User-supplied ####
  } else if (methods::is(blacklist,"GRanges")){
    messager("Using user-supplied blacklist.",v=verbose)
    if(is.null(blacklist_build)){
      stopper("When blacklist is a user-supplied GRanges object, ",
              "genome_build must include the element 'blacklist'")
    }
    blacklist <- liftover_grlist(grlist = blacklist,
                                 input_build = blacklist_build,
                                 output_build = output_build,
                                 keep_chr = NULL, 
                                 verbose = verbose)
  }
  #### Check if GRanges ####
  if(!methods::is(blacklist,"GRanges")){
    stopper("blacklist must be a GRanges object.")
  } 
  #### Return ####
  return(blacklist)
}

---
title: "`EpiCompare`: Getting started"
author: "<h4>Authors: <i>`r auths <- eval(parse(text = gsub('person','c',read.dcf('../DESCRIPTION', fields = 'Authors@R'))));paste(auths[names(auths)=='given'],auths[names(auths)=='family'], collapse = ', ')`</i></h4>" 
date: "<h4>Vignette updated: <i>`r format( Sys.Date(), '%b-%d-%Y')`</i></h4>"
csl: nature.csl
output:
  BiocStyle::html_document:
vignette: >
    %\VignetteIndexEntry{EpiCompare} 
    %\usepackage[utf8]{inputenc}
    %\VignetteEngine{knitr::rmarkdown} 
---

# Overview

The *EpiCompare* package is designed to facilitate the comparison of epigenomic 
datasets for quality control and benchmarking purposes. The package combines 
several downstream analysis tools for epigenomic data and generates a single 
report that collates all results of the analysis. This allows users to conduct 
downstream analysis of multiple epigenomic datasets simultaneously and compare
the results in a simple and efficient way. 

## Introduction

For many years, ChIP-seq has been the standard method for epigenomic profiling,
but it suffers from a host of limitations. Recently, many other epigenomic 
technologies (e.g. CUT&Run, CUT&Tag and TIP-seq etc.), designed to overcome 
these constraints, have been developed. To better understand the performance of 
these novel approaches, it is important that we systematically compare these 
technologies and benchmark against a “gold-standard”.

There are many tools in R (e.g. *ChIPseeker*) that can be used to conduct 
downstream analysis and comparison of epigenomic datasets. However, these are 
often scattered across different packages and difficult to use for researchers
with none or little computational experience. 

*EpiCompare* is designed to provide a simple and comprehensive way of analysing
and comparing epigenomic datasets. It combines many useful downstream analysis
tools, which can easily be controlled by users and it collates the results in a
single report. This allows researchers to systematically compare different
epigenomic technologies.

While the main functionality of *EpiCompare* is to contrast epigenomic
technologies, it can also be used to compare datasets generated using different
experimental conditions and data analysis workflows of one technology. This
can help researchers to establish a consensus regarding the optimal use of the 
method. 

Currently, *EpiCompare* supports human and mouse genomes (hg19, hg38, mm9 and
mm10).

# Data 

The *EpiCompare* package contains a small subset of histone mark H3K27ac profile
data obtained/generated from: 

* ENCODE (data accession: ENCFF044JNJ)
* CUT&Tag from Kaya-Okur et al., (2019). (PMID: 31036827)
* CUT&Run from Meers et al., (2019). (PMID: 31232687)

It also contains human genome hg19 and hg38 blacklisted regions obtained from 
ENCODE. The ENCODE blacklist includes regions of the genome that have anomalous 
and/or unstructured signals independent of the cell-line or experiment. Removal 
of ENCODE blacklist is recommended for quality measure.

These dataset will be used to showcase *EpiCompare* functionality

# Installation

To install the package, run the following: 

```R
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("EpiCompare") 
```

# Running *EpiCompare*

In this example analysis, we will compare CUT&Run and CUT&Tag of histone mark
H3K27ac against ENCODE ChIP-seq. 

## Load package and example datasets 

Once installed, load the package: 

```{r setup_vignette}
library(EpiCompare)
```

Load example datasets used in this analysis:

```{r}
data("encode_H3K27ac") # ENCODE ChIP-seq
data("CnT_H3K27ac") # CUT&Tag
data("CnR_H3K27ac") # CUT&Run
data("hg19_blacklist") # hg19 genome blacklist 
data("CnT_H3K27ac_picard") # CUT&Tag Picard summary output
data("CnR_H3K27ac_picard") # CUT&Run Picard summary output
```

## Prepare input data

### Peaklist

*EpiCompare* accepts datasets both as `GRanges` object and as paths to BED 
files. Peakfiles (`GRanges` or paths) that you would like to analyse must be
listed and named (see below). 

```R
# To import BED files as GRanges object
peak_GRanges <-ChIPseeker::readPeakFile("/path/to/peak/file.bed",as = "GRanges")
# EpiCompare also accepts paths (to BED files) as input 
peak_path <- "/path/to/BED/file1.bed"
# Create named peak list
peaklist <- list(peak_GRanges, peak_path)
names(peaklist) <- c("sample1", "sample2")
```

In this example, we will use built-in data, which have been converted into 
`GRanges` object previously (`CnT_H3K27ac` and `CnR_H3K27ac`). 

```{r}
peaklist <- list(CnT_H3K27ac, CnR_H3K27ac) # create list of peakfiles 
names(peaklist) <- c("CnT", "CnR") # set names 
```

### Blacklist 

ENCODE blacklist contains regions of the genome that have anomalous 
and/or unstructured signals independent of the cell-line or experiment. Removal 
of these regions from peakfiles is recommended for quality measure.

*EpiCompare* has three built-in blacklist files, hg19, hg38 and mm10, downloaded 
from ENCODE. Run `?hg19_blacklist` for more information.

In this example analysis, since all peakfiles (`encode_H3K27ac`, `CnT_H3K27ac`,
`CnR_H3K27ac`)  were generated using human genome reference build hg19, 
`hg19_blacklist` will be used. For hg38, use `data(hg38_blacklist)`.

Please ensure that you specify the correct blacklist. 

### Picard summary files

Note that this is *OPTIONAL*. If you want the report to include metrics on
DNA fragments (e.g. mapped fragments and duplication rate), please input summary
files from Picard. 

[Picard *MarkDuplicates*](https://broadinstitute.github.io/picard/command-line-overview.html#MarkDuplicates) can be used to mark duplicate reads that are found within the alignment. This
tool outputs a metrics file with the ending `.MarkDuplicates.metrics.txt`. To
import this text file into R as data frame, use:

```R
picard <- read.table("/path/to/Picard/.MarkDuplicates.metrics.txt", header = TRUE, fill = TRUE)
```

In this example. we will use built-in data, which have been converted into data
frame previously (`CnT_H3K27ac_picard` and `CnR_H3K27ac_picard`). The files
must be listed and named: 

```{r}
# create list of Picard summary
picard <- list(CnT_H3K27ac_picard, CnR_H3K27ac_picard) 
names(picard) <- c("CnT", "CnR") # set names 
```

### Reference file

This is OPTIONAL. If reference peak file is provided, `stat_plot` and 
`chromHMM_plot` of overlapping peaks are included in the report (see 
*Optional plots* section below).

Reference file must be listed and named. In this example, we will use built-in 
data (`encode_H3K27ac`), which has been converted to `GRanges` previously:

```{r}
reference_peak <- list("ENCODE_H3K27ac" = encode_H3K27ac)
```

### Output Directory 

When running `EpiCompare()`, please ensure that you specify `output_dir`. All
outputs (figures and HTML report) will be saved in the specified `output_dir`.

## Run EpiCompare

Running EpiCompare is done using the function, `EpiCompare()` . Users can choose
which analyses to run and include in the report by setting parameters to `TRUE` 
or `FALSE`.

```{r}
EpiCompare(peakfiles = peaklist,
           genome_build = "hg19",
           blacklist = hg19_blacklist,
           picard_files = picard,
           reference = reference_peak,
           upset_plot = FALSE,
           stat_plot = FALSE,
           chromHMM_plot = FALSE,
           chromHMM_annotation = "K562",
           chipseeker_plot = FALSE,
           enrichment_plot = FALSE,
           tss_plot = FALSE,
           interact = FALSE,
           save_output = FALSE,
           output_filename = "EpiCompare_test",
           output_timestamp = FALSE,
           output_dir = tempdir())
```

### Optional plots

By default, these plots will not be included in the report unless set `TRUE`.

* `upset_plot` : Upset plot showing the number of overlapping peaks between
samples. *EpiCompare* uses `UpSetR` package.  
* `stat_plot` : A `reference` peakfile must be included for this plot. The plot
displays distribution of statistical significance (q-values) of sample peaks
that are overlapping/non-overlapping with the `reference` dataset. 
* `chromHMM_plot` : [ChromHMM](http://compbio.mit.edu/ChromHMM/#:~:text=ChromHMM%20is%20software%20for%20learning,and%20spatial%20patterns%20of%20marks.)
annotation of peaks. If `reference` is provided, ChromHMM annotation of 
overlapping and non-overlapping peaks with the `reference` is also included in 
the report. 
* `chipseeker_plot` : `ChIPseeker` functional annotation of peaks.
* `enrichment_plot` : KEGG pathway and GO enrichment analysis of peaks.
* `tss_plot` : Peak frequency around (+/- 3000bp) transcriptional start site.
Note that it may take awhile to generate this plot for large sample sizes.

### Other options 

* `chromHMM_annotation` : Cell-line annotation for ChromHMM. Default is K562. 
Options are: 
  + "K562" = K-562 cells
  + "Gm12878" = Cellosaurus cell-line GM12878
  + "H1hesc" = H1 Human Embryonic Stem Cell
  + "Hepg2" = Hep G2 cell
  + "Hmec" = Human Mammary Epithelial Cell
  + "Hsmm" = Human Skeletal Muscle Myoblasts
  + "Huvec" = Human Umbilical Vein Endothelial Cells
  + "Nhek" = Normal Human Epidermal Keratinocytes
  + "Nhlf" = Normal Human Lung Fibroblasts
* `interact` : By default, all heatmaps (percentage overlap and 
ChromHMM heatmaps) in the report will be interactive. If set FALSE, all heatmaps
will be static. N.B. If `interact=TRUE`, interactive heatmaps will be saved as 
html files, which may take time for larger sample sizes.
* `output_filename` : By default, the report is named EpiCompare.html. You can 
specify the filename of the report here. 
* `output_timestamp` : By default FALSE. If TRUE, the filename of the report 
includes the date. 

## Output

*EpiCompare* outputs

* An HTML report consisting of three sections:
  + 1. **General Metrics**: Metrics on peaks (percentage of blacklisted and 
  non-standard peaks, and peak widths) and fragments (duplication rate) of 
  samples.
  + 2. **Peak Overlap**: Percentage and statistical significance of overlapping
  and non-overlapping peaks. Also includes upset plot.
  + 3. **Functional Annotation**: Functional annotation (ChromHMM, ChIPseeker 
  and enrichment analysis) of peaks. Also includes peak enrichment around TSS.
* `EpiCompare_file` containing all plots generated by EpiCompare if 
`save_output = TRUE`. 

Both outputs are saved in the specified `output_dir`. 

# Future Enhancements 

In the current version, *EpiCompare* only recognizes certain BED formats. We hope
to improve this. Moreover, if there are other downstream analysis tools that may
be suitable in *EpiCompare*, feel free to report this through 
[Github](https://github.com/neurogenomics/EpiCompare).  

# Code used to generate the example report

An example report comparing ATAC-seq, DNase-seq and ChIP-seq of K562 can be found 
[here](https://neurogenomics.github.io/EpiCompare/articles/example_report.html).

Code used to generate this:

<details>

```R
## Load data
# ATAC-seq data: https://www.encodeproject.org/files/ENCFF558BLC/ 
atac1_hg38 <- ChIPseeker::readPeakFile("/path/to/bed/ENCFF558BLC.bed", as="GRanges")
# Dnase-seq data: https://www.encodeproject.org/files/ENCFF274YGF/
dna1_hg38 <- ChIPseeker::readPeakFile("/path/to/bed/ENCFF274YGF.bed", as="GRanges")
# Dnase-seq data: https://www.encodeproject.org/files/ENCFF185XRG/
dna2_hg38 <- ChIPseeker::readPeakFile("/path/to/bed/ENCFF185XRG.bed", as="GRanges")
# ChIP-seq data: https://www.encodeproject.org/files/ENCFF038DDS/
chip_hg38 <- ChIPseeker::readPeakFile("/path/to/bed/ENCODE_H3K27ac_hg38_ENCFF038DDS.bed", as="GRanges")

## Peaklist
peaklist <- list("ATAC_ENCFF558BLC" = atac1_hg38_unique,
                 "Dnase_ENCFF274YGF" = dna1_hg38,
                 "ChIP_ENCFF038DDS" = chip_hg38)

## Reference
reference <- list("Dnase_ENCFF185XRG_reference"=dna2_hg38)

## Blacklist
data("hg38_blacklist")

## Run Epicompare
EpiCompare(peakfiles = peaklist,
           genome_build = "hg38",
           genome_build_output = "hg38",
           blacklist = hg38_blacklist,
           reference = reference,
           picard_files = NULL,
           upset_plot = T,
           stat_plot = T,
           save_output = F,
           chromHMM_plot = T,
           chromHMM_annotation = "K562",
           chipseeker_plot = T,
           enrichment_plot = T,
           tss_plot = T,
           precision_recall_plot =T,
           corr_plot = T,
           interact = T,
           output_dir = "/path/")
```

</details>

# Session Information

<details>

```{r Session_Info_vignette}
utils::sessionInfo()
```

</details> 

---
title: "Example report"
author: "<h4>Authors: <i>`r auths <- eval(parse(text = gsub('person','c',read.dcf('../DESCRIPTION', fields = 'Authors@R'))));paste(auths[names(auths)=='given'],auths[names(auths)=='family'], collapse = ', ')`</i></h4>" 
date: "<h4>Vignette updated: <i>`r format( Sys.Date(), '%b-%d-%Y')`</i></h4>"
output:
  BiocStyle::html_document:
vignette: >
    %\VignetteIndexEntry{example_report} 
    %\usepackage[utf8]{inputenc}
    %\VignetteEngine{knitr::rmarkdown} 
---
 

### `EpiArchives` is a public archive for interactive HTML reports generated by [`EpiCompare`](https://github.com/neurogenomics/EpiCompare) and the associated code used to create them.

# [Home](https://neurogenomics.github.io/EpiArchives)
## Reports

### [atac_dnase_chip_example](https://neurogenomics.github.io/EpiArchives/reports/atac_dnase_chip_example/EpiCompare.html)
- Analysis of ATAC-seq, DNase-seq and ChIP-seq data from K562 cells. All data comes from [ENCODE](https://www.encodeproject.org/). Only chromosome 1 is analysed in this example.
- [Code to reproduce](https://github.com/neurogenomics/EpiArchives/blob/master/reports/atac_dnase_chip_example/EpiCompare_code.R).

### [atac_dnase_example](https://neurogenomics.github.io/EpiArchives/reports/atac_dnase_example/EpiCompare.html)
- Analysis of ATAC-seq, DNase-seq data from K562 cells. All data comes from [ENCODE](https://www.encodeproject.org/).
- [Code to reproduce](https://github.com/neurogenomics/EpiArchives/blob/master/reports/atac_dnase_example/EpiCompare_code.R).

# Session Information

<details>

```{r Session_Info_vignette}
utils::sessionInfo()
```

</details> 

---
title: "hexSticker"
date: "<h4>Updated: <i>`r format( Sys.Date(), '%b-%d-%Y')`</i></h4>"
output:
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{hexSticker}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, echo=FALSE, include=TRUE}
pkg <- read.dcf(here::here("DESCRIPTION"), fields = "Package")[1]
description <- read.dcf(here::here("DESCRIPTION"), fields = "Description")[1]

# If you're using R<4.1.1, need this version of rvcheck
# devtools::install_version('rvcheck',version='0.1.8') 
library(hexSticker)
library(dplyr)
library(ggplot2)
library(ggimage)
# library(ggpattern)# remotes::install_github("coolbutuseless/ggpattern")
```

You can make awesome hex stickers for your R packages using:

- [hexSticker](https://github.com/GuangchuangYu/hexSticker)  
- [ggimage](https://github.com/GuangchuangYu/ggimage) 
lets you render images as data points.  
- [ggpattern](https://coolbutuseless.github.io/package/ggpattern/)
lets you fill objects with patterns or images.
- [magick](https://cran.r-project.org/web/packages/magick/vignettes/intro.html) 
modify PNGs.

# `r pkg`

## File path

Create file path to save hex sticker to.

```{r}
filename <- here::here("inst/hex/hex.png")
dir.create(dirname(filename), showWarnings = FALSE, recursive = TRUE)
```

## hexSticker 

```{r}    
s_size = .85
stick <- hexSticker::sticker( 
  subplot = here::here("inst/hex/genetic_scale.png"),
  #### Package name ####
  package = pkg, p_size=18, p_y = 1.5,
  #### Subplot #####
  s_x=1, s_y=.965,  s_height = s_size, s_width = s_size,
  #### Fill & border ####
  h_fill = "#07273f", h_color = "#007c7c", h_size = 2,
  #### Spotlight ####
  spotlight = TRUE, l_alpha = .3, l_width = 5,
  #### File output ####
  filename = filename, dpi = 300)
print(stick)
```

# Session Info 

<details> 

```{r Session Info}
utils::sessionInfo()
```

</details>  

---
title: "Docker/Singularity Containers" 
author: "<h4>Authors: <i>`r auths <- eval(parse(text = gsub('person','c',read.dcf('../DESCRIPTION', fields = 'Authors@R'))));paste(auths[names(auths)=='given'],auths[names(auths)=='family'], collapse = ', ')`</i></h4>" 
date: "<h4>Vignette updated: <i>`r format( Sys.Date(), '%b-%d-%Y')`</i></h4>"
output:
  BiocStyle::html_document: 
vignette: >
    %\VignetteIndexEntry{docker} 
    %\usepackage[utf8]{inputenc}
    %\VignetteEngine{knitr::rmarkdown}
---


```{r setup, include=FALSE}
#### Package name ####
pkg <- read.dcf("../DESCRIPTION", fields = "Package")[1]
library(pkg, character.only = TRUE)
## Docker containers must be lowercase
pkg <- tolower(pkg)
#### Username of DockerHub account ####
docker_user <- "neurogenomicslab"
```

# DockerHub 

`r pkg` is now available via 
[DockerHub](https://hub.docker.com/repository/docker/`r docker_user`/`r pkg`) 
as a containerised environment with Rstudio and 
all necessary dependencies pre-installed.

## Installation 

## Method 1: via Docker

First, [install Docker](https://docs.docker.com/get-docker/) 
if you have not already. 

Create an image of the [Docker](https://www.docker.com/) container 
in command line:  

```
docker pull `r docker_user`/`r pkg`
```

Once the image has been created, you can launch it with:  

```
docker run \
  -d \
  -e ROOT=true \
  -e PASSWORD="<your_password>" \
  -v ~/Desktop:/Desktop \
  -v /Volumes:/Volumes \
  -p 8787:8787 \
  `r docker_user`/`r pkg`
```  

### NOTES

* Make sure to replace `<your_password>` above with whatever you want your password to be.  
* Change the paths supplied to the `-v` flags for your particular use case. 
* The `-d` ensures the container will run in "detached" mode,
which means it will persist even after you've closed your command line session.  
* The username will be *"rstudio"* by default.  
* Optionally, you can also install the [Docker Desktop](https://www.docker.com/products/docker-desktop) 
to easily manage your containers.  

## Method 2: via Singularity

If you are using a system that does not allow Docker 
(as is the case for many institutional computing clusters), 
you can instead [install Docker images via Singularity](https://sylabs.io/guides/2.6/user-guide/singularity_and_docker.html).  

```
singularity pull docker://`r docker_user`/`r pkg`
```


## Usage 

Finally, launch the containerised Rstudio by entering the 
following URL in any web browser:
*http://localhost:8787/*

Login using the credentials set during the Installation steps.  


# Session Info 

<details> 

```{r Session Info}
utils::sessionInfo()
```

</details>  

<br>

---
title: "`r EpiCompare::report_header()`"
date: "`r format(Sys.Date(), '%b-%d-%Y')`"
output: 
  html_document: 
    toc: yes
    toc_float: yes
    code_folding: hide 
    number_sections: true
    css: custom.css
params:
  peakfiles: 
    value: NULL
  output_dir: 
    value: NULL
  genome_build:
    value: NULL
  genome_build_output:
    value: "hg19"
  blacklist: 
    value: NULL
  picard_files:
    value : NULL
  reference: 
    value: NULL
  upset_plot:
    value: FALSE
  stat_plot:
    value: FALSE
  chromHMM_plot:
    value: FALSE
  chromHMM_annotation:
    value: "K562"
  chipseeker_plot:
    value: FALSE
  enrichment_plot:
    value: FALSE
  tss_plot:
    value: FALSE
  tss_distance:
    value: c(-3000,3000)
  precision_recall_plot:
    value: FALSE
  n_threshold:
    value: 15
  corr_plot:
    value: FALSE  
  bin_size:
    value: 5000      
  interact:
    value: TRUE
  add_download_button:
    value: TRUE
  save_output:
    value: FALSE
  workers:
    value: 1
  error:
    value: FALSE
  debug:
    value: FALSE
---

```{r setup, warning=FALSE, message=FALSE}
knitr::opts_chunk$set(echo = TRUE, 
                      message = params$debug,
                      warning = params$debug,
                      cache = FALSE, 
                      error = params$error,
                      results = if(isTRUE(params$interact)) 'asis' else 'markup')
### Create Output Directory ###
# if save_output = TRUE
if(params$save_output){
  outfile_dir <- file.path(params$output_dir,"EpiCompare_file")
  if(!dir.exists(outfile_dir)){
    dir.create(outfile_dir, showWarnings = FALSE, recursive = TRUE)
  }
}

#### ------ Prepare genome builds ------ ####
# e.g. genome_build <- list(reference="hg19",peakfiles="hg38",blacklist="hg19")
# or... genome_build <- "hg19"
builds <- prepare_genome_builds(genome_build = params$genome_build,
                                blacklist = params$blacklist)
## Standardise all data to specified build 
output_build <- prepare_output_build(params$genome_build_output)

#### ------ Prepare peaklist(s) ------ ####
# and check that the list is named, if not default filenames are used 
peaklist <- prepare_peaklist(peaklist = params$peakfiles) 
peaklist <- liftover_grlist(grlist = peaklist, 
                            input_build = builds$peaklist,
                            output_build = output_build)

#### ------ Prepare reference(s) ------ ####
reference <- prepare_reference(reference = params$reference)
reference <- liftover_grlist(grlist = reference, 
                            input_build = builds$reference,
                            output_build = output_build)

#### ------ Prepare blacklist ------ ####
blacklist <- prepare_blacklist(blacklist = params$blacklist,
                               output_build = output_build,
                               blacklist_build = builds$blacklist) 

### Standardise peaklist(s) ###
peaklist <- tidy_peakfile(peaklist = peaklist,
                          blacklist = blacklist)
peaklist_tidy <- peaklist

### Standardise reference(s) ###
# and include in peaklist
reference_tidy <- reference
if (!is.null(reference)){
  reference_tidy <- tidy_peakfile(peaklist = reference,
                                  blacklist = blacklist)
  peaklist_tidy <- c(peaklist_tidy, reference_tidy)
}

### Obtain Genome Annotation ### 
txdb <- check_genome_build(genome_build = output_build)

### Dynamic Figure Height ###  
fig_height <- fig_length(default_size = 7,
                         number_of_items = length(peaklist_tidy),
                         max_items = 10)

needs_ref <- function(arg,
                      reference=NULL){ 
  if(isTRUE(arg)){
    if(is.null(reference)){
      cat("NOTE: This plot is not generated when reference=NULL.")
      return(FALSE)
    } else{
      return(TRUE)
    }
  } else {
    return(FALSE)
  }
}
```

[`EpiCompare`](https://github.com/neurogenomics/EpiCompare) compares epigenomic 
datasets for quality control and benchmarking purposes. 

# Summary {-}

## Table of Contents {-}

The report consists of three sections: 

1. [**General Metrics:**](#general_metrics) Metrics on peaks for each sample: % blacklisted and non-standard peaks, peak widths, fragments, duplication rates. 

2. [**Peak Overlap:**](#peak_overlap) Frequency, percentage, statistical significance of 
overlapping and non-overlapping peaks. This also includes upset, precision-recall, and correlation plots. 

3. [**Functional Annotation:**](#functional_annotation) Functional annotation (`ChromHMM` analysis, peak annotation, and enrichment analysis) of peaks. This also includes peak distributions around transcription start sites (TSS). 



## Input Datasets {-}

* Reference peakfile: `r names(reference_tidy)`
* Total of `r length(peaklist_tidy)` peak files: 
```{r list file names}
# print peak file names and numerate 
cat(paste0(" - File ",seq_len(length(names(peaklist_tidy))),": ",
        names(peaklist_tidy),
        collapse = "\n\n"))
```

Processed, filtered and lifted files for: `peaklist`, `reference`, `blacklist`

```{r download processed peaks} 
download_button(object = list(peaklist = peaklist,
                              reference = reference_tidy,
                              blacklist = blacklist),
                save_output = params$save_output, 
                filename = paste0("processed_peakfiles_",output_build), 
                outfile_dir = outfile_dir,
                add_download_button = TRUE) # Always include button
```

## Command {-}

The `EpiCompare` function call used to generate the report: 

```{r report_command}
cmd <- report_command(params = params, 
                      peaklist_tidy = peaklist_tidy,
                      reference_tidy = reference_tidy)
cat(cmd)
```




# General Metrics {#general_metrics .tabset}

## Peak Information 

### Column descriptions

* **PeakN before tidy**: Total number of peaks including those blacklisted and those 
in non-standard chromosomes. 
* **Blacklisted peaks removed (%)**: Percentage of blacklisted peaks in samples. 
e.g. [ENCODE hg19 blacklist](https://www.encodeproject.org/files/ENCFF000KJP/) 
includes regions in the hg19 genome that have anomalous and/or unstructured 
signals independent of the cell-line or experiment.
* **Non-standard peaks removed (%)**: Percentage of peaks in non-standard and/or 
mitochondrial chromosomes.
Identified using `BRGenomics::tidyChromosomes()`.  
* **PeakN after tidy**: Total number of peaks after removing those in blacklisted 
regions and non-standard chromosomes. 

:information: **Note**: All `EpiCompare` analyses conducted on the peak files after they have been *filtered* (i.e. blacklisted regions and non-standard chromosomes removed) and lifted (as needed).

<br>

```{r peak_info} 
peak_info_df <- peak_info(peaklist = peaklist,
                          blacklist = blacklist)
download_button(object = peak_info_df,
                filename = "peak_info_df", 
                output_extension = ".csv",
                add_download_button = params$add_download_button)
# Print table 
knitr::kable(peak_info_df, format = "markdown") 
save_output(save_output = params$save_output,
            file = peak_info_df,
            file_type = "data.frame",
            filename = "peak_info",
            outpath = outfile_dir) 
remove(peak_info_df)
```



## Fragment Information

Metrics on fragments is shown only if Picard summary is provided. 
See [manual](https://neurogenomics.github.io/EpiCompare/) for help. 

### Column descriptions

* **Mapped_Fragments**: Number of mapped read pairs in the file. 
* **Duplication_Rate**: Percentage of mapped sequence that is marked as duplicate.
* **Unique_Fragments**: Number of mapped sequence that is not marked as duplicate. 

<br>

```{r picard_files} 
if (!is.null(params$picard_files)){
  fragment_info_df <- fragment_info(picard_list = params$picard_files)
  download_button(object = fragment_info_df,
                  filename = "fragment_info",
                  output_extension = ".csv",
                  add_download_button = params$add_download_button)
  # Print data frame
  knitr::kable(fragment_info_df, format = "markdown") 
  save_output(save_output = params$save_output,
              file = fragment_info_df,
              file_type = "data.frame",
              filename = "fragment_info",
              outpath = outfile_dir) 
  remove(fragment_info_df)
}   
```



## Peak widths 

Distribution of peak widths in samples 

<br> 

```{r width_boxplot,fig.width=10,fig.height=fig_height} 
width_plot <- width_boxplot(peaklist = peaklist_tidy, 
                            interact = params$interact)
download_button(object = width_plot,
                filename = "width_boxplot",
                self_contained = params$interact,
                add_download_button = params$add_download_button)
width_plot$plot
# Save boxplot
save_output(save_output = params$save_output,
            file = width_plot$plot,
            file_type = "ggplot",
            filename = "width_plot",
            outpath = outfile_dir, 
            interactive = params$interact)
# Remove variable
remove(width_plot)
```



# Peak Overlap {#peak_overlap .tabset}

## Percentage Overlap  

Percentage of overlapping peaks between samples. 
Hover over heatmap for percentage values. 

The heatmap can be interpreted as follows: 

* **Recall**: *y-axis*: Percent of peaks in y-axis samples 
that overlap with peaks in x-axis samples. 
in the x-axis samples.  
* **Precision**: *x-axis*: Percent of peaks in x-axis samples 
that overlap with peaks in y-axis samples.  

<br>

```{r overlap_heatmap,fig.width=10,fig.height=fig_height} 
overlap_heatmap <- overlap_heatmap(peaklist = peaklist_tidy,
                                   interact = params$interact)
download_button(object = overlap_heatmap,
                filename = "overlap_heatmap", 
                self_contained = params$interact,
                add_download_button = params$add_download_button)
overlap_heatmap$plot

# Save output 
save_output(save_output = params$save_output,
            file = overlap_heatmap$plot,
            file_type = "ggplot",
            filename = "samples_percent_overlap",
            outpath = outfile_dir,
            interactive = params$interact)
# Delete variable
remove(overlap_heatmap)
```



## Upset Plot 

Upset plot of overlapping peaks between samples. See 
[here](https://jokergoo.github.io/ComplexHeatmap-reference/book/upset-plot.html)
on how to interpret the plot.   

```{r overlap_upset_plot,fig.width=10,fig.height=fig_height}
upset_plot <- NULL
if(isTRUE(params$upset_plot)){
  upset_plot <- overlap_upset_plot(peaklist = peaklist_tidy)
  download_button(object = upset_plot,
                  filename = "upset_plot",
                  add_download_button = params$add_download_button)
  save_output(save_output = params$save_output,
              file = upset_plot,
              file_type = "image",
              filename = "upset_plot",
              outpath = outfile_dir)
}
upset_plot
remove(upset_plot)
```



## Statistical Significance  

Depending on the format of the reference file, `EpiCompare` produces different 
plots: 

* **Reference dataset has BED6+4 format** (peak calling performed with [`MACS2`](https://github.com/macs3-project/MACS)): 
`EpiCompare` generates paired boxplot 
per sample showing the distribution of *-log10(q-value)* of reference peaks that 
are overlapping and non-overlapping with the sample dataset. 
* **Reference dataset does not have BED6+4 format**: `EpiCompare` generates a barplot 
of percentage of overlapping sample peaks with the reference. Statistical 
significance (*adjusted p-value*) is written above each bar. 

**Reference peakfile**: `r names(reference_tidy)` 

#### Keys

* **Overlap**: Sample peaks in Reference peaks 
* **Unique**: Sample peaks not in Reference peaks  

<br>

```{r overlap_stat_plot,fig.width=10,fig.height=fig_height} 
if (needs_ref(params$stat_plot, reference)){
  stat_plot <- overlap_stat_plot(reference = reference_tidy,
                                 peaklist = peaklist,
                                 txdb = txdb, 
                                 interact = params$interact)
  download_button(object = stat_plot,
                  filename = "overlap_stat_plot", 
                  button_label = "Download overlap stat plot",
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)
  stat_plot$plot
  # Save output 
  save_output(save_output = params$save_output,
              file = stat_plot$plot,
              file_type = "ggplot",
              filename = "stat_plot",
              outpath = outfile_dir, 
              interactive = params$interact)
  # Remove variables
  remove(stat_plot)
}
```



## Precision-Recall Curves

The first plot shows the balance between precision and recall across multiple peak calling stringency thresholds.  

- **Precision**: Number of sample peaks in reference.
- **Recall**: Number of reference peaks in sample. 

The second plot shows F1 score 
(a score that combines precision and recall) across the different peak calling stringency thresholds.  

- **F1**: `2*(precision*recall) / (precision+recall)`


```{r precision_recall_plot,fig.width=7,fig.height=fig_height}
pr_out <- NULL
if(needs_ref(params$precision_recall_plot, reference)){
    #### Create save path ####
    save_path <- if(isFALSE(params$save_output)){NULL}else{
        file.path(outfile_dir,"precision_recall.csv")
    }
    pr_out <- plot_precision_recall(peakfiles = peaklist, 
                                    reference = reference_tidy,
                                    n_threshold = params$n_threshold,
                                    workers = params$workers,
                                    show_plot = FALSE,
                                    verbose = FALSE,
                                    save_path = save_path, 
                                    interact = params$interact)
    download_button(object = pr_out,
                    filename = "precision_recall", 
                    self_contained = params$interact,
                    add_download_button = params$add_download_button)  
}
pr_out$precision_recall_plot
cat("\n\n")
pr_out$f1_plot
remove(pr_out)
```



## Correlation Plot

The correlation plot shows the correlation between the quantiles when the genome
is binned at a set size. These quantiles are based on the intensity of the peak,
dependent on the peak caller used (q-value for MACS2):

```{r Correlation Plot,fig.width=7,fig.height=fig_height}
cp_out <- NULL
if(isTRUE(params$corr_plot)){
    #### Create save path ####
    save_path <- if(isFALSE(params$save_output)){NULL}else{
        file.path(outfile_dir,"corr.csv.gz")
    }
    cp_out <- plot_corr(peakfiles = peaklist_tidy, 
                        # reference can be NULL
                        reference = reference_tidy,
                        genome_build = output_build,
                        bin_size = params$bin_size,
                        workers = params$workers,
                        show_plot = FALSE,
                        save_path = save_path,
                        interact = params$interact)  
    download_button(object = cp_out,
                    filename = "correlation_plot", 
                    self_contained = params$interact,
                    add_download_button = params$add_download_button)
}
cp_out$corr_plot
remove(cp_out)
```



# Functional Annotation {#functional_annotation}

## ChromHMM {.tabset}

[`ChromHMM`](http://compbio.mit.edu/ChromHMM/) annotates and characterises peaks into different chromatin states. 

`ChromHMM` annotations used in `EpiCompare` were obtained from
[here](http://hgdownload.cse.ucsc.edu/goldenPath/hg19/encodeDCC/wgEncodeBroadHmm/).

* Cell-type annotation file used in this analysis: __`r params$chromHMM_annotation`__


**ChromHMM annotation definitions:**

<details>

For more information on ChromHMM states, please see [here](https://www.nature.com/articles/nprot.2017.124#:~:text=ChromHMM%20helps%20to%20annotate%20the,annotation%20for%20each%20cell%20type.)

1. Active promoter: being transcribed  
2. Weak promoter: less transcriptional activity
3. Poised promoter: ready for transcriptional activity
4. Strong enhancer: activate transcription at high level
5. Strong enhancer: activate transcription at high level
6. Weak enhancer: activate transcription at low level
7. Weak enhancer: activate transcription at low level
8. Insulator: block transcription
9. Txn elongation: transcription elongation
10. Txn elongation: transcription elongation
11. Weak Txn: weak transcription
12. Repressed: decreased transcription
13. Heterochrom.Io: heterochromatin and/or low signal 
14. Repetitive CNV: repetitive or copy number variation
15. Repetitive CNV: repetitive or copy number variation

</details>

<br>

### All samples 

ChromHMM annotation of individual samples. 

```{r plot_chromHMM1, fig.width=9, fig.height=fig_height} 
samples_chromHMM <- NULL
if(isTRUE(params$chromHMM_plot)){
  # Get ChromHMM annotation file
  chromHMM_list <- get_chromHMM_annotation(cell_line = params$chromHMM_annotation)
  # Plot chromHMM
  samples_chromHMM <- plot_chromHMM(peaklist = peaklist_tidy,
                                    chromHMM_annotation = chromHMM_list,
                                    genome_build = output_build,
                                    interact = params$interact)
  download_button(object = samples_chromHMM,
                  filename = "samples_ChromHMM",
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)
  save_output(save_output = params$save_output,
              file = samples_chromHMM,
              file_type = "ggplot",
              filename = "samples_ChromHMM",
              outpath = outfile_dir,
              interactive = params$interact)
}
samples_chromHMM
remove(samples_chromHMM)
```

### **Overlap**: Sample peaks in Reference peaks 

Percentage of Sample peaks found in Reference peaks 
(**Reference peakfile: `r names(reference_tidy)`**)

```{r overlap_percent1, fig.width=9}
if(needs_ref(params$chromHMM_plot, reference)){
  # generate data frame of percentage overlap 
  sample_in_ref_df <- overlap_percent(peaklist1 = peaklist_tidy,
                                      peaklist2 = reference_tidy,
                                      invert = FALSE)
  download_button(object = sample_in_ref_df,
                  filename = "sample_in_ref_df",
                  output_extension = ".csv",
                  add_download_button = params$add_download_button)
  knitr::kable(sample_in_ref_df, format = "markdown") 
}
```

ChromHMM annotation of sample peaks found in reference peaks.   

```{r plot_chromHMM2,fig.width=9, fig.height=fig_height}
sample_in_ref_chromHMM <- NULL
if(needs_ref(params$chromHMM_plot, reference)){ 
  # Obtain overlapping peaks 
  sample_in_ref_list <- mapply(peaklist_tidy, FUN=function(file){
      IRanges::subsetByOverlaps(x = file, 
                                ranges = reference_tidy[[1]])
  }) 
  # Run ChromHMM
  sample_in_ref_chromHMM <- plot_chromHMM(peaklist = sample_in_ref_list,
                                          chromHMM_annotation = chromHMM_list,
                                          genome_build = output_build,
                                          interact = params$interact)
  download_button(object = sample_in_ref_chromHMM,
                filename = "sample_in_ref_chromHMM", 
                self_contained = params$interact,
                add_download_button = params$add_download_button) 
  save_output(save_output = params$save_output,
              file = sample_in_ref_chromHMM,
              file_type = "ggplot",
              filename = "sample_in_ref_ChromHMM",
              outpath = outfile_dir,
              interactive = params$interact)  
}
sample_in_ref_chromHMM
remove(sample_in_ref_chromHMM)
```

### **Overlap**: Reference peaks in Sample peaks 

Percentage of Reference peaks found in Sample peaks 
(**Reference peakfile: `r names(reference_tidy)`**)

```{r overlap_percent2, fig.width=9}
if (needs_ref(params$chromHMM_plot, reference)){
    # Data frame of overlapping peaks 
    ref_in_sample_df <- overlap_percent(peaklist1 = reference_tidy,
                                        peaklist2 = peaklist_tidy,
                                        invert = FALSE)
   download_button(object = ref_in_sample_df,
                  filename = "ref_in_sample_df",
                  output_extension = ".csv",
                  add_download_button = params$add_download_button)
   knitr::kable(ref_in_sample_df, format = "markdown")
}
```

ChromHMM annotation of reference peaks found in sample peaks.   

```{r plot_chromHMM3, fig.width=9, fig.height=fig_height}
ref_in_sample_chromHMM <- NULL
if (needs_ref(params$chromHMM_plot, reference)){
  # Subset overlapping peaks
  ref_in_sample_list <- mapply(peaklist_tidy, FUN = function(file){
    IRanges::subsetByOverlaps(x = reference_tidy[[1]], 
                              ranges = file)
    }) 
  # Plot ChromHMM
  ref_in_sample_chromHMM <- plot_chromHMM(peaklist = ref_in_sample_list,
                                          chromHMM_annotation = chromHMM_list,
                                          genome_build = output_build,
                                          interact = params$interact)
  download_button(object = ref_in_sample_chromHMM,
                  filename = "ref_in_sample_chromHMM", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)  
  save_output(save_output = params$save_output,
              file = ref_in_sample_chromHMM,
              file_type = "ggplot",
              filename = "ref_in_sample_ChromHMM",
              outpath = outfile_dir,
              interactive = params$interact) 
}
ref_in_sample_chromHMM
remove(ref_in_sample_chromHMM)
```

### **Unique**: Sample peaks not in Reference peaks 

Percentage of sample peaks not found in reference peaks 
(**Reference peakfile: `r names(reference_tidy)`**)

```{r overlap_percent3, fig.width=9}
if (needs_ref(params$chromHMM_plot, reference)){
  # Data frame of non-overlapping peaks 
  sample_not_in_ref_df <- overlap_percent(peaklist1 = peaklist_tidy,
                                          peaklist2 = reference_tidy,
                                          invert = TRUE)
  download_button(object = sample_not_in_ref_df,
                  filename = "sample_not_in_ref_df", 
                  output_extension = ".csv",
                  add_download_button = params$add_download_button)
  knitr::kable(sample_not_in_ref_df, format = "markdown")
}
```

ChromHMM annotation of sample peaks not found in reference peaks. 

```{r plot_chromHMM4, fig.width=9, fig.height=fig_height}
sample_not_in_ref_chromHMM <- NULL
if (needs_ref(params$chromHMM_plot, reference)){
  sample_not_in_ref_list <- mapply(peaklist_tidy, FUN = function(file){
    IRanges::subsetByOverlaps(x = file, 
                              ranges = reference_tidy[[1]], 
                              invert = TRUE)
    }) 
  # Run ChromHMM
  sample_not_in_ref_chromHMM<-plot_chromHMM(peaklist = sample_not_in_ref_list,
                                            chromHMM_annotation = chromHMM_list,
                                            genome_build = output_build,
                                            interact = params$interact)
  download_button(object = sample_not_in_ref_chromHMM,
                 filename = "sample_not_in_ref_chromHMM", 
                 self_contained = params$interact,
                 add_download_button = params$add_download_button)  
  save_output(save_output = params$save_output,
              file = sample_not_in_ref_chromHMM,
              file_type = "ggplot",
              filename = "sample_not_in_ref_ChromHMM",
              outpath = outfile_dir,
              interactive = params$interact)  
} 
sample_not_in_ref_chromHMM
remove(sample_not_in_ref_chromHMM)
```

### **Unique**: Reference peaks not in Sample peaks 

Percentage of reference peaks not found in sample peaks 
(**Reference peakfile: `r names(reference_tidy)`**)

```{r overlap_percent4, fig.width=9}
if (needs_ref(params$chromHMM_plot, reference)){
  # Data frame of non-overlapping peaks 
  ref_not_in_sample_df <- overlap_percent(peaklist1 = reference_tidy,
                                          peaklist2 = peaklist_tidy,
                                          invert = TRUE)
  download_button(object = ref_not_in_sample_df,
                  filename = "ref_not_in_sample_df", 
                  output_extension = ".csv",
                  add_download_button = params$add_download_button)
  knitr::kable(ref_not_in_sample_df, format = "markdown")
}
```

ChromHMM annotation of reference peaks not found in sample peaks. 

```{r plot_chromHMM5, fig.width=9, fig.height=fig_height}
ref_not_in_sample_chromHMM <- NULL
if (needs_ref(params$chromHMM_plot, reference)){
  # Subset unique peaks 
  ref_not_in_sample_list <- mapply(peaklist_tidy, FUN = function(file){
    IRanges::subsetByOverlaps(x = reference_tidy[[1]], 
                              ranges = file, 
                              invert = TRUE)
    })
  # Run ChromHMM
  ref_not_in_sample_chromHMM<-plot_chromHMM(peaklist = ref_not_in_sample_list,
                                            chromHMM_annotation = chromHMM_list,
                                            genome_build = output_build,
                                            interact = params$interact)
  download_button(object = ref_not_in_sample_chromHMM,
                  filename = "ref_not_in_sample_chromHMM", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)  
  save_output(save_output = params$save_output,
              file = ref_not_in_sample_chromHMM,
              file_type = "ggplot",
              filename = "ref_not_in_sample_ChromHMM",
              outpath = outfile_dir,
              interactive = params$interact) 
}
ref_not_in_sample_chromHMM
remove(ref_not_in_sample_chromHMM)
```



## Annotate Peaks 

`EpiCompare` uses `ChIPseeker::annotatePeak()` to annotate peaks with the nearest 
gene and genomic regions where the peak is located. The peaks are annotated 
with genes taken from human genome annotations (hg19 or hg38) distributed by 
[*Bioconductor*](https://bioconductor.org/).

```{r Annotate Peaks, fig.width=9} 
chipseeker_plot <- NULL
if(isTRUE(params$chipseeker_plot)){
  chipseeker_plot <- plot_ChIPseeker_annotation(
    peaklist = peaklist_tidy,
    txdb = txdb, 
    tss_distance = params$tss_distance,
    interact = params$interact)
  download_button(object = chipseeker_plot,
                  filename = "chipseeker_plot", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button) 
  save_output(save_output = params$save_output,
              file = chipseeker_plot,
              file_type = "ggplot",
              filename = "chipseeker_annotation",
              outpath = outfile_dir, 
              interactive = params$interact) 
}
chipseeker_plot
remove(chipseeker_plot)
```



## Functional Enrichment Analyses {.tabset}

`EpiCompare` performs KEGG pathway and GO enrichment analysis using 
`clusterProfiler`. `ChIPseeker::annotatePeak()` is first used to assign peaks 
to nearest genes. Biological themes amongst the genes are identified using 
ontologies (KEGG and GO). The peaks are annotated with genes taken from 
annotations of human genome (hg19 or hg38) provided by Bioconductor.


```{r plot_enrichment} 
enrichment_plots <- NULL
if (isTRUE(params$enrichment_plot)){
  enrichment_plots <- plot_enrichment(peaklist = peaklist_tidy,
                                      txdb = txdb, 
                                      tss_distance = params$tss_distance,
                                      interact = params$interact) 
  # Figure height
  max_terms <- max(
    length(unique(enrichment_plots$kegg_plot$data$Description)),
    length(unique(enrichment_plots$go_plot$data$Description))
  )
  fig_height <- fig_length(default_size = 10,
                           number_of_items = max_terms,
                           max_items = 20)
}
```



### KEGG

[Kyoto Encyclopedia of Genes and Genomes (KEGG)](https://www.genome.jp/kegg) enrichment results.

```{r KEGG_plot,fig.width=10,fig.height=fig_height}
if (isTRUE(params$enrichment_plot)){
  download_button(object = enrichment_plots$kegg_plot,
                  filename = "KEGG_plot", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)  
  save_output(save_output = params$save_output,
              file = enrichment_plots$kegg_plot,
              file_type = "ggplot",
              filename = "KEGG_analysis",
              outpath = outfile_dir, 
              interactive = params$interact)
}
enrichment_plots$kegg_plot
```



### GO 

[Gene Ontology (GO)](http://geneontology.org/) enrichment results.

**GeneRatio definition**:

<details>
*GeneRatio* is the number of observed genes divided by the number of expected genes
from each GO category. GO terms can be useful when assessing different biological
themes present in each epigenomic dataset. 
</details>

```{r GO_plot,fig.width=10,fig.height=fig_height}
if (isTRUE(params$enrichment_plot)){
  download_button(object = enrichment_plots$go_plot,
                  filename = "GO_plot", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button) 
  save_output(save_output = params$save_output,
              file = enrichment_plots$go_plot,
              file_type = "ggplot",
              filename = "GO_analysis",
              outpath = outfile_dir,
              interactive = params$interact)  
} 
enrichment_plots$go_plot
```

```{r}
remove(enrichment_plots)
```




## Peak Frequency around TSS 

This plots peaks that are mapping to transcriptional start sites (TSS). 
TSS regions are defined as the flanking sequence of the TSS sites.   

* By default, this function plots the frequency of peaks upstream (*-3000bp*) 
and downstream (*default: +3000bp*) of TSS. These ranges can be adjusted with 
the argument `EpCompare(tss_distance=)`. 

* The grey area around the main frequency line represents 
the 95% confidence interval estimated by bootstrapping. 


```{r tss_plots,fig.height=3,fig.width=10} 
tssplt <- NULL
if (isTRUE(params$tss_plot)){
  tssplt <- tss_plot(peaklist = peaklist_tidy,
                     txdb = txdb,
                     tss_distance = params$tss_distance,
                     workers = params$workers,
                     interact = params$interact)
  download_button(object = tssplt,
                  filename = "tss_plots", 
                  self_contained = params$interact,
                  add_download_button = params$add_download_button)
}
tssplt
remove(tssplt,p)
```

# Citation {-}

If you use `EpiCompare`, please cite: 

```{r} 
cat(utils::citation("EpiCompare")$textVersion)
```


# Session Info {-}

<details> 

```{r Session Info, results='markup'}
utils::sessionInfo()
```

</details> 
<hr>

---
title: "`r EpiCompare::report_header()`"
date: "`r format(Sys.Date(), '%b-%d-%Y')`"
output: 
  html_document: 
    toc: yes
    toc_float: yes
    code_folding: hide 
    number_sections: true
    css: custom.css
params:
  peakfiles: 
    value: NULL
  output_dir: 
    value: NULL
  genome_build:
    value: NULL
  genome_build_output:
    value: "hg19"
  blacklist: 
    value: NULL
  picard_files:
    value : NULL
  reference: 
    value: NULL
  upset_plot:
    value: FALSE
  stat_plot:
    value: FALSE
  chromHMM_plot:
    value: FALSE
  chromHMM_annotation:
    value: "K562"
  chipseeker_plot:
    value: FALSE
  enrichment_plot:
    value: FALSE
  tss_plot:
    value: FALSE
  tss_distance:
    value: c(-3000,3000)
  precision_recall_plot:
    value: FALSE
  n_threshold:
    value: 15
  corr_plot:
    value: FALSE  
  bin_size:
    value: 5000      
  interact:
    value: TRUE
  add_download_button:
    value: TRUE
  save_output:
    value: FALSE
  workers:
    value: 1
  error:
    value: FALSE
  debug:
    value: FALSE
---

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_cell_lines.R
\name{check_cell_lines}
\alias{check_cell_lines}
\title{Check cell lines}
\usage{
check_cell_lines(cell_lines = NULL, verbose = TRUE)
}
\arguments{
\item{cell_lines}{A character vector of cell line names.
If \code{NULL} (default), will return names of all cell lines.}

\item{verbose}{Print messages.}
}
\value{
Character vector, or NULL.
}
\description{
Check whether a list of cell lines matches any of those that are made
available through \code{EpiCompare}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap_heatmap.R
\name{overlap_heatmap}
\alias{overlap_heatmap}
\title{Generate heatmap of percentage overlap}
\usage{
overlap_heatmap(
  peaklist,
  interact = TRUE,
  draw_cellnote = TRUE,
  fill_diag = NA,
  verbose = TRUE
)
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names will be assigned.}

\item{interact}{Default TRUE. By default heatmap is interactive.
If FALSE, heatmap is static.}

\item{draw_cellnote}{Draw the numeric values within each heatmap cell.}

\item{fill_diag}{Fill the diagonal of the overlap matrix.}

\item{verbose}{Print messages.}
}
\value{
An interactive heatmap
}
\description{
This function generates a heatmap showing percentage of overlapping peaks
between peak files.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peakfile GRanges object
data("CnT_H3K27ac") # example peakfile GRanges object
### Create Named List ###
peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)
### Run ###
my_heatmap <- overlap_heatmap(peaklist = peaklist) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bpplapply.R
\name{bpplapply}
\alias{bpplapply}
\title{Wrapper for \link[BiocParallel]{bplapply}}
\usage{
bpplapply(
  X,
  FUN,
  apply_fun = parallel::mclapply,
  workers = check_workers(),
  progressbar = workers > 1,
  verbose = workers == 1,
  use_snowparam = TRUE,
  register_now = FALSE,
  ...
)
}
\arguments{
\item{X}{
    Any object for which methods \code{length}, \code{[}, and
    \code{[[} are implemented.
  }

\item{FUN}{
    The \code{function} to be applied to each element of \code{X}.
  }

\item{apply_fun}{Iterator function to use.}

\item{workers}{Number of threads to parallelize across.}

\item{progressbar}{
    \code{logical(1)} Enable progress bar (based on plyr:::progress_text).
  }

\item{verbose}{Print messages.}

\item{use_snowparam}{Whether to use
\link[BiocParallel]{SnowParam} (default: \code{TRUE}) or
\link[BiocParallel]{MulticoreParam} (\code{FALSE})
 when parallelising across multiple \code{workers}.}

\item{register_now}{Register the cores now with
\link[BiocParallel]{register} (\code{TRUE}),
 or simply return the \code{BPPARAM object} (default: \code{FALSE}).}

\item{...}{
  Arguments passed on to \code{\link[BiocParallel:bplapply]{BiocParallel::bplapply}}
  \describe{
    \item{\code{BPPARAM}}{
    An optional \code{\link[BiocParallel]{BiocParallelParam}} instance
    determining the parallel back-end to be used during evaluation, or a
    \code{list} of \code{BiocParallelParam} instances, to be applied in
    sequence for nested calls to \pkg{BiocParallel} functions.
  }
    \item{\code{BPREDO}}{A \code{list} of output from \code{bplapply} with one or
    more failed elements. When a list is given in \code{BPREDO},
    \code{bpok} is used to identify errors, tasks are rerun and inserted
    into the original results.
  }
    \item{\code{BPOPTIONS}}{
    Additional options to control the behavior of the parallel evaluation, see \code{\link[BiocParallel]{bpoptions}}.
  }
  }}
}
\value{
(Named) list.
}
\description{
Wrapper function for \link[BiocParallel]{bplapply} that automatically
handles issues with \pkg{BiocParallel} related to different OS platforms.
}
\examples{
X <- stats::setNames(seq_len(length(letters)), letters)
out <- bpplapply(X, print) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_nonstandard_chrom.R
\name{remove_nonstandard_chrom}
\alias{remove_nonstandard_chrom}
\title{Remove non-standard chromosomes}
\usage{
remove_nonstandard_chrom(
  grlist,
  keep_chr = paste0("chr", c(seq_len(22), "X", "Y")),
  verbose = TRUE
)
}
\arguments{
\item{grlist}{Named list of \link[GenomicRanges]{GRanges} objects.}

\item{keep_chr}{Which chromosomes to keep.}

\item{verbose}{Print messages.}
}
\value{
Named list of \link[GenomicRanges]{GRanges} objects.
}
\description{
Remove non-standard chromosomes from a list of 
\link[GenomicRanges]{GRanges} objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fig_length.R
\name{fig_length}
\alias{fig_length}
\title{Dynamic Figure Length Generator}
\usage{
fig_length(default_size, number_of_items, max_items)
}
\arguments{
\item{default_size}{The default figure length. Must be numeric.}

\item{number_of_items}{Number of peak files, or terms.}

\item{max_items}{Maximum number of peak files, or terms.}
}
\value{
Figure height/width. A number.
}
\description{
This function calculates the appropriate figure height depending on the
number of items.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_chromosomes.R
\name{tidy_chromosomes}
\alias{tidy_chromosomes}
\title{Remove odd chromosomes from GRanges objects}
\usage{
tidy_chromosomes(
  gr,
  keep.X = TRUE,
  keep.Y = TRUE,
  keep.M = FALSE,
  keep.nonstandard = FALSE,
  genome = NULL
)
}
\arguments{
\item{gr}{Any GRanges object, or any another object with associated
\code{seqinfo} (or a \code{Seqinfo} object itself). The object should
typically have a standard genome associated with it, e.g. \code{genome(gr)
<- "hg38"}. \code{gr} can also be a list of such GRanges objects.}

\item{keep.X, keep.Y, keep.M, keep.nonstandard}{Logicals indicating which
non-autosomes should be kept. By default, sex chromosomes are kept, but
mitochondrial and non-standard chromosomes are removed.}

\item{genome}{An optional string that, if supplied, will be used to set the
genome of \code{gr}.}
}
\value{
A GRanges object in which both ranges and \code{seqinfo} associated
  with trimmed chromosomes have been removed.
}
\description{
This convenience function removes non-standard, mitochondrial, and/or sex
chromosomes from any GRanges object.
}
\details{
This function is adapted from \code{tidyChromosomes} in the
\code{BRGenomics} package licensed under the Artistic License 2.0.
Original author: Mike DeBerardine <https://github.com/mdeber>

Standard chromosomes are defined using the
  \code{\link[GenomeInfoDb]{standardChromosomes}} function
  from the \pkg{GenomeInfoDb} package.
}
\examples{
# make a GRanges
chrom <- c("chr2", "chr3", "chrX", "chrY", "chrM", "junk")
gr <- GenomicRanges::GRanges(seqnames = chrom,
              ranges = IRanges::IRanges(start = 2*(1:6), end = 3*(1:6)),
              strand = "+",
              seqinfo = Seqinfo::Seqinfo(chrom))
Seqinfo::genome(gr) <- "hg38"

gr

tidy_chromosomes(gr)

tidy_chromosomes(gr, keep.M = TRUE)

tidy_chromosomes(gr, keep.M = TRUE, keep.Y = FALSE)

tidy_chromosomes(gr, keep.nonstandard = TRUE)

}
\seealso{
\code{\link[GenomeInfoDb]{standardChromosomes}}
}
\author{
Mike DeBerardine
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_ChIPseeker_annotation.R
\name{plot_ChIPseeker_annotation}
\alias{plot_ChIPseeker_annotation}
\title{Create ChIPseeker annotation plot}
\usage{
plot_ChIPseeker_annotation(
  peaklist,
  txdb = NULL,
  tss_distance = c(-3000, 3000),
  interact = FALSE
)
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names will be assigned.}

\item{txdb}{A TxDb annotation object from Bioconductor.}

\item{tss_distance}{A vector specifying the distance upstream and downstream
around transcription start sites (TSS). 
The default value is \code{c(-3000,3000)}; meaning peak frequency 
3000bp upstream and downstream of TSS will be displayed.}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}
}
\value{
ggplot barplot
}
\description{
This function annotates peaks using \code{ChIPseeker::annotatePeak}.
It outputs functional annotation of each peak file in a barplot.
}
\examples{
### Load Data ###
data("CnT_H3K27ac") # example peakfile GRanges object
data("CnR_H3K27ac") # example peakfile GRanges object 
peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
my_plot <- plot_ChIPseeker_annotation(peaklist = peaklist, 
                                      tss_distance = c(-50,50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/message_parallel.R
\name{message_parallel}
\alias{message_parallel}
\title{Message parallel}
\usage{
message_parallel(...)
}
\value{
A message
}
\description{
Send messages to console even from within parallel processes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_granges.R
\name{is_granges}
\alias{is_granges}
\title{Is an object of class GRanges}
\usage{
is_granges(obj)
}
\arguments{
\item{obj}{Any R object.}
}
\value{
Boolean.
}
\description{
Check whether an object is of the class \link[GenomicRanges]{GRanges}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_header.R
\name{report_header}
\alias{report_header}
\title{Report header}
\usage{
report_header()
}
\value{
Header string to be rendering within Rmarkdown file.
}
\description{
Generate a header for \link[EpiCompare]{EpiCompare} reports generated using 
the \emph{EpiCompare.Rmd} template.
}
\examples{
report_header()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CnR_H3K27ac_picard}
\alias{CnR_H3K27ac_picard}
\title{Example Picard duplication metrics file 2}
\format{
An object of class \code{data.frame} with 1 rows and 10 columns.
}
\source{
The code to prepare the .Rda file is:

\code{picard <- read.table("path/to/picard/duplication/output", 
header = TRUE, fill = TRUE)} \cr
\code{CnR_H3K27ac_picard <- picard[1,]} \cr
\code{usethis::use_data(CnR_H3K27ac_picard, overwrite = TRUE)} \cr
}
\usage{
data("CnR_H3K27ac_picard")
}
\description{
Duplication metrics output on CUT&Run H3K27ac file 
(sample accession: SRR8581604).
Raw sequences were aligned to hg19 genome and after, 
Picard was performed by Leyla Abbasova.
The duplication summary output generated by Picard 
was processed to reduce the size of data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/precision_recall_matrix.R
\name{precision_recall_matrix}
\alias{precision_recall_matrix}
\title{Create a precision-recall matrix}
\usage{
precision_recall_matrix(peaklist, fill_diag = NA, verbose = TRUE)
}
\arguments{
\item{fill_diag}{Fill the diagonal of the overlap matrix.}

\item{verbose}{Print messages.}
}
\value{
matrix
}
\description{
Converts a list of peak files to a symmetric matrix where 
the y-axis indicates precision and the x-axis indicates recall.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_chromHMM.R
\name{plot_chromHMM}
\alias{plot_chromHMM}
\title{Plot ChromHMM heatmap}
\usage{
plot_chromHMM(
  peaklist,
  chromHMM_annotation,
  genome_build,
  cell_line = NULL,
  interact = FALSE,
  return_data = FALSE
)
}
\arguments{
\item{peaklist}{A named \link[base]{list} of peak files as GRanges object.
If list is not named, default names will be assigned.}

\item{chromHMM_annotation}{ChromHMM annotation list.}

\item{genome_build}{The human genome reference build used to generate
peakfiles. "hg19" or "hg38".}

\item{cell_line}{If not \code{cell_line},
will replace \code{chromHMM_annotation}
 by importing chromHMM data for a given cell line using
 \link[EpiCompare]{get_chromHMM_annotation}.}

\item{interact}{Default TRUE. By default, the heatmaps are interactive.
If\code{FALSE}, the function generates a static ChromHMM heatmap.}

\item{return_data}{Return the plot data as in addition to the plot itself.}
}
\value{
ChromHMM heatmap, or a named list.
}
\description{
Creates a heatmap using outputs from ChromHMM using ggplot2.The function
takes a list of peakfiles, performs ChromHMM and outputs a heatmap. ChromHMM
annotation file must be loaded prior to using this function.
ChromHMM annotations are aligned to hg19, and will be automatically lifted
over to the \code{genome_build} to match the build of the \code{peaklist}.
}
\examples{
### Load Data ###
data("CnT_H3K27ac") # example dataset as GRanges object
data("CnR_H3K27ac") # example dataset as GRanges object 
### Create Named Peaklist ###
peaklist <- list(CnT=CnT_H3K27ac, CnR=CnR_H3K27ac) 
### Run ###
my_plot <- plot_chromHMM(peaklist = peaklist,
                         cell_line = "K562",
                         genome_build = "hg19") 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_interactive.R
\name{as_interactive}
\alias{as_interactive}
\title{As interactive}
\source{
\href{https://github.com/yihui/knitr/issues/926}{
GitHub Issue to check whether knitting}
}
\usage{
as_interactive(
  plt,
  to_widget = isTRUE(getOption("knitr.in.progress")),
  add_boxmode = FALSE
)
}
\arguments{
\item{plt}{ggplot object.}

\item{to_widget}{Convert to a widget so it works within Rmarkdown HTML files.
By default, this will be only be set to \code{TRUE} when being run within 
the context of \pkg{knitr} rendering.}

\item{add_boxmode}{Add extra \link[plotly]{layout} to enable dodged boxplots.}
}
\value{
A \link[plotly]{plotly} object or a \link[htmltools]{tagList} 
wrapping the \link[plotly]{plotly} object.
}
\description{
Convert a \link[ggplot2]{ggplot} object to \link[plotly]{plotly}, 
and enable it to be plotted within an Rmarkdown HTML file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_genome_build.R
\name{check_genome_build}
\alias{check_genome_build}
\title{Check genome build}
\usage{
check_genome_build(genome_build, type = "txdb")
}
\arguments{
\item{genome_build}{Genome build name.}

\item{type}{whether to fetch the txdb or bsgen reference data}
}
\value{
txdb or bsgen
}
\description{
Check that the genome build is valid and 
require specific reference datasets to be installed.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap_upset_plot.R
\name{overlap_upset_plot}
\alias{overlap_upset_plot}
\title{Generate Upset plot for overlapping peaks}
\usage{
overlap_upset_plot(peaklist, verbose = TRUE)
}
\arguments{
\item{peaklist}{A named list of peak files as GRanges object.
Objects must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names are assigned.}

\item{verbose}{Print messages}
}
\value{
Upset plot of overlapping peaks.
}
\description{
This function generates upset plot of overlapping peaks files
using the 
\href{https://cran.r-project.org/web/packages/ComplexUpset}{
\pkg{ComplexUpset}} package.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # load example data
data("CnT_H3K27ac") # load example data 
peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac) 
my_plot <- overlap_upset_plot(peaklist = peaklist)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gather_files.R
\name{gather_files}
\alias{gather_files}
\title{Gather files}
\usage{
gather_files(
  dir,
  type = "peaks.stringent",
  nfcore_cutandrun = FALSE,
  return_paths = FALSE,
  rbind_list = FALSE,
  workers = check_workers(),
  verbose = TRUE
)
}
\arguments{
\item{dir}{Directory to search within.}

\item{type}{File type to search for. Options include:
\itemize{
\item{"<pattern>"}{Finds files matching an arbitrary regex pattern
specified by user.}
\item{"peaks.stringent"}{Finds files ending in "*.stringent.bed$"}
\item{"peaks.consensus"}{Finds files ending in "*.consensus.peaks.bed$"}
\item{"peaks.consensus.filtered"}{
Finds files ending in"*.consensus.peaks.filtered.awk.bed$"}
\item{"picard"}{Finds files ending in
"*.target.markdup.MarkDuplicates.metrics.txt$"}
}}

\item{nfcore_cutandrun}{Whether the files were generated by the
\href{https://nf-co.re/cutandrun}{nf-core/cutandrun} Nextflow pipeline.
 If \code{TRUE}, can use the standardised folder structure to
 automatically generate more descriptive file names with sample IDs.}

\item{return_paths}{Return only the file paths without actually reading them 
in as \link[GenomicRanges]{GRanges}.}

\item{rbind_list}{Bind all objects into one.}

\item{workers}{Number of cores to parallelise across 
(in applicable functions).
If \code{NULL}, will set to the total number of available cores minus 1.}

\item{verbose}{Print messages.}
}
\value{
A named list of \link[GenomicRanges]{GRanges} objects.
}
\description{
Recursively find peak/picard files stored within subdirectories and import
them as a list of \link[GenomicRanges]{GRanges} objects.
}
\details{
For "peaks.stringent" files called with 
\href{https://github.com/FredHutch/SEACR}{SEACR}, column names will be
automatically added:
\itemize{
\item{total_signal : }{Total signal contained within denoted coordinates.}
\item{max_signal}{Maximum bedgraph signal attained at any base pair 
 within denoted coordinates.}
\item{max_signal_region}{ Region representing the farthest upstream 
and farthest downstream bases within the denoted coordinates 
 that are represented by the maximum bedgraph signal.}
}
}
\examples{
#### Make example files ####
save_paths <- EpiCompare::write_example_peaks()
dir <- unique(dirname(save_paths))
#### Gather/import files ####
peaks <- EpiCompare::gather_files(dir=dir, 
                                  type="peaks.narrow",
                                  workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_bowtie.R
\name{read_bowtie}
\alias{read_bowtie}
\title{Read bowtie}
\usage{
read_bowtie(path, verbose = TRUE)
}
\arguments{
\item{path}{Path to bowtie file.}

\item{verbose}{Print messages.}
}
\value{
\link[data.table]{data.table}
}
\description{
Read a bowtie file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mm10_blacklist}
\alias{mm10_blacklist}
\title{Mouse genome mm10 blacklisted regions}
\format{
An object of class \code{GRanges} of length 164.
}
\source{
The code to prepare the .Rda file is:

\code{## blacklisted regions were directly downloaded} \cr
\code{## from https://www.encodeproject.org/files/ENCFF547MET/} \cr
\code{mm10_blacklist <- 
ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
\code{usethis::use_data(mm10_blacklist, overwrite = TRUE)} \cr
}
\usage{
data("mm10_blacklist")
}
\description{
Obtained from \url{https://www.encodeproject.org/files/ENCFF547MET/}.
The ENCODE blacklist includes regions of the mm10 genome that have anomalous 
and/or unstructured signals independent of the cell-line or experiment. 
Removal of ENCODE blacklist is recommended for quality measure.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap_stat_plot.R
\name{overlap_stat_plot}
\alias{overlap_stat_plot}
\title{Statistical significance of overlapping peaks}
\usage{
overlap_stat_plot(
  reference,
  peaklist,
  txdb = NULL,
  interact = FALSE,
  nShuffle = 50,
  digits = 4,
  workers = check_workers()
)
}
\arguments{
\item{reference}{A reference peak file as GRanges object.}

\item{peaklist}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names will be assigned.}

\item{txdb}{A TxDb annotation object from Bioconductor. This is
required only if the reference file does not have BED6+4 format.}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}

\item{nShuffle}{shuffle numbers}

\item{digits}{integer indicating the number of decimal places
    (\code{round}) or significant digits (\code{signif}) to be used.
    For \code{round}, negative values are allowed (see \sQuote{Details}).}

\item{workers}{Number of threads to parallelize across.}
}
\value{
A named list. 
\itemize{
\item{"plot"}{boxplot/barplot showing the statistical significance of
overlapping/non-overlapping peaks.}
\item{"data"}{Plot data.}
}
}
\description{
This function calculates the statistical significance of overlapping/
non-overlapping peaks against a reference peak file. If the reference peak
file has the BED6+4 format (peak called by MACS2), the function generates a
series of box plots showing the distribution of q-values for sample peaks 
that are overlapping and non-overlapping with the reference. 
If the reference peak file does not have the BED6+4 format, the function uses 
\link[ChIPseeker]{enrichPeakOverlap}
from \pkg{ChIPseeker} package to calculate the statistical significance of
overlapping peaks only. In this case, please provide an annotation file as
a TxDb object.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peakfile GRanges object
data("CnT_H3K27ac") # example peakfile GRanges object
data("CnR_H3K27ac") # example peakfile GRanges object 
### Create Named Peaklist & Reference ###
peaklist <- list('CnT'=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
reference <- list("ENCODE"=encode_H3K27ac) 
out <- overlap_stat_plot(reference = reference,
                         peaklist = peaklist,
                         workers = 1) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/precision_recall.R
\name{precision_recall}
\alias{precision_recall}
\title{Compute precision-recall}
\usage{
precision_recall(
  peakfiles,
  reference,
  thresholding_cols = c("total_signal", "qValue", "Peak Score"),
  initial_threshold = 0,
  n_threshold = 20,
  max_threshold = 1,
  cast = TRUE,
  workers = 1,
  verbose = TRUE,
  save_path = tempfile(fileext = "precision_recall.csv"),
  ...
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{reference}{A named list containing reference peak file(s) as GRanges
 object. Please ensure that the reference file is listed and named
i.e. \code{list("reference_name" = reference_peak)}. If more than one
reference is specified, individual reports for each reference will be
generated. However, please note that specifying more than one reference can
take awhile. If a reference is specified, it enables two analyses: (1) plot
showing statistical significance of overlapping/non-overlapping peaks; and
(2) ChromHMM of overlapping/non-overlapping peaks.}

\item{thresholding_cols}{Depending on which columns are present, 
 \link[GenomicRanges]{GRanges} will 
 be filtered at each threshold according to one or more of the following:
\itemize{
 \item{"total_signal" : }{Used by the peak calling software 
 \href{https://github.com/FredHutch/SEACR}{SEACR}. 
 \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
 together or instead of "total_signal".}
 \item{"qValue"}{Used by the peak calling software 
 \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
 Should contain the negative log of the p-values after multiple 
 testing correction.}
 \item{"Peak Score" : }{
 Used by the peak calling software
  \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
}}

\item{initial_threshold}{Numeric threshold that was provided to SEACR 
(via the parameter \code{--ctrl}) when calling peaks without an IgG 
control.}

\item{n_threshold}{Number of thresholds to test.}

\item{max_threshold}{Maximum threshold to test.}

\item{cast}{Cast the data into a format that's more compatible with 
\pkg{ggplot2}.}

\item{workers}{Number of threads to parallelize across.}

\item{verbose}{Print messages.}

\item{save_path}{File path to save precision-recall results to.}

\item{...}{
  Arguments passed on to \code{\link[=bpplapply]{bpplapply}}
  \describe{
    \item{\code{apply_fun}}{Iterator function to use.}
    \item{\code{register_now}}{Register the cores now with
\link[BiocParallel]{register} (\code{TRUE}),
 or simply return the \code{BPPARAM object} (default: \code{FALSE}).}
    \item{\code{use_snowparam}}{Whether to use
\link[BiocParallel]{SnowParam} (default: \code{TRUE}) or
\link[BiocParallel]{MulticoreParam} (\code{FALSE})
 when parallelising across multiple \code{workers}.}
    \item{\code{progressbar}}{
    \code{logical(1)} Enable progress bar (based on plyr:::progress_text).
  }
    \item{\code{X}}{
    Any object for which methods \code{length}, \code{[}, and
    \code{[[} are implemented.
  }
    \item{\code{FUN}}{
    The \code{function} to be applied to each element of \code{X}.
  }
  }}
}
\value{
Overlap
}
\description{
Compute precision and recall using each \link[GenomicRanges]{GRanges}
 object in \code{peakfiles} as the "query" 
 against each \link[GenomicRanges]{GRanges} object in \code{reference}
 as the subject.
}
\examples{
data("CnR_H3K27ac")
data("CnT_H3K27ac")
data("encode_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
reference <- list("encode_H3K27ac" = encode_H3K27ac)

pr_df <- precision_recall(peakfiles = peakfiles,
                          reference = reference,
                          workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_example_peaks.R
\name{write_example_peaks}
\alias{write_example_peaks}
\title{Write example peaks}
\usage{
write_example_peaks(
  dir = file.path(tempdir(), "processed_results"),
  datasets = c("encode_H3K27ac", "CnT_H3K27ac", "CnR_H3K27ac")
)
}
\arguments{
\item{dir}{Directory to save peak files to.}

\item{datasets}{Example datasets from \pkg{EpiCompare} to write.}
}
\value{
Named vector of paths to saved peak files.
}
\description{
Write example peaks datasets to disk.
}
\examples{
save_paths <- EpiCompare::write_example_peaks()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_corr.R
\name{compute_corr}
\alias{compute_corr}
\title{Compute correlation matrix}
\usage{
compute_corr(
  peakfiles,
  reference = NULL,
  genome_build,
  keep_chr = NULL,
  drop_empty_chr = FALSE,
  bin_size = 5000,
  method = "spearman",
  intensity_cols = c("total_signal", "qValue", "Peak Score", "score"),
  return_bins = FALSE,
  fill_diag = NA,
  workers = check_workers(),
  save_path = tempfile(fileext = ".corr.csv.gz")
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{reference}{A named list containing reference peak file(s) as GRanges
 object. Please ensure that the reference file is listed and named
i.e. \code{list("reference_name" = reference_peak)}. If more than one
reference is specified, individual reports for each reference will be
generated. However, please note that specifying more than one reference can
take awhile. If a reference is specified, it enables two analyses: (1) plot
showing statistical significance of overlapping/non-overlapping peaks; and
(2) ChromHMM of overlapping/non-overlapping peaks.}

\item{genome_build}{The build of **all** peak and reference files to 
calculate the correlation matrix on. If all peak and reference files are not 
of the same build use 
\link[EpiCompare]{liftover_grlist} to convert them all before running. Genome
build should be one of hg19, hg38, mm9, mm10.}

\item{keep_chr}{Which chromosomes to keep.}

\item{drop_empty_chr}{Drop chromosomes that are not present in any of the 
\code{peakfiles} (default: \code{FALSE}).}

\item{bin_size}{Default of 100. Base-pair size of the bins created to measure
correlation. Use smaller value for higher resolution but longer run time and 
larger memory usage.}

\item{method}{Default spearman (i.e. non-parametric). A character string 
indicating which correlation coefficient (or covariance) is to be computed. 
One of "pearson", "kendall", or "spearman": can be abbreviated.}

\item{intensity_cols}{Depending on which columns are present, this
value will be used to get quantiles and ultimately calculate the 
correlations:
\itemize{
 \item{"total_signal" : }{Used by the peak calling software 
 \href{https://github.com/FredHutch/SEACR}{SEACR}. 
 \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
 together or instead of "total_signal".}
 \item{"qValue"}{Used by the peak calling software 
 \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
 Should contain the negative log of the p-values after multiple 
 testing correction.}
 \item{"Peak Score" : }{
 Used by the peak calling software
  \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
}}

\item{return_bins}{If \code{TRUE}, returns a named list
 with both the rebinned 
(standardised) peaks ("bin") and the correlation matrix ("cor").
If \code{FALSE} (default), returns only the correlation matrix (unlisted).}

\item{fill_diag}{Fill the diagonal of the overlap matrix.}

\item{workers}{Number of threads to parallelize across.}

\item{save_path}{Path to save a table of correlation results to.}
}
\value{
correlation matrix
}
\description{
Compute correlation matrix on all peak files.
}
\examples{
data("CnR_H3K27ac")
data("CnT_H3K27ac")
data("encode_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
reference <- list("encode_H3K27ac"=encode_H3K27ac)

#increasing bin_size for speed but lower values will give more granular corr
corr_mat <- compute_corr(peakfiles = peakfiles,
                         reference = reference,
                         genome_build = "hg19",
                         bin_size = 200000, 
                         workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CnT_H3K27ac}
\alias{CnT_H3K27ac}
\title{Example CUT&Tag peak file}
\format{
An object of class \code{GRanges} of length 1670.
}
\source{
The code to prepare the .Rda file from the raw peak file is:

\code{# sequences were directly downloaded from 
https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8383507} \cr
\code{# and peaks (BED file) were generated by Leyla Abbasova 
(Neurogenomics Lab, Imperial College London)} \cr
\code{CnT_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
\code{CnT_H3K27ac <- CnT_H3K27ac[seqnames(CnT_H3K27ac)== "chr1"]} \cr
\code{my_label <- 
c("name","score","strand","signalValue","pValue","qValue","peak")} \cr
\code{colnames(GenomicRanges::mcols(CnT_H3K27ac)) <- my_label} \cr
\code{usethis::use_data(CnT_H3K27ac)} \cr
}
\usage{
data("CnT_H3K27ac")
}
\description{
Human H3K27ac peak file generated with CUT&Tag using K562 cell-line from
Kaya-Okur et al., (2019). Human genome build hg19 was used.
Raw peak file (.BED) was obtained from GEO
(\url{https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8383507}).
Peak calling was performed by Leyla Abbasova using MACS2.
The peak file was then imported as an GRanges object.
Peaks located on chromosome 1 were subsetted to reduce the dataset size.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_enrichment.R
\name{plot_enrichment}
\alias{plot_enrichment}
\title{Generate enrichment analysis plots}
\usage{
plot_enrichment(
  peaklist,
  txdb = NULL,
  tss_distance = c(-3000, 3000),
  pvalueCutoff = 0.05,
  interact = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names will be assigned.}

\item{txdb}{A TxDb annotation object from Bioconductor.}

\item{tss_distance}{A vector specifying the distance upstream and downstream
around transcription start sites (TSS). 
The default value is \code{c(-3000,3000)}; meaning peak frequency 
3000bp upstream and downstream of TSS will be displayed.}

\item{pvalueCutoff}{P-value cutoff,
passed to \link[clusterProfiler]{compareCluster}.}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}

\item{verbose}{Print messages.}
}
\value{
KEGG and GO dot plots
}
\description{
This function runs KEGG and GO enrichment analysis of peak files
and generates dot plots.
}
\examples{
### Load Data ###
data("CnT_H3K27ac") # example peakfile GRanges object
data("CnR_H3K27ac") # example peakfile GRanges object 
### Create Named Peaklist ###
peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac) 
enrich_res <- plot_enrichment(peaklist = peaklist, pvalueCutoff=1,
                              tss_distance = c(-50,50)) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hg19_blacklist}
\alias{hg19_blacklist}
\title{Human genome hg19 blacklisted regions}
\format{
An object of class \code{GRanges} of length 411.
}
\source{
The code to prepare the .Rda file is:

\code{# blacklisted regions were directly downloaded} \cr
\code{# from https://www.encodeproject.org/files/ENCFF001TDO/} \cr
\code{hg19_blacklist <- 
ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
\code{usethis::use_data(hg19_blacklist, overwrite = TRUE)} \cr
}
\usage{
data("hg19_blacklist")
}
\description{
Obtained from \url{https://www.encodeproject.org/files/ENCFF001TDO/}.
The ENCODE blacklist includes regions of the hg19 genome that have anomalous
and/or unstructured signals independent of the cell-line or experiment.
Removal of ENCODE blacklist is recommended for quality measure.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_corr.R
\name{plot_corr}
\alias{plot_corr}
\title{Plot correlation of peak files}
\usage{
plot_corr(
  peakfiles,
  reference = NULL,
  genome_build,
  bin_size = 5000,
  keep_chr = NULL,
  drop_empty_chr = FALSE,
  method = "spearman",
  intensity_cols = c("total_signal", "qValue", "Peak Score", "score"),
  interact = FALSE,
  draw_cellnote = TRUE,
  fill_diag = NA,
  workers = check_workers(),
  show_plot = TRUE,
  save_path = tempfile(fileext = ".corr.csv.gz")
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{reference}{A named list containing reference peak file(s) as GRanges
 object. Please ensure that the reference file is listed and named
i.e. \code{list("reference_name" = reference_peak)}. If more than one
reference is specified, individual reports for each reference will be
generated. However, please note that specifying more than one reference can
take awhile. If a reference is specified, it enables two analyses: (1) plot
showing statistical significance of overlapping/non-overlapping peaks; and
(2) ChromHMM of overlapping/non-overlapping peaks.}

\item{genome_build}{The build of **all** peak and reference files to 
calculate the correlation matrix on. If all peak and reference files are not 
of the same build use 
\link[EpiCompare]{liftover_grlist} to convert them all before running. Genome
build should be one of hg19, hg38, mm9, mm10.}

\item{bin_size}{Default of 100. Base-pair size of the bins created to measure
correlation. Use smaller value for higher resolution but longer run time and 
larger memory usage.}

\item{keep_chr}{Which chromosomes to keep.}

\item{drop_empty_chr}{Drop chromosomes that are not present in any of the 
\code{peakfiles} (default: \code{FALSE}).}

\item{method}{Default spearman (i.e. non-parametric). A character string 
indicating which correlation coefficient (or covariance) is to be computed. 
One of "pearson", "kendall", or "spearman": can be abbreviated.}

\item{intensity_cols}{Depending on which columns are present, this
value will be used to get quantiles and ultimately calculate the 
correlations:
\itemize{
 \item{"total_signal" : }{Used by the peak calling software 
 \href{https://github.com/FredHutch/SEACR}{SEACR}. 
 \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
 together or instead of "total_signal".}
 \item{"qValue"}{Used by the peak calling software 
 \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
 Should contain the negative log of the p-values after multiple 
 testing correction.}
 \item{"Peak Score" : }{
 Used by the peak calling software
  \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
}}

\item{interact}{Default TRUE. By default heatmap is interactive.
If FALSE, heatmap is static.}

\item{draw_cellnote}{Draw the numeric values within each heatmap cell.}

\item{fill_diag}{Fill the diagonal of the overlap matrix.}

\item{workers}{Number of threads to parallelize across.}

\item{show_plot}{Show the plot.}

\item{save_path}{Path to save a table of correlation results to.}
}
\value{
list with correlation plot (corr_plot) and correlation matrix (data)
}
\description{
Plot correlation by binning genome and measuring correlation of peak quantile
ranking. This ranking is based on p-value or other peak intensity measure 
dependent on the peak calling approach.
}
\examples{
data("CnR_H3K27ac")
data("CnT_H3K27ac")
data("encode_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
reference <- list("encode_H3K27ac"=encode_H3K27ac)
## Increasing bin_size for speed here,
## but lower values will give more precise results (and lower correlations)
cp <- plot_corr(peakfiles = peakfiles,
                reference = reference,
                genome_build = "hg19",
                bin_size = 5000,
                workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_list_names.R
\name{check_list_names}
\alias{check_list_names}
\title{Check peaklist is named}
\usage{
check_list_names(peaklist, default_prefix = "sample")
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.}

\item{default_prefix}{Default prefix to use when creating names
for \code{peaklist}.}
}
\value{
named peaklist
}
\description{
This function checks whether the peaklist is named.
If not, default file names are assigned.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_peaklist.R
\name{prepare_peaklist}
\alias{prepare_peaklist}
\title{Prepare peaklist as GRanges}
\usage{
prepare_peaklist(peaklist, remove_empty = TRUE, as_grangeslist = FALSE)
}
\arguments{
\item{peaklist}{A named list of peaks as \link[GenomicRanges]{GRanges}
or paths to BED files.}

\item{remove_empty}{Remove any empty elements in the list.}

\item{as_grangeslist}{Convert output to class
\link[GenomicRanges]{GRangesList} before returning.}
}
\value{
A list of \link[GenomicRanges]{GRanges} objects
}
\description{
Prepare peaklist as GRanges
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/translate_genome.R
\name{translate_genome}
\alias{translate_genome}
\title{Translate genome}
\usage{
translate_genome(
  genome,
  style = c("UCSC", "Ensembl", "NCBI"),
  default_genome = NULL,
  omit_subversion = TRUE
)
}
\arguments{
\item{genome}{A character vector of genomes equivalent to UCSC version
    or Ensembl Assemblies}

\item{style}{A single value equivalent to "UCSC" or "Ensembl"
    specifying the output genome}

\item{default_genome}{Default genome build when \code{genome} is \code{NULL}.}

\item{omit_subversion}{Omit any subversion suffixes after the ".".}
}
\value{
Standardized genome build name as a character string.
}
\description{
Translate the name of a genome build from one format to another.
}
\examples{
genome <- translate_genome(genome="hg38", style="Ensembl")
genome2 <- translate_genome(genome="mm10", style="UCSC")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EpiCompare.R
\name{EpiCompare}
\alias{EpiCompare}
\title{Compare epigenomic datasets}
\usage{
EpiCompare(
  peakfiles,
  genome_build,
  genome_build_output = "hg19",
  blacklist = NULL,
  picard_files = NULL,
  reference = NULL,
  upset_plot = FALSE,
  stat_plot = FALSE,
  chromHMM_plot = FALSE,
  chromHMM_annotation = "K562",
  chipseeker_plot = FALSE,
  enrichment_plot = FALSE,
  tss_plot = FALSE,
  tss_distance = c(-3000, 3000),
  precision_recall_plot = FALSE,
  n_threshold = 20,
  corr_plot = FALSE,
  bin_size = 5000,
  interact = TRUE,
  add_download_button = FALSE,
  save_output = FALSE,
  output_filename = "EpiCompare",
  output_timestamp = FALSE,
  output_dir,
  display = NULL,
  run_all = FALSE,
  workers = 1,
  quiet = FALSE,
  error = FALSE,
  debug = FALSE
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{genome_build}{A named list indicating the genome build used to
generate each of the following inputs:
\itemize{
\item{"peakfiles" : }{Genome build for the \code{peakfiles} input.
Assumes genome build is the same for each element in the \code{peakfiles}
list.}
\item{"reference" : }{Genome build for the \code{reference} input.}
\item{"blacklist" : }{Genome build for the \code{blacklist} input.}
}
Example input list:\cr
 \code{genome_build = list(peakfiles="hg38",
 reference="hg19", blacklist="hg19")}\cr\cr
Alternatively, you can supply a single character string instead of a list.
This should \emph{only} be done in situations where all three inputs
(\code{peakfiles}, \code{reference}, \code{blacklist}) are of the same
genome build. For example:\cr
\code{genome_build = "hg19"}\cr\cr
Supported genome builds are: "hg19", "hg38", "mm9" and "mm10".}

\item{genome_build_output}{Genome build to standardise all inputs to.
Liftovers will be performed automatically as needed.
Default: "hg19".\cr\cr
\strong{Note:} Cross-species liftovers are supported.}

\item{blacklist}{A \link[GenomicRanges]{GRanges} object
 containing blacklisted genomic regions.
Blacklists included in \pkg{EpiCompare} are:
\itemize{
\item{\code{NULL} (default): }{Automatically selects the appropriate
 blacklist based on the \code{genome_build_output} argument.}
\item{"hg19_blacklist": }{Regions of hg19 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{hg19_blacklist}}
\item{"hg38_blacklist": }{Regions of hg38 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{hg38_blacklist}}
\item{"mm10_blacklist": }{Regions of mm10 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{mm10_blacklist}}
\item{"mm9_blacklist": }{Blacklisted regions of mm10 genome that have been
 lifted over from \link[EpiCompare]{mm10_blacklist}.
 \link[EpiCompare]{mm9_blacklist}}
\item{\code{<user_input>}: }{A custom user-provided blacklist in
\link[GenomicRanges]{GRanges} format.}
}}

\item{picard_files}{A list of summary metrics output from Picard.
Files must be in data.frame format and listed using \code{list()}
and named using \code{names()}.
To import Picard duplication metrics (.txt file)
 into R as data frame, use:\cr
\code{picard <- read.table("/path/to/picard/output",
 header = TRUE, fill = TRUE)}.}

\item{reference}{A named list containing reference peak file(s) as GRanges
 object. Please ensure that the reference file is listed and named
i.e. \code{list("reference_name" = reference_peak)}. If more than one
reference is specified, individual reports for each reference will be
generated. However, please note that specifying more than one reference can
take awhile. If a reference is specified, it enables two analyses: (1) plot
showing statistical significance of overlapping/non-overlapping peaks; and
(2) ChromHMM of overlapping/non-overlapping peaks.}

\item{upset_plot}{Default FALSE. If TRUE, the report includes upset plot of
overlapping peaks.}

\item{stat_plot}{Default FALSE. If TRUE, the function creates a plot showing
the statistical significance of overlapping/non-overlapping peaks.
Reference peak file must be provided.}

\item{chromHMM_plot}{Default FALSE. If TRUE, the function outputs ChromHMM
heatmap of individual peak files. If a reference peak file is provided,
ChromHMM annotation of overlapping and non-overlapping peaks
is also provided.}

\item{chromHMM_annotation}{ChromHMM annotation for ChromHMM plots.
Default K562 cell-line. Cell-line options are:
\itemize{
  \item "K562" = K-562 cells
  \item "Gm12878" = Cellosaurus cell-line GM12878
  \item "H1hesc" = H1 Human Embryonic Stem Cell
  \item "Hepg2" = Hep G2 cell
  \item "Hmec" = Human Mammary Epithelial Cell
  \item "Hsmm" = Human Skeletal Muscle Myoblasts
  \item "Huvec" = Human Umbilical Vein Endothelial Cells
  \item "Nhek" = Normal Human Epidermal Keratinocytes
  \item "Nhlf" = Normal Human Lung Fibroblasts
}}

\item{chipseeker_plot}{Default FALSE. If TRUE, the report includes a barplot
of ChIPseeker annotation of peak files.}

\item{enrichment_plot}{Default FALSE. If TRUE, the report includes dotplots
of KEGG and GO enrichment analysis of peak files.}

\item{tss_plot}{Default FALSE. If TRUE, the report includes peak count
frequency around transcriptional start site. Note that this can take awhile.}

\item{tss_distance}{A vector specifying the distance upstream and downstream
around transcription start sites (TSS). 
The default value is \code{c(-3000,3000)}; meaning peak frequency 
3000bp upstream and downstream of TSS will be displayed.}

\item{precision_recall_plot}{Default is FALSE. If TRUE,
creates a precision-recall curve plot and an F1 plot using
\link[EpiCompare]{plot_precision_recall}.}

\item{n_threshold}{Number of thresholds to test.}

\item{corr_plot}{Default is FALSE. If TRUE, creates a correlation plot across
all peak files using
\link[EpiCompare]{plot_corr}.}

\item{bin_size}{Default of 100. Base-pair size of the bins created to measure
correlation. Use smaller value for higher resolution but longer run time and 
larger memory usage.}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}

\item{add_download_button}{Add download buttons for each plot or dataset.}

\item{save_output}{Default FALSE. If TRUE, all outputs (tables and plots) of
the analysis will be saved in a folder (EpiCompare_file).}

\item{output_filename}{Default EpiCompare.html. If otherwise, the html report
will be saved in the specified name.}

\item{output_timestamp}{Default FALSE. If TRUE, date will be included in the
file name.}

\item{output_dir}{Path to where output HTML file should be saved.}

\item{display}{After completion, automatically display the HTML report file
 in one of the following ways:
\itemize{
\item{"browser" : }{Display the report in your default web browser.}
\item{"rsstudio" : }{Display the report in Rstudio.}
\item{NULL (default) : }{Do not display the report.}
}}

\item{run_all}{Convenience argument that enables all plots/features
(without specifying each argument manually)
 by overriding the default values.
 Default: \code{FALSE}.}

\item{workers}{Number of threads to parallelize across.}

\item{quiet}{An option to suppress printing during rendering from knitr,
pandoc command line and others. To only suppress printing of the last
"Output created: " message, you can set \code{rmarkdown.render.message} to
\code{FALSE}}

\item{error}{If \code{TRUE}, the Rmarkdown report will continue to render
even when some chunks encounter errors (default: \code{FALSE}).
Passed to \link[knitr]{opts_chunk}.}

\item{debug}{Run in debug mode, where are messages and warnings
are printed within the HTML report (default: \code{FALSE}).}
}
\value{
Path to one or more HTML report files.
}
\description{
This function compares and analyses multiple epigenomic datasets and outputs
an HTML report containing all results of the analysis. The report is mainly
divided into three sections: (1) General Metrics on Peakfiles,
(2) Peak Overlaps and (3) Functional Annotation of Peaks.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example dataset as GRanges object
data("CnT_H3K27ac") # example dataset as GRanges object
data("CnR_H3K27ac") # example dataset as GRanges object
data("CnT_H3K27ac_picard") # example Picard summary output
data("CnR_H3K27ac_picard") # example Picard summary output

#### Prepare Input ####
# create named list of peakfiles
peakfiles <- list(CnR=CnR_H3K27ac, CnT=CnT_H3K27ac)
# create named list of picard outputs
picard_files <- list(CnR=CnR_H3K27ac_picard, CnT=CnT_H3K27ac_picard)
# reference peak file
reference <- list("ENCODE" = encode_H3K27ac)

### Run EpiCompare ###
output_html <- EpiCompare(peakfiles = peakfiles,
           genome_build = list(peakfiles="hg19",
                               reference="hg19"),
           picard_files = picard_files,
           reference = reference,
           output_filename = "EpiCompare_test",
           output_dir = tempdir())
# utils::browseURL(output_html)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/liftover_grlist.R
\name{liftover_grlist}
\alias{liftover_grlist}
\title{Liftover peak list}
\usage{
liftover_grlist(
  grlist,
  input_build,
  output_build = "hg19",
  style = "UCSC",
  keep_chr = paste0("chr", c(seq_len(22), "X", "Y")),
  as_grangeslist = FALSE,
  merge_all = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{grlist}{A named list of \link[GenomicRanges]{GRanges} objects,
or simply a single unlisted \link[GenomicRanges]{GRanges} object.
Can perform liftover within species or across species.}

\item{input_build}{The genome build of \code{grlist}.}

\item{output_build}{Desired genome build for
\code{grlist} to be lifted over to.}

\item{style}{Chromosome style,
set by \link[GenomeInfoDb]{seqlevelsStyle}.
\itemize{
\item{"UCSC" : }{Uses the chromosome style "chr1".}
\item{"NCBI" : }{Uses the chromosome style "1"}
}}

\item{keep_chr}{Which chromosomes to keep.}

\item{as_grangeslist}{Return as a \link[GenomicRanges]{GRangesList}.}

\item{merge_all}{Merge all \link[GenomicRanges]{GRanges} into a single
\link[GenomicRanges]{GRanges} object.}

\item{verbose}{Print messages.}
}
\value{
Named list of lifted \link[GenomicRanges]{GRanges} objects.
}
\description{
Perform genome build liftover to one or more
 \link[GenomicRanges]{GRanges} objects at once.
}
\examples{
grlist <- list("gr1"=GenomicRanges::GRanges("4:1-100000"),
               "gr2"=GenomicRanges::GRanges("6:1-100000"),
               "gr3"=GenomicRanges::GRanges("8:1-100000"))

grlist_lifted <- liftover_grlist(grlist = grlist,
                                 input_build = "hg19",
                                 output_build="hg38")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap_percent.R
\name{overlap_percent}
\alias{overlap_percent}
\title{Calculate percentage of overlapping peaks}
\usage{
overlap_percent(
  peaklist1,
  peaklist2,
  invert = FALSE,
  precision_recall = TRUE,
  suppress_messages = TRUE
)
}
\arguments{
\item{peaklist1}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
If not named, default file names will be assigned.}

\item{peaklist2}{peaklist1 A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.}

\item{invert}{
    If \code{TRUE}, keep only the ranges in \code{x} that do \emph{not}
    overlap \code{ranges}.
  }

\item{precision_recall}{Return percision-recall results for all combinations 
of \code{peaklist1} (the "query") and \code{peaklist2} (the "subject"). 
See \link[IRanges]{subsetByOverlaps} for more details on this terminology.}

\item{suppress_messages}{Suppress messages.}
}
\value{
data frame
}
\description{
This function calculates the percentage of overlapping peaks and outputs
a table or matrix of results.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peakfile GRanges object
data("CnT_H3K27ac") # example peakfile GRanges object
data("CnR_H3K27ac") # example peakfile GRanges object

### Create Named Peaklist ###
peaks <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac)
reference_peak <- list("ENCODE"=encode_H3K27ac)

### Run ###
overlap <- overlap_percent(peaklist1=peaks,
                           peaklist2=reference_peak)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/width_boxplot.R
\name{width_boxplot}
\alias{width_boxplot}
\title{Peak width boxplot}
\usage{
width_boxplot(peaklist, interact = FALSE)
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.
Files must be named and listed using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}
}
\value{
A boxplot of peak widths.
}
\description{
This function creates boxplots showing the distribution of widths in
each peak file.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peaklist GRanges object
data("CnT_H3K27ac") # example peaklist GRanges object  
peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)  
my_plot <- width_boxplot(peaklist = peaklist) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bpparam.R
\name{get_bpparam}
\alias{get_bpparam}
\title{Get \link[BiocParallel]{BiocParallel} parameters}
\usage{
get_bpparam(
  workers,
  progressbar = workers > 1,
  use_snowparam = TRUE,
  register_now = FALSE
)
}
\arguments{
\item{workers}{Number of threads to parallelize across.}

\item{progressbar}{
    \code{logical(1)} Enable progress bar (based on plyr:::progress_text).
  }

\item{use_snowparam}{Whether to use
\link[BiocParallel]{SnowParam} (default: \code{TRUE}) or
\link[BiocParallel]{MulticoreParam} (\code{FALSE})
 when parallelising across multiple \code{workers}.}

\item{register_now}{Register the cores now with
\link[BiocParallel]{register} (\code{TRUE}),
 or simply return the \code{BPPARAM object} (default: \code{FALSE}).}
}
\value{
BPPARAM
}
\description{
Get (and optionally register) \link[BiocParallel]{BiocParallel} parameter 
(\code{BPPARAM}). \link[BiocParallel]{SnowParam} is the default function
as it tends to be more robust. However, because it doesn't work on Windows, 
this function automatically detected the Operating System and switches 
to  \link[BiocParallel]{SerialParam} as needed.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_values.R
\name{predict_values}
\alias{predict_values}
\title{Predict values}
\usage{
predict_values(df, fun, values, input_var, predicted_var)
}
\arguments{
\item{df}{data.frame}

\item{fun}{Function to fit the data with.}

\item{values}{Values to make predictions from.}

\item{input_var}{Input variable column name.}

\item{predicted_var}{Predicted variable name.}
}
\value{
data.frame
}
\description{
Fit a model and make predictions from it.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_min_max.R
\name{set_min_max}
\alias{set_min_max}
\title{Set min/max}
\usage{
set_min_max(df, colname, min_val = 0, max_val = 100)
}
\arguments{
\item{df}{data.frame}

\item{colname}{Column name to check.}

\item{min_val}{Minimum value.}

\item{max_val}{Maximum value.}
}
\value{
data.frame
}
\description{
Set the min/max values in a data.frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CnT_H3K27ac_picard}
\alias{CnT_H3K27ac_picard}
\title{Example Picard duplication metrics file 1}
\format{
An object of class \code{data.frame} with 1 rows and 10 columns.
}
\source{
The code to prepare the .Rda file is:

\code{picard <- read.table("path/to/picard/duplication/output", 
header = TRUE, fill = TRUE)]} \cr
\code{CnT_H3K27ac_picard <- picard[1,]} \cr
\code{usethis::use_data(CnT_H3K27ac_picard, overwrite = TRUE)} \cr
}
\usage{
data("CnT_H3K27ac_picard")
}
\description{
Duplication metrics output of CUT&Tag H3K27ac file 
(sample accession: SRR8581604).
Raw sequences were aligned to hg19 genome and Picard 
was performed by Leyla Abbasova.
The duplication summary output generated by Picard was processed 
to reduce the size of data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_peakfile.R
\name{tidy_peakfile}
\alias{tidy_peakfile}
\title{Tidy peakfiles in GRanges}
\usage{
tidy_peakfile(peaklist, blacklist)
}
\arguments{
\item{peaklist}{A named list of peak files as GRanges object.
Objects must be named and listed using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}
If not named, default names are assigned.}

\item{blacklist}{Peakfile specifying blacklisted regions as GRanges object.}
}
\value{
list of GRanges object
}
\description{
This function filters peak files by removing peaks in blacklisted regions
and in non-standard chromosomes. It also checks that the input list of
peakfiles is named. If no names are provided, default file names will be
used.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peakfile GRanges object
data("CnT_H3K27ac") # example peakfile GRanges object
data("hg19_blacklist") # blacklist region for hg19 genome

### Create Named Peaklist ###
peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)

### Run ###
peaklist_tidy <- tidy_peakfile(peaklist = peaklist,
                               blacklist = hg19_blacklist)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_granges.R
\name{clean_granges}
\alias{clean_granges}
\title{Clean GRanges}
\usage{
clean_granges(
  gr,
  nono_cols = c("seqnames", "ranges", "strand", "seqlevels", "seqlengths", "isCircular",
    "start", "end", "width", "element")
)
}
\arguments{
\item{gr}{A \link[GenomicRanges]{GRanges} object.}

\item{nono_cols}{Problematic columns to search for and remove (if present).}
}
\value{
Cleaned \link[GenomicRanges]{GRanges} object.
}
\description{
Remove columns from the metadata (\code{GenomicRanges::mcols}) that
conflicts with \link[GenomicRanges]{GRanges} conventions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_precision_recall.R
\name{plot_precision_recall}
\alias{plot_precision_recall}
\title{Plot precision-recall curves}
\usage{
plot_precision_recall(
  peakfiles,
  reference,
  thresholding_cols = c("total_signal", "qValue", "Peak Score"),
  initial_threshold = 0,
  n_threshold = 20,
  max_threshold = 1,
  workers = check_workers(),
  plot_f1 = TRUE,
  subtitle = NULL,
  color = "peaklist1",
  shape = color,
  rows = "peaklist2",
  cols = NULL,
  interact = FALSE,
  show_plot = TRUE,
  save_path = tempfile(fileext = "precision_recall.csv"),
  verbose = TRUE
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{reference}{A named list containing reference peak file(s) as GRanges
 object. Please ensure that the reference file is listed and named
i.e. \code{list("reference_name" = reference_peak)}. If more than one
reference is specified, individual reports for each reference will be
generated. However, please note that specifying more than one reference can
take awhile. If a reference is specified, it enables two analyses: (1) plot
showing statistical significance of overlapping/non-overlapping peaks; and
(2) ChromHMM of overlapping/non-overlapping peaks.}

\item{thresholding_cols}{Depending on which columns are present, 
 \link[GenomicRanges]{GRanges} will 
 be filtered at each threshold according to one or more of the following:
\itemize{
 \item{"total_signal" : }{Used by the peak calling software 
 \href{https://github.com/FredHutch/SEACR}{SEACR}. 
 \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
 together or instead of "total_signal".}
 \item{"qValue"}{Used by the peak calling software 
 \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
 Should contain the negative log of the p-values after multiple 
 testing correction.}
 \item{"Peak Score" : }{
 Used by the peak calling software
  \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
}}

\item{initial_threshold}{Numeric threshold that was provided to SEACR 
(via the parameter \code{--ctrl}) when calling peaks without an IgG 
control.}

\item{n_threshold}{Number of thresholds to test.}

\item{max_threshold}{Maximum threshold to test.}

\item{workers}{Number of threads to parallelize across.}

\item{plot_f1}{Generate a plot with the F1 score vs. threshold as well.}

\item{subtitle}{Plot subtitle.}

\item{color}{Variable to color data points by.}

\item{shape}{Variable to set data point shapes by.}

\item{rows, cols}{A set of variables or expressions quoted by
\code{\link[ggplot2:vars]{vars()}} and defining faceting groups on the rows or columns
dimension. The variables can be named (the names are passed to
\code{labeller}).

For compatibility with the classic interface, \code{rows} can also be
a formula with the rows (of the tabular display) on the LHS and
the columns (of the tabular display) on the RHS; the dot in the
formula is used to indicate there should be no faceting on this
dimension (either row or column).}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}

\item{show_plot}{Show the plot.}

\item{save_path}{File path to save precision-recall results to.}

\item{verbose}{Print messages.}
}
\value{
list with data and precision recall and F1 plots
}
\description{
Plot precision-recall curves (and optionally F1 plots) by
iteratively testing for peak overlap across a series of
thresholds used to filter \code{peakfiles}.
Each \link[GenomicRanges]{GRanges}
 object in \code{peakfiles} will be used as the "query"
 against each \link[GenomicRanges]{GRanges} object in \code{reference}
 as the subject.
 Will automatically use any columns that are
 specified with \code{thresholding_cols} and present within each
 \link[GenomicRanges]{GRanges} object
 to create percentiles  for thresholding.
\emph{NOTE} : Assumes that all \link[GenomicRanges]{GRanges} in
\code{peakfiles} and \code{reference} are already
aligned to the same genome build.
}
\examples{
data("CnR_H3K27ac")
data("CnT_H3K27ac")
data("encode_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
reference <- list("encode_H3K27ac" = encode_H3K27ac)

pr_out <- plot_precision_recall(peakfiles = peakfiles,
                                reference = reference,
                                workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_peaks.R
\name{read_peaks}
\alias{read_peaks}
\title{Read peaks}
\usage{
read_peaks(path, type, verbose = TRUE)
}
\arguments{
\item{path}{Path to peak file.}

\item{type}{File type to search for. Options include:
\itemize{
\item{"<pattern>"}{Finds files matching an arbitrary regex pattern
specified by user.}
\item{"peaks.stringent"}{Finds files ending in "*.stringent.bed$"}
\item{"peaks.consensus"}{Finds files ending in "*.consensus.peaks.bed$"}
\item{"peaks.consensus.filtered"}{
Finds files ending in"*.consensus.peaks.filtered.awk.bed$"}
\item{"picard"}{Finds files ending in
"*.target.markdup.MarkDuplicates.metrics.txt$"}
}}

\item{verbose}{Print messages.}
}
\value{
\link[GenomicRanges]{GRanges}
}
\description{
Read peak files.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gather_files_names.R
\name{gather_files_names}
\alias{gather_files_names}
\title{Make file names}
\usage{
gather_files_names(paths, type, nfcore_cutandrun, verbose = TRUE)
}
\arguments{
\item{paths}{Character vector of file paths.}

\item{type}{File type to search for. Options include:
\itemize{
\item{"<pattern>"}{Finds files matching an arbitrary regex pattern
specified by user.}
\item{"peaks.stringent"}{Finds files ending in "*.stringent.bed$"}
\item{"peaks.consensus"}{Finds files ending in "*.consensus.peaks.bed$"}
\item{"peaks.consensus.filtered"}{
Finds files ending in"*.consensus.peaks.filtered.awk.bed$"}
\item{"picard"}{Finds files ending in
"*.target.markdup.MarkDuplicates.metrics.txt$"}
}}

\item{nfcore_cutandrun}{Whether the files were generated by the
\href{https://nf-co.re/cutandrun}{nf-core/cutandrun} Nextflow pipeline.
 If \code{TRUE}, can use the standardised folder structure to
 automatically generate more descriptive file names with sample IDs.}

\item{verbose}{Print messages.}
}
\value{
Named character vector.
}
\description{
Support function for \link[EpiCompare]{gather_files}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_precision_recall.R
\name{predict_precision_recall}
\alias{predict_precision_recall}
\title{Predict precision-recall}
\source{
\href{https://stackoverflow.com/a/27796497}{
Fix for producing NAs from loess fun.}
}
\usage{
predict_precision_recall(
  pr_df,
  fun = stats::loess,
  precision = seq(10, 100, 10),
  recall = seq(10, 100, 10)
)
}
\arguments{
\item{pr_df}{Precision-recall data.frame generated by 
\link[EpiCompare]{precision_recall}.}

\item{fun}{Function to fit the data with.}

\item{precision}{Precision values to predict recall from.}

\item{recall}{Recall values to predict precision from.}
}
\value{
A named list of fitted models and predictions.
}
\description{
Predict specific values of precision or recall by fitting a model to
a precision-recall curve. 
Predictions that are <0 will automatically be set to 0.
Predictions that are >100 will automatically be set to 100.
}
\examples{
data("CnR_H3K27ac")
data("CnT_H3K27ac")
data("encode_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac)
reference <- list("encode_H3K27ac" = encode_H3K27ac)
pr_df <- precision_recall(peakfiles = peakfiles,
                          reference = reference)
predictions <- predict_precision_recall(pr_df = pr_df)           
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_workers.R
\name{check_workers}
\alias{check_workers}
\title{Check workers}
\usage{
check_workers(workers = NULL)
}
\arguments{
\item{workers}{Number of cores to parallelise across 
(in applicable functions).
If \code{NULL}, will set to the total number of available cores minus 1.}
}
\value{
Integer
}
\description{
Assign parallel worker cores.
}
\examples{
workers <- check_workers()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CnR_H3K27ac}
\alias{CnR_H3K27ac}
\title{Example CUT&Run peak file}
\format{
An object of class \code{GRanges} of length 2707.
}
\source{
The code to prepare the .Rda file from the raw peak file is:

\code{# sequences were directly downloaded from
 https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8581604} \cr
\code{# and peaks (BED file) were generated by Leyla Abbasova 
(Neurogenomics Lab, Imperial College London)} \cr
\code{CnR_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
\code{CnR_H3K27ac <- CnR_H3K27ac[seqnames(CnR_H3K27ac)== "chr1"]} \cr
\code{my_label <- 
c("name","score","strand","signalValue","pValue","qValue","peak")}\cr
\code{colnames(GenomicRanges::mcols(CnR_H3K27ac)) <- my_label} \cr
\code{usethis::use_data(CnR_H3K27ac, overwrite = TRUE)} \cr
}
\usage{
data("CnR_H3K27ac")
}
\description{
Human H3K27ac peak file generated with CUT&Run using K562 cell-line from
Meers et al., (2019). Human genome build hg19 was used.
Raw peak file (.BED) was obtained from GEO
(\url{https://trace.ncbi.nlm.nih.gov/Traces/sra/?run=SRR8581604}).
Peak calling was performed by Leyla Abbasova using MACS2.
The peak file was then processed into GRanges object.
Peaks located on chromosome 1 were subsetted to reduce the dataset size.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_genome_builds.R
\name{prepare_genome_builds}
\alias{prepare_genome_builds}
\title{Prepare genome builds}
\usage{
prepare_genome_builds(genome_build, blacklist = NULL)
}
\arguments{
\item{genome_build}{A named list indicating the genome build used to
generate each of the following inputs:
\itemize{
\item{"peakfiles" : }{Genome build for the \code{peakfiles} input.
Assumes genome build is the same for each element in the \code{peakfiles}
list.}
\item{"reference" : }{Genome build for the \code{reference} input.}
\item{"blacklist" : }{Genome build for the \code{blacklist} input.}
}
Example input list:\cr
 \code{genome_build = list(peakfiles="hg38",
 reference="hg19", blacklist="hg19")}\cr\cr
Alternatively, you can supply a single character string instead of a list.
This should \emph{only} be done in situations where all three inputs
(\code{peakfiles}, \code{reference}, \code{blacklist}) are of the same
genome build. For example:\cr
\code{genome_build = "hg19"}\cr\cr
Supported genome builds are: "hg19", "hg38", "mm9" and "mm10".}

\item{blacklist}{A \link[GenomicRanges]{GRanges} object
 containing blacklisted genomic regions.
Blacklists included in \pkg{EpiCompare} are:
\itemize{
\item{\code{NULL} (default): }{Automatically selects the appropriate
 blacklist based on the \code{genome_build_output} argument.}
\item{"hg19_blacklist": }{Regions of hg19 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{hg19_blacklist}}
\item{"hg38_blacklist": }{Regions of hg38 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{hg38_blacklist}}
\item{"mm10_blacklist": }{Regions of mm10 genome that have anomalous
and/or unstructured signals. \link[EpiCompare]{mm10_blacklist}}
\item{"mm9_blacklist": }{Blacklisted regions of mm10 genome that have been
 lifted over from \link[EpiCompare]{mm10_blacklist}.
 \link[EpiCompare]{mm9_blacklist}}
\item{\code{<user_input>}: }{A custom user-provided blacklist in
\link[GenomicRanges]{GRanges} format.}
}}
}
\value{
Named list.
}
\description{
Parse the \code{genome_build} argument into
\code{peaklist_build} and \code{reference_build}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_command.R
\name{report_command}
\alias{report_command}
\title{Report command}
\usage{
report_command(params, peaklist_tidy, reference_tidy)
}
\arguments{
\item{params}{Parameters supplied to the Rmarkdown template.}

\item{peaklist_tidy}{Post-processed target peaks.}

\item{reference_tidy}{Post-processed reference peaks.}
}
\value{
String reconstructing R function call.
}
\description{
Reconstruct the \link[EpiCompare]{EpiCompare} command 
used to generate the current Rmarkdown report.
}
\examples{
# report_command()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tss_plot.R
\name{tss_plot}
\alias{tss_plot}
\title{Read count frequency around TSS}
\usage{
tss_plot(
  peaklist,
  txdb = NULL,
  tss_distance = c(-3000, 3000),
  conf = 0.95,
  resample = 500,
  interact = FALSE,
  workers = check_workers()
)
}
\arguments{
\item{peaklist}{A list of peak files as GRanges object.
Files must be listed and named using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}
If not named, default file names will be assigned.}

\item{txdb}{A TxDb annotation object from Bioconductor.}

\item{tss_distance}{A vector specifying the distance upstream and downstream
around transcription start sites (TSS). 
The default value is \code{c(-3000,3000)}; meaning peak frequency 
3000bp upstream and downstream of TSS will be displayed.}

\item{conf}{Confidence interval threshold estimated by bootstrapping
 (\code{0.95} means 95%).
Argument passed to \link[ChIPseeker]{plotAvgProf}.}

\item{resample}{Number of bootstrapped iterations to run.
Argument passed to \link[ChIPseeker]{plotAvgProf}.}

\item{interact}{Default TRUE. By default, plots are interactive.
If set FALSE, all plots in the report will be static.}

\item{workers}{Number of cores to parallelise bootstrapping across.
Argument passed to \link[ChIPseeker]{plotAvgProf}.}
}
\value{
A named list of profile plots.
}
\description{
This function generates a plot of read count frequency around TSS.
}
\examples{
### Load Data ###
data("CnT_H3K27ac") # example peaklist GRanges object
data("CnR_H3K27ac") # example peaklist GRanges object
### Create Named Peaklist ###
peaklist <- list("CnT"=CnT_H3K27ac, "CnR"=CnR_H3K27ac) 
my_plot <- tss_plot(peaklist = peaklist, 
                    tss_distance=c(-50,50),
                    workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_button.R
\name{download_button}
\alias{download_button}
\title{Download local file}
\source{
\href{https://github.com/fmmattioni/downloadthis/issues/31}{
csv2 Issue.}

\href{https://github.com/fmmattioni/downloadthis/issues/16}{
Plotly Issue}
}
\usage{
download_button(
  object,
  save_output = FALSE,
  outfile_dir = NULL,
  filename = NULL,
  button_label = paste0("Download: ", "<code>", filename, "</code>"),
  output_extension = ".rds",
  icon = "fa fa-save",
  button_type = "success",
  self_contained = TRUE,
  add_download_button = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{object}{\R object to serialize.}

\item{save_output}{Default FALSE. If TRUE, all outputs (tables and plots) of
the analysis will be saved in a folder (EpiCompare_file).}

\item{outfile_dir}{Directory to save the file to.}

\item{filename}{Name of the file to save.}

\item{button_label}{Character (HTML), button label}

\item{output_extension}{Extension of the output file. Currently,  \code{.csv},
\code{.xlsx}, and \code{.rds} are supported. If a (named) list is passed to the
function, only \code{.xlsx} and \code{.rds} are supported.}

\item{icon}{Fontawesome tag e.g.: "fa fa-save"}

\item{button_type}{Character, one of the standard Bootstrap types}

\item{self_contained}{A boolean to specify whether your HTML output is
self-contained. Default to \code{FALSE}.}

\item{add_download_button}{Add download buttons for each plot or dataset.}

\item{verbose}{Print messages.}
}
\value{
Download button as HTML text.
}
\description{
Save an object as RDS and create a download button that can be rendered to
 Rmarkdown HTML pages.
Uses the package \pkg{downloadthis}.
}
\examples{
button <- download_button(object=mtcars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rebin_peaks.R
\name{rebin_peaks}
\alias{rebin_peaks}
\title{Rebin peaks}
\usage{
rebin_peaks(
  peakfiles,
  genome_build,
  intensity_cols = c("total_signal", "qValue", "Peak Score", "score"),
  bin_size = 5000,
  keep_chr = NULL,
  sep = c(":", "-"),
  drop_empty_chr = FALSE,
  as_sparse = TRUE,
  workers = check_workers(),
  verbose = TRUE,
  ...
)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{genome_build}{The build of **all** peak and reference files to 
calculate the correlation matrix on. If all peak and reference files are not 
of the same build use 
\link[EpiCompare]{liftover_grlist} to convert them all before running. Genome
build should be one of hg19, hg38, mm9, mm10.}

\item{intensity_cols}{Depending on which columns are present, this
value will be used to get quantiles and ultimately calculate the 
correlations:
\itemize{
 \item{"total_signal" : }{Used by the peak calling software 
 \href{https://github.com/FredHutch/SEACR}{SEACR}. 
 \emph{NOTE}: Another SEACR column (e.g. "max_signal") can be used 
 together or instead of "total_signal".}
 \item{"qValue"}{Used by the peak calling software 
 \href{https://github.com/macs3-project/MACS}{MACS2/3}. 
 Should contain the negative log of the p-values after multiple 
 testing correction.}
 \item{"Peak Score" : }{
 Used by the peak calling software
  \href{http://homer.ucsd.edu/homer/ngs/peaks.html}{HOMER}.}
}}

\item{bin_size}{Default of 100. Base-pair size of the bins created to measure
correlation. Use smaller value for higher resolution but longer run time and 
larger memory usage.}

\item{keep_chr}{Which chromosomes to keep.}

\item{sep}{Separator to be used after chromosome name (first item) and 
between start/end genomic coordinates (second item).}

\item{drop_empty_chr}{Drop chromosomes that are not present in any of the 
\code{peakfiles} (default: \code{FALSE}).}

\item{as_sparse}{Return the rebinned peaks as a sparse matrix
(default: \code{TRUE}), 
which is more efficiently stored than a dense matrix (\code{FALSE}).}

\item{workers}{Number of threads to parallelize across.}

\item{verbose}{Print messages.}

\item{...}{
  Arguments passed on to \code{\link[=bpplapply]{bpplapply}}
  \describe{
    \item{\code{apply_fun}}{Iterator function to use.}
    \item{\code{register_now}}{Register the cores now with
\link[BiocParallel]{register} (\code{TRUE}),
 or simply return the \code{BPPARAM object} (default: \code{FALSE}).}
    \item{\code{use_snowparam}}{Whether to use
\link[BiocParallel]{SnowParam} (default: \code{TRUE}) or
\link[BiocParallel]{MulticoreParam} (\code{FALSE})
 when parallelising across multiple \code{workers}.}
    \item{\code{progressbar}}{
    \code{logical(1)} Enable progress bar (based on plyr:::progress_text).
  }
    \item{\code{X}}{
    Any object for which methods \code{length}, \code{[}, and
    \code{[[} are implemented.
  }
    \item{\code{FUN}}{
    The \code{function} to be applied to each element of \code{X}.
  }
  }}
}
\value{
Binned peaks matrix
}
\description{
Standardise a list of peak files by rebinning them into fixd-width
 tiles across the genome.
}
\examples{
data("CnR_H3K27ac") 
data("CnT_H3K27ac")
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, CnT_H3K27ac=CnT_H3K27ac) 

#increasing bin_size for speed
peakfiles_rebinned <- rebin_peaks(peakfiles = peakfiles,
                                  genome_build = "hg19",
                                  bin_size = 5000,
                                  workers = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/messager.R
\name{messager}
\alias{messager}
\title{Print messages}
\usage{
messager(..., v = TRUE, parallel = FALSE)
}
\arguments{
\item{v}{Whether to print messages or not.}

\item{parallel}{Whether to enable message print when wrapped 
in parallelised functions.}
}
\value{
Null
}
\description{
Conditionally print messages.
 Allows developers to easily control verbosity of functions, 
 and meet Bioconductor requirements that dictate the message 
 must first be stored to a variable before passing to \link[base]{message}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_output.R
\name{save_output}
\alias{save_output}
\title{Save output}
\usage{
save_output(
  save_output = FALSE,
  file,
  file_type,
  filename,
  outpath,
  interactive = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{save_output}{Default FALSE. If TRUE, outputs are saved.}

\item{file}{Tables and plots to be saved.}

\item{file_type}{Type of file to be saved.
"data.frame", "ggplot", "image"}

\item{filename}{Name of file.}

\item{outpath}{Outpath}

\item{interactive}{Default FALSE. If TRUE, interactive plots are saved as
html.}

\item{verbose}{Print messages.}
}
\value{
Saved data frames and plots.
}
\description{
This function saves data frames and plots generated by EpiCompare.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mm9_blacklist}
\alias{mm9_blacklist}
\title{Mouse genome mm9 blacklisted regions}
\format{
An object of class \code{GRanges} of length 292.
}
\source{
\code{
tmp <- base::get("mm10_blacklist", asNamespace("EpiCompare"))
mm9_blacklist <- liftover_grlist(grlist = tmp,
                                 input_build = "mm10",
                                 output_build = "mm9", 
                                 keep_chr = NULL)
usethis::use_data(mm9_blacklist, overwrite = TRUE)                                
}
}
\usage{
data("mm9_blacklist")
}
\description{
Blaklisted regions of the mm9 genome build 
btained by lifting over the \code{mm10_blacklist}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_blacklist.R
\name{prepare_blacklist}
\alias{prepare_blacklist}
\title{Prepare blacklist as GRanges}
\usage{
prepare_blacklist(
  blacklist,
  output_build,
  blacklist_build = NULL,
  verbose = TRUE
)
}
\arguments{
\item{output_build}{Desired genome build for
\code{grlist} to be lifted over to.}

\item{blacklist_build}{Genome build of the blacklist.
Only used when \code{blacklist} is a user-supplied 
\link[GenomicRanges]{GRanges} object.}

\item{verbose}{Print messages.}
}
\value{
A \link[GenomicRanges]{GRanges} objects of blacklisted 
genomic regions from the relevant genome build.
}
\description{
Selects the appropriate blacklist in a variety of conditions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peak_info.R
\name{peak_info}
\alias{peak_info}
\title{Summary of Peak Information}
\usage{
peak_info(peaklist, blacklist)
}
\arguments{
\item{peaklist}{A named list of peak files as GRanges object.
Objects listed using \code{list("name1" = peak, "name2" = peak2)}.}

\item{blacklist}{A GRanges object containing blacklisted regions.}
}
\value{
A summary table of peak information
}
\description{
This function outputs a table summarizing information on the peak files.
Provides the total number of peaks and the percentage of peaks in blacklisted
 regions.
}
\examples{
### Load Data ###
data("encode_H3K27ac") # example peakfile GRanges object
data("CnT_H3K27ac") # example peakfile GRanges object
data("hg19_blacklist") # example blacklist GRanges object

### Named Peaklist ###
peaklist <- list("encode"=encode_H3K27ac, "CnT"=CnT_H3K27ac)

### Run ###
df <- peak_info(peaklist = peaklist,
                blacklist = hg19_blacklist) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stopper.R
\name{stopper}
\alias{stopper}
\title{Stop messages}
\usage{
stopper(..., v = TRUE)
}
\arguments{
\item{v}{Whether to print messages or not.}
}
\value{
Null
}
\description{
Conditionally print stop messages.
Allows developers to easily control verbosity of functions, 
 and meet Bioconductor requirements that dictate the stop message 
 must first be stored to a variable before passing to \link[base]{stop}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkCache.R
\name{checkCache}
\alias{checkCache}
\title{Check cache}
\usage{
checkCache(cache = BiocFileCache::BiocFileCache(ask = FALSE), url)
}
\arguments{
\item{cache}{BiocFileCache.}

\item{url}{Path to cached file.}
}
\value{
path
}
\description{
Quick function to check if object is already saved.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group_files.R
\name{group_files}
\alias{group_files}
\title{Group files}
\usage{
group_files(peakfiles, searches)
}
\arguments{
\item{peakfiles}{A list of peak files as GRanges object and/or as paths to
BED files. If paths are provided, EpiCompare imports the file as GRanges
object. EpiCompare also accepts a list containing a mix of GRanges objects
and paths.Files must be listed and named using \code{list()}.
E.g. \code{list("name1"=file1, "name2"=file2)}. If no names are specified,
default file names will be assigned.}

\item{searches}{A named list of substrings to group \code{peakfiles} by.}
}
\value{
Named peak files
}
\description{
Assign group names to each file in a named list based on a series of string
searches based on combinations of relevant metadata factors.
}
\examples{
data("encode_H3K27ac") # example dataset as GRanges object
data("CnT_H3K27ac") # example dataset as GRanges object
data("CnR_H3K27ac") # example dataset as GRanges object
peakfiles <- list(CnR_H3K27ac=CnR_H3K27ac, 
                  CnT_H3K27ac=CnT_H3K27ac, 
                  encode_H3K27ac=encode_H3K27ac)

peaks_grouped <- group_files(peakfiles = peakfiles,
                             searches=list(assay=c("H3K27ac"),
                                           source=c("Cn","ENCODE")))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{encode_H3K27ac}
\alias{encode_H3K27ac}
\title{Example ChIP-seq peak file}
\format{
An object of class \code{GRanges} of length 5142.
}
\source{
The code to prepare the .Rda file from the raw peak file is:

\code{# dataset was directly downloaded from} \cr
\code{# https://www.encodeproject.org/files/ENCFF044JNJ/}
\code{encode_H3K27ac <- ChIPseeker::readPeakFile("path", as = "GRanges")} \cr
\code{encode_H3K27ac <- 
encode_H3K27ac[seqnames(encode_H3K27ac) == "chr1"]} \cr
\code{my_label <-
 c("name","score","strand","signalValue","pValue","qValue","peak")} \cr
\code{colnames(GenomicRanges::mcols(encode_H3K27ac)) <- my_label} \cr
\code{usethis::use_data(encode_H3K27ac, overwrite = TRUE)} \cr
}
\usage{
data("encode_H3K27ac")
}
\description{
Human H3K27ac peak file generated with ChIP-seq using K562 cell-line.
Human genome build hg19 was used.
The peak file (.BED) was obtained from ENCODE project
(\url{https://www.encodeproject.org/files/ENCFF044JNJ/}).
The BED file was then imported as an GRanges object.
Peaks located on chromosome 1 were subsetted to reduce the dataset size.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_reference.R
\name{prepare_reference}
\alias{prepare_reference}
\title{Prepare referemce as GRanges}
\usage{
prepare_reference(
  reference,
  max_elements = NULL,
  remove_empty = TRUE,
  as_list = TRUE,
  as_grangeslist = FALSE
)
}
\arguments{
\item{reference}{A named list of \link[GenomicRanges]{GRanges} objects, 
or a single \link[GenomicRanges]{GRanges} object to be converted into a 
named list.}

\item{max_elements}{Max number of elements to use within the list.
Set to \code{NULL} (default) to use all elements.}

\item{remove_empty}{Remove any empty elements in the list.}

\item{as_list}{Return as a list.}

\item{as_grangeslist}{Return as a \link[GenomicRanges]{GRangesList}
(overrides \code{as_list}).}
}
\value{
A list of \link[GenomicRanges]{GRanges} objects
}
\description{
Prepare referemce as GRanges
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fragment_info.R
\name{fragment_info}
\alias{fragment_info}
\title{Summary on fragments}
\usage{
fragment_info(picard_list)
}
\arguments{
\item{picard_list}{Named list of duplication metrics generated by Picard
as data frame. Data frames must be named and listed using \code{list()}.
e.g. \code{list("name1"=file1, "name2"=file2)}.
To import Picard duplication metrics (.txt file) into R as data frame, use
\code{picard <- read.table("/path/to/picard/output", header = TRUE,
fill = TRUE)}.}
}
\value{
A table summarizing metrics on fragments.
}
\description{
This function outputs a summary on fragments using metrics generated by
Picard. Provides the number of mapped fragments, duplication rate and number
of unique fragments.
}
\examples{
### Load Data ###
data(CnT_H3K27ac_picard) # example picard output
data(CnR_H3K27ac_picard) # example picard output 
### Import Picard Metrics ###
# To import Picard duplication metrics (.txt file) into R as data frame
# CnT_H3K27ac_picard <- read.table("/path/to/picard/output.txt",
#  header = TRUE,fill = TRUE) 
### Create Named List ###
picard_list <- list("CnT_H3K27ac"=CnT_H3K27ac_picard,
               "CnR_H3K27ac"=CnR_H3K27ac_picard) 
df <- fragment_info(picard_list = picard_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_grlist_cols.R
\name{check_grlist_cols}
\alias{check_grlist_cols}
\title{Check \link[GenomicRanges]{GRanges} list columns}
\usage{
check_grlist_cols(grlist, target_cols)
}
\arguments{
\item{grlist}{Named list of \link[GenomicRanges]{GRanges} objects.}

\item{target_cols}{A character vector of column names to search for.}
}
\value{
Named list of \link[GenomicRanges]{GRanges} objects.
}
\description{
Check that at least one of the required columns is in 
a list of \link[GenomicRanges]{GRanges} objects.
Elements that do not meet this criterion will be dropped from the list.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_consensus_peaks.R
\name{compute_consensus_peaks}
\alias{compute_consensus_peaks}
\title{Compute consensus peaks}
\source{
\href{https://ro-che.info/articles/2018-07-11-chip-seq-consensus}{
GenomicRanges tutorial}

\href{https://doi.org/doi:10.18129/B9.bioc.consensusSeekeR}{
consensusSeekeR}
}
\usage{
compute_consensus_peaks(
  grlist,
  groups = NULL,
  genome_build,
  lower = 2,
  upper = Inf,
  min.gapwidth = 1L,
  method = c("granges", "consensusseeker"),
  ...
)
}
\arguments{
\item{grlist}{Named list of \link[GenomicRanges]{GRanges} objects.}

\item{groups}{A character vector of the same length as \code{grlist} 
defining how to group \link[GenomicRanges]{GRanges} objects when 
computing consensus peaks.}

\item{genome_build}{Genome build name.}

\item{lower, upper}{
    The lower and upper bounds for the slice.
  }

\item{min.gapwidth}{
    Ranges separated by a gap of at least \code{min.gapwidth} positions
    are not merged.
  }

\item{method}{Method to call peaks with:
\itemize{
\item{"granges" : }{Simple overlap procedure using 
\link[GenomicRanges]{GRanges} functions.
Faster but less accurate.}
\item{"consensusseeker" : }{
Uses \link[consensusSeekeR]{findConsensusPeakRegions} to compute consensus
peaks. 
Slower but more accurate.}
}}

\item{...}{
  Arguments passed on to \code{\link[consensusSeekeR:findConsensusPeakRegions]{consensusSeekeR::findConsensusPeakRegions}}
  \describe{
    \item{\code{narrowPeaks}}{a \code{GRanges} containing
called peak regions of signal enrichment based on pooled, normalized data
for all analyzed experiments. All \code{GRanges} entries must
have a metadata field called "name" which identifies the region to
the called peak. All \code{GRanges} entries must also
have a row name which identifies the experiment of origin. Each
\code{peaks} entry must have an associated \code{narrowPeaks} entry.
A \code{GRanges} entry is associated to a \code{narrowPeaks} entry by
having a identical metadata "name" field and a identical row name.}
    \item{\code{peaks}}{a \code{GRanges} containing called peaks of signal
enrichment based on pooled, normalized data
for all analyzed experiments. All \code{GRanges} entries must
have a metadata field called "name" which identifies the called
peak. All \code{GRanges} entries must
have a row name which identifies the experiment of origin. Each
\code{peaks} entry must have an associated \code{narrowPeaks} entry. A
\code{GRanges} entry is associated to a \code{narrowPeaks} entry by having
a identical metadata "name" field and a identical row name.}
    \item{\code{chrInfo}}{a \code{Seqinfo} containing the name and the length of the
chromosomes to analyze. Only the chomosomes contained in this
\code{Seqinfo} will be analyzed.}
    \item{\code{extendingSize}}{a \code{numeric} value indicating the size of padding
on both sides of the position of the peaks median to create the
consensus region. The minimum size of the consensus region is
equal to twice the value of the \code{extendingSize} parameter.
The size of the \code{extendingSize} must be a positive integer.
Default = 250.}
    \item{\code{expandToFitPeakRegion}}{a \code{logical} indicating if the region size,
which is set by the \code{extendingSize} parameter is extended to include
the entire narrow peak regions of all peaks included in the unextended
consensus region. The narrow peak regions of the peaks added because of the
extension are not considered for the extension. Default: \code{FALSE}.}
    \item{\code{shrinkToFitPeakRegion}}{a \code{logical} indicating if the region size,
which is set by the \code{extendingSize} parameter is shrinked to
fit the narrow peak regions of the peaks when all those regions
are smaller than the consensus region. Default: \code{FALSE}.}
    \item{\code{minNbrExp}}{a positive \code{numeric} or a positive \code{integer}
indicating the minimum number of experiments in which at least one peak
must be present for a potential consensus region. The numeric must be a
positive integer inferior or equal to the number of experiments present
in the \code{narrowPeaks} and \code{peaks} parameters. Default = 1.}
    \item{\code{nbrThreads}}{a \code{numeric} or a \code{integer} indicating the
number of threads to use in parallel. The \code{nbrThreads} must be a
positive integer. Default = 1.}
  }}
}
\value{
Named list of consensus peak \link[GenomicRanges]{GRanges}.
}
\description{
Compute consensus peaks from a list of \link[GenomicRanges]{GRanges}.
}
\details{
\emph{NOTE:} If you get the error 
\code{"Error in serialize(data, node$con) : error writing to connection"},
try running \link[base]{closeAllConnections} 
and rerun \link[EpiCompare]{compute_consensus_peaks}. 
This error can sometimes occur when 
\link[EpiCompare]{compute_consensus_peaks}
has been disrupted partway through.
}
\examples{
data("encode_H3K27ac") # example dataset as GRanges object
data("CnT_H3K27ac") # example dataset as GRanges object
data("CnR_H3K27ac") # example dataset as GRanges object 
grlist <- list(CnR=CnR_H3K27ac, CnT=CnT_H3K27ac, ENCODE=encode_H3K27ac)

consensus_peaks <- compute_consensus_peaks(grlist = grlist,
                                           groups = c("Imperial",
                                                      "Imperial",
                                                      "ENCODE"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hg38_blacklist}
\alias{hg38_blacklist}
\title{Human genome hg38 blacklisted regions}
\format{
An object of class \code{GRanges} of length 910.
}
\source{
The code to prepare the .Rda file is:

\code{## blacklisted regions were directly downloaded} \cr
\code{## from https://www.encodeproject.org/files/ENCFF356LFX/} \cr
\code{hg38_blacklist <- 
ChIPseeker::readPeakFile(file.path(path), as = "GRanges")} \cr
\code{usethis::use_data(hg38_blacklist, overwrite = TRUE)} \cr
}
\usage{
data("hg38_blacklist")
}
\description{
Obtained from \url{https://www.encodeproject.org/files/ENCFF356LFX/}.
The ENCODE blacklist includes regions of the hg38 genome that have 
anomalous and/or unstructured signals 
independent of the cell-line or experiment. Removal
of ENCODE blacklist is recommended for quality measure.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_chromHMM_annotation.R
\name{get_chromHMM_annotation}
\alias{get_chromHMM_annotation}
\title{Download ChromHMM annotation file(s)}
\usage{
get_chromHMM_annotation(
  cell_line,
  cache = BiocFileCache::BiocFileCache(ask = FALSE)
)
}
\arguments{
\item{cell_line}{ChromHMM annotation for user-specified cell-line.
Cell-line options are:
\itemize{
  \item "K562" = K-562 cells
  \item "Gm12878" = Cellosaurus cell-line GM12878
  \item "H1hesc" = H1 Human Embryonic Stem Cell
  \item "Hepg2" = Hep G2 cell
  \item "Hmec" = Human Mammary Epithelial Cell
  \item "Hsmm" = Human Skeletal Muscle Myoblasts
  \item "Huvec" = Human Umbilical Vein Endothelial Cells
  \item "Nhek" = Normal Human Epidermal Keratinocytes
  \item "Nhlf" = Normal Human Lung Fibroblasts
}}
}
\value{
Cell-line specific ChromHMM annotation file. Default K562 cell-line.
}
\description{
Download ChromHMM annotation file(s) for a given cell-line
 (returned as a \link[GenomicRanges]{GRanges} object) 
 or a list of cell-lines
 (returned as a named list of \link[GenomicRanges]{GRanges} objects).
 All annotations are aligned to the hg19 genome build.
 All data can be found on the UCSC Genome Browser 
 \href{https://hgdownload.cse.ucsc.edu/goldenPath/hg19/encodeDCC/wgEncodeBroadHmm/}{
 here}.
}
\keyword{internal}
